local table  = table
local string = string

local effect_category = {
    [ Turbine.Gameplay.EffectCategory.Cry ]         = "Cry",
    [ Turbine.Gameplay.EffectCategory.Fear ]        = "Fear",
    [ Turbine.Gameplay.EffectCategory.Song ]        = "Song",
    [ Turbine.Gameplay.EffectCategory.Wound ]       = "Wound",
    [ Turbine.Gameplay.EffectCategory.Poison ]      = "Poison",
    [ Turbine.Gameplay.EffectCategory.Disease ]     = "Disease",
    [ Turbine.Gameplay.EffectCategory.Physical ]    = "Physical",
    [ Turbine.Gameplay.EffectCategory.Elemental ]   = "Elemental",
    [ Turbine.Gameplay.EffectCategory.Undefined ]   = "Undefined",
    [ Turbine.Gameplay.EffectCategory.Corruption ]  = "Corruption",
    [ Turbine.Gameplay.EffectCategory.Dispellable ] = "Dispellable",
    
    [ 16384 ] = "Unknown"
}

Group.Library = table.load( Turbine.PluginData.Load( Turbine.DataScope.Account, "MiniRaid_Effect_Library" ) )

if not Group.Library then
    Group.Library = {}

    Turbine.PluginData.Save( Turbine.DataScope.Account, "MiniRaid_Effect_Library", table.save( Group.Library ) )
end

Group.Effect = table.load( Turbine.PluginData.Load( Turbine.DataScope.Character, "MiniRaid_Effects" ) )

if not Group.Effect then
    Group.Effect = {
        enable         = true,
        enable_alert   = true,
        enable_special = true,

        Track          = {},
        Alert          = {},
        Special        = {
            [ 1 ] = {
                name = "Do Not Fall This Day",
                icon = 1091454172.000000
            }
        },
        Blacklist      = {}
    }

    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
end

local language = Group:GetLanguage()

Group.Alert = UI:Create( "BASE", nil, function( main )
    main:SetSize( UI.SCREEN_W, UI.SCREEN_H )
    main:SetColor( 0, 0, 0, 0 )
    main:SetMouseVisible( false )
    main:SetZOrder( 60 )

    main.list = UI:Create( "LIST", main, function( ui )
        ui:SetSize( UI.SCREEN_W, UI.SCREEN_H )
        ui:SetPos( 0, UI.SCREEN_H / 2 - 32 )
        ui:SetMouseVisible( false )
        ui:SetWantsUpdates( true )

        ui.list   = {}
        ui.active = {}

        function ui:Update()
            local time = Turbine.Engine.GetGameTime()

            local group = Group

            for i = 1, #self.list do
                local item = self.list[ i ]

                if item.active then
                    if time > item.time + 5 then
                        if item.check and group.PartyName[ item.player ] then
                            local list  = group.Party[ group.PartyName[ item.player ] ].player:GetEffects()
                            local count = list:GetCount()

                            local clear = true

                            for i = 1, count do
                                if item.name == list:Get( i ):GetName() and item.icon == list:Get( i ):GetIcon() then
                                    clear = false
                                end
                            end

                            if clear then
                                self.active[ item.name .. item.player ][ item.icon ] = false

                                item.check  = false
                                item.active = false
                            else
                                item.time = time - 3 + 13
                            end
                        else
                            item.check = true

                            self:RemoveItem( item )

                            item.time = time - 3 + 13
                        end
                    end
                end
            end
        end
    end )

    function main:Add( name, icon, player, message )
        if self.list.active[ name .. player ] then
            if self.list.active[ name .. player ][ icon ] then 
                return 
            else
                self.list.active[ name .. player ][ icon ] = true
            end
        else
            self.list.active[ name .. player ] = {
                [ icon ] = true
            }
        end

        local time = Turbine.Engine.GetGameTime()

        for i = 1, #self.list.list do
            local item = self.list.list[ i ]

            if not item.active then
                item.time   = time
                item.active = true

                item:SetMessage( name, icon, player, message )

                self.list:AddItem( item )

                return
            end
        end

        local item

        self.list:AddItem(
            UI:Create( "LIST", self.list, function( ui )
                ui:SetSize( UI.SCREEN_W, 32 + 16 )
                ui:SetMouseVisible( false )
                ui:SetColor( 0, 100, 0, 0 )

                ui.time   = time
                ui.active = true

                item = ui

                ui.box = UI:Create( "BOX", ui, function( ui )
                    ui:SetPos( UI.SCREEN_W / 2, 0 )
                    ui:SetSize( UI.SCREEN_W, 32 + 16 )
                    ui:SetColor( 0, 0, 100, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                    ui:SetMouseVisible( false )

                    ui.icon = UI:Create( "BOX", ui, function( ui )
                        ui:SetPos( 0, 8 )
                        ui:SetSize( 32, 32 )
                        ui:SetColor( 0, 0, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                        ui:SetMouseVisible( false )
                    end )

                    ui.text1 = UI:Create( "TEXT", ui, function( ui )
                        ui:SetPos( 32 + UI.Spacing, 0 )
                        ui:SetSize( UI.SCREEN_W, 32 + 16 )
                        ui:SetTextAlignment( 4 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanProBold24 )

                        if language == Turbine.Language.German then
                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana23 )
                        end

                        ui:SetMouseVisible( false )
                    end )

                    ui.text2 = UI:Create( "TEXT", ui, function( ui )
                        ui:SetPos( 32 + UI.Spacing, 0 )
                        ui:SetSize( UI.SCREEN_W, 32 + 16 )
                        ui:SetTextAlignment( 4 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanProBold24 )

                        if language == Turbine.Language.German then
                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana23 )
                        end

                        ui:SetMouseVisible( false )
                    end )

                    ui.text3 = UI:Create( "TEXT", ui, function( ui )
                        ui:SetPos( 32 + UI.Spacing, 0 )
                        ui:SetSize( UI.SCREEN_W, 32 + 16 )
                        ui:SetTextAlignment( 4 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanProBold24 )

                        if language == Turbine.Language.German then
                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana23 )
                        end

                        ui:SetMouseVisible( false )
                    end )
                end )

                function ui:SetMessage( name, icon, player, message )
                    self.name   = name
                    self.icon   = icon
                    self.player = player

                    local str = "<player>"

                    if language == Turbine.Language.German then
                        str = "<spieler>"
                    end

                    if message ~= "" then
                        if string.find( message, str ) then
                            local tbl = string.explode( str, message )

                            if tbl[ 1 ] == "" then
                                self.box.text1:SetText( "" )
                                self.box.text1:SetWidth( 0 )
                            else
                                self.box.text1:SetText( tbl[ 1 ] )
                                self.box.text1:SizeToContentsX()
                                self.box.text1:SetForeColor( Color( 255, 255, 255 ) )
                            end

                            self.box.text2:SetText( player )
                            self.box.text2:SizeToContentsX()
                            self.box.text2:SetForeColor( Color( 255, 128, 0 ) )--165

                            if tbl[ 2 ] == "" then
                                self.box.text3:SetText( "" )
                                self.box.text3:SetWidth( 0 )
                            else
                                self.box.text3:SetText( tbl[ 2 ] )
                                self.box.text3:SizeToContentsX()
                                self.box.text3:SetForeColor( Color( 255, 255, 255 ) )
                            end

                            local w1 = self.box.text1:GetWidth()
                            local w2 = self.box.text2:GetWidth()
                            local w3 = self.box.text3:GetWidth()

                            self.box.text2:SetPos( 32 + UI.Spacing + w1 )
                            self.box.text3:SetPos( 32 + UI.Spacing + w1 + w2 )

                            self.box:SetPos( UI.SCREEN_W / 2 - ( 32 + UI.Spacing + w1 + w2 + w3 ) / 2, 0 )
                        else
                            self.box.text1:SetText( message )
                            self.box.text1:SizeToContentsX()
                            self.box.text1:SetForeColor( Color( 255, 255, 255 ) )

                            self.box.text2:SetText( "" )
                            self.box.text3:SetText( "" )

                            local w1 = self.box.text1:GetWidth()

                            self.box:SetPos( UI.SCREEN_W / 2 - ( 32 + UI.Spacing + w1 ) / 2, 0 )
                        end
                    else
                        self.box.text1:SetText( name )
                        self.box.text1:SizeToContentsX()
                        self.box.text1:SetForeColor( Color( 0, 200, 200 ) )

                        self.box.text2:SetText( " On " )
                        self.box.text2:SizeToContentsX()
                        self.box.text2:SetForeColor( Color( 255, 255, 255 ) )

                        self.box.text3:SetText( player )
                        self.box.text3:SizeToContentsX()
                        self.box.text3:SetForeColor( Color( 255, 128, 0 ) )--165

                        local w1 = self.box.text1:GetWidth()
                        local w2 = self.box.text2:GetWidth()
                        local w3 = self.box.text3:GetWidth()

                        self.box.text2:SetPos( 32 + UI.Spacing + w1 )
                        self.box.text3:SetPos( 32 + UI.Spacing + w1 + w2 )

                        self.box:SetPos( UI.SCREEN_W / 2 - ( 32 + UI.Spacing + w1 + w2 + w3 ) / 2, 0 )
                    end

                    self.box.icon:SetBackground( icon )
                end

                ui:SetMessage( name, icon, player, message )
            end )
        )

        self.list.list[ #self.list.list + 1 ] = item
    end
end )

UI:CreateTemplate( "BOX", "EFFECTITEM", function( template )
    function template:Init( main )
        main:SetHeight( 32 + UI.Spacing * 2 )
        main:SetColor( 255, 0, 0, 0 )
        main:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

        main.hover = UI:Create( "BOX", main, function( ui )
            ui:SetPos( 1, 0 )
            ui:SetHeight( 32 + UI.Spacing * 2 )
            ui:SetColor( 0, 0, 0, 0 )
            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
            ui:SetMouseVisible( false )
        end )

        main.icon = UI:Create( "BOX", main, function( ui )
            ui:SetSize( 32, 32 )
            ui:SetPos( 1 + UI.Spacing, UI.Spacing )
            ui:SetMouseVisible( false )
            ui:SetColor( 0, 0, 0, 0 )
            ui:SetBackground( icon )
            ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
        end )

        main.text = UI:Create( "TEXT", main, function( ui )
            ui:SetPos( 1 + UI.Spacing * 2 + 32, 0 )
            ui:SetHeight( 32 + UI.Spacing * 2 )
            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

            if language == Turbine.Language.German then
                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
            end
            
            ui:SetTextAlignment( 4 )
            ui:SetText( "Effect Name" )
            ui:SetMouseVisible( false )
        end )

        function main:SizeChanged()
            local w = self:GetWidth()

            self.text:SetWidth( w - 1 - UI.Spacing * 2 - 32 - 1 )

            self.hover:SetWidth( w - 2 )
        end

        function main:MouseEnter()
            if not self:GetParent() or self:GetParent().selected == self then return end

            self.hover:SetColor( 150, 150, 150, 50 )
        end

        function main:MouseLeave()
            if not self:GetParent() or self:GetParent().selected == self then return end

            self.hover:SetColor( 0, 0, 0, 0 )
        end

        function main:MouseUp()
            if not self:GetParent() or self:GetParent().selected == self then return end

            self.hover:SetColor( 150, 150, 150, 50 )

            if self:GetParent().selected then
                self:GetParent().selected.hover:SetColor( 0, 0, 0, 0 )
                self:GetParent().selected.text:SetForeColor( Turbine.UI.Color( 1, 1, 1 ) )
            end

            self:GetParent().selected = self

            self:Selected()
        end

        function main:SetSelected( callback )
            if not self:GetParent() or self:GetParent().selected then return end

            self.hover:SetColor( 150, 150, 150, 50 )

            self:GetParent().selected = self

            self:Selected()

            if callback then callback() end
        end

        function main:ClearSelected( callback )
            if not self:GetParent().selected then return end

            self:GetParent().selected = nil

            self.hover:SetColor( 0, 0, 0, 0 )

            if callback then callback() end
        end

        function main:Selected()
            --Override
        end

        function main:SetIcon( icon )
            self.icon:SetBackground( icon )

            self.effect_icon = icon
        end

        function main:SetName( name )
            self.text:SetText( name )

            self.effect_name = name
        end

        function main:GetName()
            return self.effect_name
        end

        function main:GetIcon()
            return self.effect_icon
        end
    end
end )

UI:CreateTemplate( "LIST", "EFFECTINFO", function( template )
    function template:Init( main )
        main:AddItem(
            UI:Create( "HEADER", main, function( ui )
                ui:SetText( Group:Local( "Effects_Info_Header" )  )

                main.header = ui
            end )
        )

        main:SetHeight( 16 * 7 + 36 + 20 + UI.Spacing * 4 + 16 * 4 )

        function main:AddFiller( x, y )
            self:AddItem(
                UI:Create( "BOX", self, function( ui )
                    ui:SetSize( x, y )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                end )
            )
        end

        main:AddFiller( 0, 16 )

        --Icon
        main:AddItem(
            UI:Create( "BOX", main, function( ui )
                ui:SetHeight( 36 )
                ui:SetColor( 0, 0, 0, 0 )
                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                main.icon_box = ui

                UI:Create( "BORDER", ui, function( ui )
                    ui:SetSize( 36, 36 )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 200, 200, 200 ) )

                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 32, 32 )
                        ui:SetPos( 2, 2 )
                        ui:SetColor( 0, 0, 0, 0 )
                        ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )

                        main.icon = ui
                    end )
                end )

                ui.name = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 36 )
                    ui:SetPos( 36 + UI.Spacing )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetTextAlignment( 4 )
                    ui:SetText( "Effect Name" )
    
                    main.name = ui
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.name:SetWidth( w - 36 - UI.Spacing )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        --Debuff
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetTextAlignment( 4 )
                ui:SetText( "Debuff: N/A" )

                main.debuff = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Curable
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetTextAlignment( 4 )
                ui:SetText( "Curable: N/A" )

                main.curable = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Category
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetTextAlignment( 4 )
                ui:SetText( "Category: N/A" )

                main.category = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Duration
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetTextAlignment( 4 )
                ui:SetText( "Duration: N/A" )

                main.duration = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Description
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 * 5 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetTextAlignment( 1 )
                ui:SetText( "Description: N/A" )

                main.description = ui
            end )
        )

        function main:SizeChanged()
            local w = self:GetWidth()

            self.header:SetWidth( w )
            self.header:SizeChanged()

            self.icon_box:SetWidth( w )
            self.debuff:SetWidth( w )
            self.curable:SetWidth( w )
            self.category:SetWidth( w )
            self.duration:SetWidth( w )
            self.description:SetWidth( w )
        end

        function main:SetName( str )
            self.name:SetText( str )
        end

        function main:SetIcon( icon )
            self.icon:SetBackground( icon )
        end

        function main:SetDebuff( bool )
            self.debuff:SetText( Group:Local( "Effects_Info_Debuff" ) .. tostring( bool )  )
        end

        function main:SetCurable( bool  )
            self.curable:SetText( Group:Local( "Effects_Info_Curable" ) .. tostring( bool )  )
        end

        function main:SetCategory( category )
            self.category:SetText( Group:Local( "Effects_Info_Category" ) .. effect_category[ category ]  )
        end

        function main:SetDuration( duration )
            if duration > 86400 then 
                duration = "Static" 
            else
                duration = duration .. " Seconds"
            end

            self.duration:SetText( Group:Local( "Effects_Info_Duration" ) .. duration  )
        end

        function main:SetDescription( str )
            if str == "" then str = "N/A" end

            self.description:SetText( Group:Local( "Effects_Info_Description" ) .. str  )
        end
    end
end )

UI:CreateTemplate( "LIST", "EFFECTLIST", function( template )
    function template:Init( main )
        main:SetSize( 200, 200 )

        main.items       = 0
        main.effect_item = {}

        function main:AddFiller( x, y )
            self:AddItem(
                UI:Create( "BOX", self, function( ui )
                    ui:SetSize( x, y )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                end )
            )
        end

        function main:HasItems()
            --Override
        end

        function main:NoItems()
            --Override
        end

        main:AddItem(
            UI:Create( "LIST", main, function( ui )
                ui:SetHeight( 24 )
                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                main.filter = ui

                ui:AddItem(
                    UI:Create( "TEXT", ui, function( ui )
                        ui:SetSize( 48, 24 )
                        ui:SetTextAlignment( 4 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                        if language == Turbine.Language.German then
                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                        end
                        
                        ui:SetText( Group:Local( "Effects_List_Filter" )  )
                    end )
                )
                
                ui:AddFiller( 16, 0 )

                ui:AddItem(
                    UI:Create( "DROPDOWN", ui, function( ui )
                        local tbl = {
                            Group:Local( "Effects_List_All" ),
                            Group:Local( "Effects_List_Debuffs" ),
                            Group:Local( "Effects_List_Curables" )
                        }

                        main.type = {
                            [ Group:Local( "Effects_List_Debuffs" ) ]  = "debuff",
                            [ Group:Local( "Effects_List_Curables" ) ] = "curable"
                        }

                        main.filter.dropdown = ui

                        for i = 1, #tbl do
                            local item = ui:Add( tbl[ i ], function()
                                main.filter_type = tbl[ i ]

                                local selected = main:GetSelected()

                                if selected then selected:ClearSelected() end

                                main.scroll.list:ClearItems()

                                main.search:SetText( "" )
                                main.search.placeholder:SetVisible( true )

                                main.items = 0 

                                local hide = true

                                for j = 1, #main.effect_item do
                                    local item = main.effect_item[ j ]

                                    if main.type[ tbl[ i ] ] then
                                        if item.effect[ main.type[ tbl[ i ] ] ] then
                                            main.scroll.list:AddItem( item )

                                            item:SetVisible( true )
                                            item:SetSelected()

                                            hide = false

                                            main.items = main.items + 1
                                        end
                                    else
                                        main.scroll.list:AddItem( item )

                                        item:SetVisible( true )
                                        item:SetSelected()

                                        hide = false

                                        main.items = main.items + 1
                                    end
                                end

                                if main.items > 15 then
                                    main.scroll.thumb:SetVisible( true )
                                else
                                    main.scroll.thumb:SetVisible( false )
                                end

                                if hide then
                                    main:NoItems()
                                else
                                    main:HasItems()
                                end
                            end )

                            if i == 1 then
                                ui:SetSelected( item )
                            end
                        end
                    end )
                )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.dropdown:SetWidth( w - 48 - 16 - 16 - UI.Spacing / 2 )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        main:AddItem(
            UI:Create( "SCROLLBOX", main, function( ui )
                main.scroll = ui

                ui.thumb:SetHeight( 64 )

                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                end )

                function ui.list:SizeChanged() 
                    local w, h = self:GetSize()

                    main.scroll.border:SetSize( w, h )
                end
            end )
        )

        main:AddFiller( 0, UI.Spacing )
                        
        main:AddItem(
            UI:Create( "TEXTBOX", main, function( ui )
                ui:SetHeight( 24 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                end
                
                ui:SetTextAlignment( 5 )
                ui:SetMultiline( false )
                ui:SetText( "" )

                main.search = ui

                function ui:TextChanged()
                    local text = self:GetText()

                    if text == "" then
                        self.placeholder:SetVisible( true )
                    else
                        self.placeholder:SetVisible( false )
                    end

                    text = string.lower( text )

                    local selected = main:GetSelected()

                    if selected then selected:ClearSelected() end

                    main.scroll.list:ClearItems()

                    main.items = 0 

                    local hide = true

                    for j = 1, #main.effect_item do
                        local item = main.effect_item[ j ]
                        local name = string.lower( item:GetName() )

                        if main.type[ main.filter_type ] then
                            if item.effect[ main.type[ main.filter_type ] ] then
                                --if text == string.sub( name, 1, #text ) then
                                if string.find( name, text ) then
                                    main.scroll.list:AddItem( item )

                                    item:SetVisible( true )
                                    item:SetSelected()

                                    hide = false

                                    main.items = main.items + 1
                                end
                            end
                        else
                            --if text == string.sub( name, 1, #text ) then
                            if string.find( name, text ) then
                                main.scroll.list:AddItem( item )

                                item:SetVisible( true )
                                item:SetSelected()

                                hide = false

                                main.items = main.items + 1
                            end
                        end
                    end

                    if main.items > 15 then
                        main.scroll.thumb:SetVisible( true )
                    else
                        main.scroll.thumb:SetVisible( false )
                    end

                    if hide then
                        main:NoItems()
                    else
                        main:HasItems()
                    end
                end

                ui.placeholder = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetText( Group:Local( "Effects_List_Search" )  ) --"Search" )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetForeColor( Turbine.UI.Color( 0.5, 0.5, 0.5 ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetMouseVisible( false )
                    ui:SetZOrder( 50 )
                end )

                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                    ui:SetZOrder( 50 )
                    ui:SetMouseVisible( false )
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.placeholder:SetWidth( w )
                    self.border:SetWidth( w )
                end
            end )
        )

        function main:AddItem( ui )
            self.items = self.items + 1

            if self.items > 16 then
                self.scroll.thumb:SetVisible( true )
            else
                self.scroll.thumb:SetVisible( false )
            end

            self.effect_item[ #self.effect_item + 1 ] = ui

            ui.effect_item = #self.effect_item

            local text = string.lower( main.search:GetText() )
            local name = string.lower( ui:GetName() )

            if self.type[ self.filter_type ] then
                if ui.effect[ self.type[ self.filter_type ] ] then
                    if text ~= "" and string.find( name, text ) then
                        self.scroll:AddItem( ui )
                    else
                        ui:SetVisible( false )
                    end
                else
                    ui:SetVisible( false )
                end
            else
                if text ~= "" and string.find( name, text ) then
                    self.scroll:AddItem( ui )
                elseif text == "" then
                    self.scroll:AddItem( ui )
                else
                    ui:SetVisible( false )
                end
            end

            return ui
        end

        function main:RemoveItem( ui )
            self.items = self.items - 1

            table.remove( self.effect_item, ui.effect_item )

            self.scroll:Remove( ui )
        end

        function main:GetItemCount()
            return self.items
        end

        function main:EnableScrollbar( enable )
            self.scroll.thumb:SetVisible( enable )
        end

        function main:AddFiller( x, y )
            self.scroll:AddFiller( x, y )
        end

        function main:SizeChanged() 
            local w, h = self:GetSize()

            self.filter:SetWidth( w )
            self.scroll:SetSize( w, h - 16 - 24 * 2 - UI.Spacing )
            self.search:SetWidth( w - 16 - UI.Spacing / 2 )
            self.search:SizeChanged()
        end
    end
end )

UI:CreateTemplate( "LIST", "EFFECTCONFIG", function( template )
    function template:Init( main )
        main:SetHeight( 20 + 16 * 6 + UI.Spacing * 3 + 14 * 4 )

        function main:AddFiller( x, y )
            self:AddItem(
                UI:Create( "BOX", self, function( ui )
                    ui:SetSize( x, y )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                end )
            )
        end

        main:AddItem(
            UI:Create( "HEADER", main, function( ui )
                ui:SetText( Group:Local( "Effects_Config_Header" ) )

                main.header = ui
            end )
        )

        main:AddFiller( 0, 16 )

        function main:TrackSelfChanged( checked )
            --Override
        end

        function main:TrackOthersChanged( checked )
            --Override
        end

        function main:StackAmountChanged( amount )
            --Override
        end

        --Track self
        main:AddItem(
            UI:Create( "BOX", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetColor( 0, 100, 0, 0 )
                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                main.track_self_box = ui

                ui.text = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 16 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetText( Group:Local( "Effects_Config_Track_Self" ) )
                end )

                ui.box = UI:Create( "CHECKBOX", ui, function( ui )
                    ui:SetSize( 16, 16 )
                    ui:SetChecked( true )

                    main.track_self = ui
                    
                    function ui:CheckedChanged()
                        main:TrackSelfChanged( self:IsChecked() )
                    end
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.text:SetWidth( w - 16 - UI.Spacing )
                    self.box:SetPos( w - 16 )
                end
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Track others
        main:AddItem(
            UI:Create( "BOX", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetColor( 0, 100, 0, 0 )
                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                main.track_others_box = ui

                ui.text = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 16 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetText( Group:Local( "Effects_Config_Track_Others" ) )
                end )

                ui.box = UI:Create( "CHECKBOX", ui, function( ui )
                    ui:SetSize( 16, 16 )
                    ui:SetChecked( true )

                    main.track_others = ui

                    function ui:CheckedChanged()
                        main:TrackOthersChanged( self:IsChecked() )
                    end
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.text:SetWidth( w - 16 - UI.Spacing )
                    self.box:SetPos( w - 16 )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetText( Group:Local( "Effects_Config_Stack" ) )

                main.stack_header = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Stack Amount
        main:AddItem(
            UI:Create( "SLIDER_RESET", main, function( ui )
                ui:SetRange( 1, 10 )
                ui:SetKey( "" )
                ui:SetKeySize( 0 )
                ui:SetValue( 1 )

                main.stack_amount = ui

                function ui:ValueChanged( value )
                    main:StackAmountChanged( value )
                end

                function ui:Reset()
                    ui:SetValue( 1 )
                end
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Note
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 14 * 4 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                end
                
                ui:SetText( Group:Local( "Effects_Config_Note" ) )
                ui:SetForeColor( Color( 125, 125, 125 ) )

                main.note = ui
            end )
        )

        function main:SizeChanged()
            local w = self:GetWidth()

            self.header:SetWidth( w )
            self.header:SizeChanged()

            self.track_self_box:SetWidth( w )
            self.track_others_box:SetWidth( w )

            self.stack_header:SetWidth( w )
            self.stack_amount:SetWidth( w )

            self.note:SetWidth( w )
        end

        function main:Reset()
            main.stack_amount:Reset()

            main.track_self:SetChecked( true )
            main.track_others:SetChecked( true )
        end

        function main:GetConfig()
            return {
                track_self   = self.track_self:IsChecked(),
                track_others = self.track_others:IsChecked(),
                stack_amount = self.stack_amount:GetValue()
            }
        end

        function main:SetConfig( config )
            self.track_self:SetChecked( config.track_self, true )
            self.track_others:SetChecked( config.track_others, true )
            self.stack_amount:SetValue( config.stack_amount, true )
            self.stack_amount.slider.Value:SetText( config.stack_amount )
        end
    end 
end )

UI:CreateTemplate( "LIST", "EFFECTALERTCONFIG", function( template )
    function template:Init( main )
        main:SetHeight( 20 + 16 * 2 + 24 + UI.Spacing * 2 + 14 * 6 + 16 * 2 + 24 + UI.Spacing * 2 + 16 * 2 + 16 * 2 )

        function main:AddFiller( x, y )
            self:AddItem(
                UI:Create( "BOX", self, function( ui )
                    ui:SetSize( x, y )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                end )
            )
        end

        main:AddItem(
            UI:Create( "HEADER", main, function( ui )
                ui:SetText( Group:Local( "Effects_Config_Alert_Header" ) )

                main.header = ui
            end )
        )

        main:AddFiller( 0, 16 )

        --Enable Effect Alert
        main:AddItem(
            UI:Create( "BOX", ui, function( ui )
                ui:SetSize( 256 + 32, 16 )
                ui:SetColor( 0, 100, 0, 0 )
                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                main.enable_self = ui

                ui.text = UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( 256 + 32, 16 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetText( Group:Local( "Effects_Config_Alert_Enable_Self" ) )
                end )

                ui.check = UI:Create( "CHECKBOX", ui, function( ui )
                    ui:SetSize( 16, 16 )
                    ui:SetPos( 256 + 32 - 16, 0 )

                    function ui:CheckedChanged()
                        main:CheckChangedSelf( self:IsChecked() )
                    end
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.text:SetWidth( w - 16 - UI.Spacing )
                    self.check:SetPos( w - 16, 0 )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetText( Group:Local( "Effects_Config_Alert_Message_Self" ) )

                main.placeholder_self_header = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Placeholder
        main:AddItem(
            UI:Create( "TEXTBOX", main, function( ui )
                ui:SetHeight( 24 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                end
                
                ui:SetTextAlignment( 5 )
                ui:SetMultiline( false )
                ui:SetText( "" )

                main.placeholder_self = ui

                function ui:TextChanged()
                    local text = self:GetText()

                    if string.sub( text, 1, 1 ) == " " then
                        self:SetText( string.sub( text, 2, #text ) )

                        return 
                    end

                    if text == "" then
                        self.placeholder:SetVisible( true )
                    else
                        self.placeholder:SetVisible( false )
                    end

                    main:PlaceholderChangedSelf( text )
                end

                ui.placeholder = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetText( Group:Local( "Effects_Config_Alert_Placeholder" ) )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetForeColor( Turbine.UI.Color( 0.5, 0.5, 0.5 ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetMouseVisible( false )
                    ui:SetZOrder( 50 )
                end )

                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                    ui:SetZOrder( 50 )
                    ui:SetMouseVisible( false )
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.placeholder:SetWidth( w )
                    self.border:SetWidth( w )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        --Enable Effect Alert
        main:AddItem(
            UI:Create( "BOX", ui, function( ui )
                ui:SetSize( 256 + 32, 16 )
                ui:SetColor( 0, 100, 0, 0 )
                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                main.enable_others = ui

                ui.text = UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( 256 + 32, 16 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetText( Group:Local( "Effects_Config_Alert_Enable_Others" ) )
                end )

                ui.check = UI:Create( "CHECKBOX", ui, function( ui )
                    ui:SetSize( 16, 16 )
                    ui:SetPos( 256 + 32 - 16, 0 )

                    function ui:CheckedChanged()
                        main:CheckChangedOthers( self:IsChecked() )
                    end
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.text:SetWidth( w - 16 - UI.Spacing )
                    self.check:SetPos( w - 16, 0 )
                end
            end )
        )

        main:AddFiller( 0, 16 )

        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 16 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                end
                
                ui:SetText( Group:Local( "Effects_Config_Alert_Message_Others" ) )

                main.placeholder_others_header = ui
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Placeholder
        main:AddItem(
            UI:Create( "TEXTBOX", main, function( ui )
                ui:SetHeight( 24 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                end
                
                ui:SetTextAlignment( 5 )
                ui:SetMultiline( false )
                ui:SetText( "" )

                main.placeholder_others = ui

                function ui:TextChanged()
                    local text = self:GetText()

                    if string.sub( text, 1, 1 ) == " " then
                        self:SetText( string.sub( text, 2, #text ) )

                        return 
                    end

                    if text == "" then
                        self.placeholder:SetVisible( true )
                    else
                        self.placeholder:SetVisible( false )
                    end

                    main:PlaceholderChangedOthers( text )
                end

                ui.placeholder = UI:Create( "TEXT", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetText( Group:Local( "Effects_Config_Alert_Placeholder" ) )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetForeColor( Turbine.UI.Color( 0.5, 0.5, 0.5 ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetMouseVisible( false )
                    ui:SetZOrder( 50 )
                end )

                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetHeight( 24 )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                    ui:SetZOrder( 50 )
                    ui:SetMouseVisible( false )
                end )

                function ui:SizeChanged()
                    local w = self:GetWidth()

                    self.placeholder:SetWidth( w )
                    self.border:SetWidth( w )
                end
            end )
        )

        main:AddFiller( 0, UI.Spacing )

        --Note
        main:AddItem(
            UI:Create( "TEXT", main, function( ui )
                ui:SetHeight( 14 * 6 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                end
                
                ui:SetText( Group:Local( "Effects_Config_Alert_Note" ) )
                ui:SetForeColor( Color( 125, 125, 125 ) )

                main.note = ui
            end )
        )

        function main:PlaceholderChangedSelf( str )
            --Override
        end

        function main:PlaceholderChangedOthers( str )
            --Override
        end

        function main:CheckChangedSelf( check )
            --Override
        end

        function main:CheckChangedOthers( check )
            --Override
        end

        function main:SizeChanged()
            local w = self:GetWidth()

            self.header:SetWidth( w )
            self.header:SizeChanged()

            self.enable_self:SetWidth( w )

            self.placeholder_self_header:SetWidth( w )

            self.placeholder_self:SetWidth( w )
            self.placeholder_self:SizeChanged()

            self.enable_others:SetWidth( w )

            self.placeholder_others_header:SetWidth( w )

            self.placeholder_others:SetWidth( w )
            self.placeholder_others:SizeChanged()

            self.note:SetWidth( w )
        end
    end 
end )

function Group:CreateEffectMenu()
    self.Effects = UI:Create( "BASE", nil, function( main )
        main:SetSize( 1200 - 2, 800 - 44 )
        main:SetPos( UI.SCREEN_W / 2 - 600, UI.SCREEN_H / 2 - 400 )
        main:SetWantsKeyEvents( true )
        main:SetColor( 0, 0, 0, 0 )

        function main:MouseDown()
            if self.header:IsHovering() then
                self.canDrag = true

                local x, y   = self:GetPos()
                local mx, my = Turbine.UI.Display.GetMousePosition()

                self.start_x = mx - x
                self.start_y = my - y
            end
        end

        function main:MouseUp()
            self.canDrag = false
        end

        function main:MouseMove()
            if self.canDrag then
                local mx, my = Turbine.UI.Display.GetMousePosition()

                local x = mx - self.start_x
                local y = my - self.start_y

                local w, h = self:GetSize()

                if x < 0 then x = 0 end
                if x > ( UI.SCREEN_W - w ) then x = UI.SCREEN_W - w end

                if y < 0 then y = 0 end
                if y > ( UI.SCREEN_H - h ) then y = UI.SCREEN_H - h end

                self:SetPos( x, y )
            end
        end

        main.ui_toggle    = false
        main.save_visible = main:IsVisible()

        function main:KeyDown( key )
            if key.Action == Turbine.UI.Lotro.Action.Escape then
                if self.ui_toggle then
                    self.ui_toggle = false

                    self:SetVisible( self.save_visible )
                else
                    self:SetVisible( false )

                    Group.Settings.force_effect_menu = nil

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                end
            elseif key.Action == Turbine.UI.Lotro.Action.UI_Toggle then
                self.disable_changed = true
    
                if self.ui_toggle then
                    self:SetVisible( self.save_visible )
                else
                    self:SetVisible( false )
                end
    
                self.disable_changed = false
    
                self.ui_toggle = not self.ui_toggle
            end
        end
    
        function main:VisibleChanged()
            if not self.disable_changed then
                self.save_visible = self:IsVisible()
            end
        end

        main.tabs = {
            Group:Local( "Effects_Tab_1" ),
            Group:Local( "Effects_Tab_2" ),
            Group:Local( "Effects_Tab_3" ),
            Group:Local( "Effects_Tab_4" ),
            Group:Local( "Effects_Tab_5" ),
            Group:Local( "Effects_Tab_6" )
        }

        main.window = {}

        UI:Create( "BOX", main, function( ui )
            ui:SetPos( 0, 30 )
            ui:SetSize( 1200 - 2, 800 - 30 - 44 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:Fill()
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
            end )

            UI:Create( "BOX", ui, function( ui )
                ui:SetPos( 256 + 32 + 32 + 2, 16 + 2 )
                ui:SetSize( 1, 800 - 30 - 32 - 4 - 44 )
                ui:SetColor( 100, 100, 100 )
            end )

            --Main
            UI:Create( "LIST", ui, function( ui )
                ui:SetPos( 16 + 2, 16 + 2 )
                ui:SetSize( 256 + 32, 800 - 30 - 44 )

                local w = 256 + 32

                ui:AddItem(
                    UI:Create( "TEXT", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                        if language == Turbine.Language.German then
                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                        end
                        
                        ui:SetText( Group:Local( "Effects_Record" ) )
                    end )
                )

                ui:AddFiller( 0, UI.Spacing )

                --Record button
                ui:AddItem(
                    UI:Create( "BUTTON", ui, function( ui )
                        ui:SetSize( 256 + 32, 48 )
                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanProBold22 )
                        ui:SetText( Group:Local( "Effects_Record_Inactive" ) )
                        ui:SetForeColor( Color( 200, 0, 0 ) )

                        main.record_button = ui

                        ui.recording = Group.record

                        function ui:MouseEnter()
                            if self.recording then
                                ui:SetText( Group:Local( "Effects_Record_Active_Stop" ) )
                                ui:SetForeColor( Color( 200, 0, 0 ) )
                            else
                                ui:SetText( Group:Local( "Effects_Record_Inactive_Start" ) )
                                ui:SetForeColor( Color( 0, 200, 0 ) )
                            end
                        end

                        function ui:MouseLeave()
                            if self.recording then
                                ui:SetText( Group:Local( "Effects_Record_Active" ) )
                                ui:SetForeColor( Color( 0, 200, 0 ) )
                            else
                                ui:SetText( Group:Local( "Effects_Record_Inactive" ) )
                                ui:SetForeColor( Color( 200, 0, 0 ) )
                            end
                        end

                        ui:MouseLeave()

                        function ui:Pressed()
                            self.recording = not self.recording

                            Group.record = self.recording

                            if self.recording then
                                ui:SetText( Group:Local( "Effects_Record_Active" ) )
                                ui:SetForeColor( Color( 0, 200, 0 ) )
                            else
                                ui:SetText( Group:Local( "Effects_Record_Inactive" ) )
                                ui:SetForeColor( Color( 200, 0, 0 ) )
                            end

                            if Group.record then
                                Group.QuickMenu.status:SetText( Group:Local( "QuickMenu_Status_On" ) )
                                Group.QuickMenu.status:SetForeColor( Color( 0, 255, 0 ) )
                            else
                                Group.QuickMenu.status:SetText( Group:Local( "QuickMenu_Status_Off" ) )
                                Group.QuickMenu.status:SetForeColor( Color( 255, 0, 0 ) )
                            end
                        end
                    end )
                )

                ui:AddFiller( 0, 16 )

                ui:AddItem( --Divider
                    UI:Create( "DIVIDER", ui, function( ui )
                        ui:SetWidth( w )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Enable Effects
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Enable" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Effect.enable )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Effect.enable = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                local options = Group.Options

                                if options then
                                    for i = 1, #options.Frame.effect do
                                        if i <= options.effect_count then
                                            options.Frame.effect[ i ]:SetVisible( checked )
                                        end
                                    end
                                end

                                for i = 1, #Group.Party do
                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                end
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Enable Alerts
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Enable_Alert" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Effect.enable_alert )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Effect.enable_alert = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Enable Special Effect
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Enable_Special" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Effect.enable_special )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Effect.enable_special = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                local options = Group.Options

                                if options then
                                    options.Frame.special:SetVisible( checked )

                                    if checked then
                                        options.Frame.special_text:SetVisible( Group.Settings.enable_special_effect_duration )
                                    else
                                        options.Frame.special_text:SetVisible( false )
                                    end
                                end

                                for i = 1, #Group.Party do
                                    if Group.Party[ i ].special.active then
                                        Group.Party[ i ].special:SetVisible( checked )
                                        if checked then
                                            Group.Party[ i ].special_text:SetVisible( Group.Settings.enable_special_effect_duration )
                                        else
                                            Group.Party[ i ].special_text:SetVisible( false )
                                        end
                                    end
                                end
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                ui:AddItem( --Divider
                    UI:Create( "DIVIDER", ui, function( ui )
                        ui:SetWidth( w )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Track Debuff Effects
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( w, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( w, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Track_Debuff" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( w - 16, 0 )
                            ui:SetChecked( Group.Settings.track_debuff )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Settings.track_debuff = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                for i = 1, #Group.Party do
                                    if Group.Party[ i ].localplayer then
                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                    end
                                end
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Track Curable Effects
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( w, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( w, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Track_Curable" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( w - 16, 0 )
                            ui:SetChecked( Group.Settings.track_curable )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Settings.track_curable = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                for i = 1, #Group.Party do
                                    if Group.Party[ i ].localplayer then
                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                    end
                                end
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Track All Effects
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( w, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( w, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                            if language == Turbine.Language.German then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end
                            
                            ui:SetText( Group:Local( "Effects_Track_All" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( w - 16, 0 )
                            ui:SetChecked( Group.Settings.track_all )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Settings.track_all = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                for i = 1, #Group.Party do
                                    if Group.Party[ i ].localplayer then
                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                    end
                                end
                            end
                        end )
                    end )
                )
            end )

            local tabs = main.tabs

            for i = 1, #tabs do
                main.window[ tabs[ i ] ] = UI:Create( "LIST", ui, function( ui )
                    ui:SetPos( 256 + 32 + 32 + 2 + 16, 16 + 2 )
                    ui:SetSize( 1200 - 4 - 256 - 32 - 32 - 32 - 2, 800 - 30 - 32 - 4 - 44 )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                    ui:SetVisible( false )

                    local w, h = ui:GetSize()

                    if tabs[ i ] == Group:Local( "Effects_Tab_1") then --Blacklist
                        ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Effect List
                        ui:AddItem(
                            UI:Create( "EFFECTLIST", ui, function( ui )
                                ui:SetSize( w / 2, h )

                                main.blacklist_list = ui

                                function ui:GetSelected()
                                    return self.scroll.list.selected
                                end

                                function ui:HasItems()
                                    main.blacklist:SetVisible( true )
                                end

                                function ui:NoItems()
                                    main.blacklist:SetVisible( false )
                                end

                                function ui:CreateItem( name, icon, effect )
                                    local item = false

                                    self:AddItem(
                                        UI:Create( "EFFECTITEM", self, function( ui, parent )
                                            ui:SetWidth( w / 2 )
                                            
                                            ui:SetName( name )
                                            ui:SetIcon( icon )

                                            ui.effect = effect

                                            item = ui

                                            function ui:Selected()
                                                main.blacklist_info:SetName( name )
                                                main.blacklist_info:SetIcon( icon )
                                                main.blacklist_info:SetDebuff( self.effect.debuff )
                                                main.blacklist_info:SetCurable( self.effect.curable )
                                                main.blacklist_info:SetCategory( self.effect.category )
                                                main.blacklist_info:SetDuration( self.effect.duration )
                                                main.blacklist_info:SetDescription( self.effect.description )

                                                main.blacklist:SetVisible( true )
                                            end
                                        end )
                                    )

                                    item:SetSelected( function()
                                        main.blacklist:SetVisible( true )
                                    end )
                                end
                            end )
                        )

                        ui:AddFiller( 14, 0 )

                        --Info + Remove
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w / 2 - 14, h )
                                ui:SetVisible( false )

                                main.blacklist = ui

                                --Info
                                ui:AddItem(
                                    UI:Create( "EFFECTINFO", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.blacklist_info = ui
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Remove
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Remove" ) )

                                        function ui:Pressed()
                                            local selected = main.blacklist_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Blacklist[ name ] then
                                                if Group.Effect.Blacklist[ name ][ icon ] then
                                                    Group.Effect.Blacklist[ name ][ icon ] = nil

                                                    local clear = true

                                                    for k, v in pairs( Group.Effect.Blacklist[ name ] ) do
                                                        clear = false
                                                    end

                                                    if clear then
                                                        Group.Effect.Blacklist[ name ] = nil
                                                    end

                                                    selected:ClearSelected( function()
                                                        main.blacklist:SetVisible( false )
                                                    end )

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.blacklist_list:RemoveItem( selected )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Blacklist_Removed" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        for name, effect in pairs( Group.Effect.Blacklist ) do
                            for icon, item in pairs( effect ) do
                                main.blacklist_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )
                            end
                        end
                    elseif tabs[ i ] == Group:Local( "Effects_Tab_2" ) then --Special
                        --ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Info
                        ui:AddItem(
                            UI:Create( "EFFECTINFO", ui, function( ui )
                                ui:SetWidth( w / 2 - 14 )

                                main.special_info = ui

                                ui.header:SetText( Group:Local( "Effects_Info_Header_Special" ) )

                                local name = Group.Effect.Special[ 1 ].name
                                local icon = Group.Effect.Special[ 1 ].icon

                                if not Group.Library[ name ] then ui:SetVisible( false ) return end

                                local effect = Group.Library[ name ][ icon ]

                                ui:SetName( name )
                                ui:SetIcon( icon )
                                ui:SetDebuff( effect.debuff )
                                ui:SetCurable( effect.curable )
                                ui:SetCategory( effect.category )
                                ui:SetDuration( effect.duration )
                                ui:SetDescription( effect.description )
                            end )
                        )
                    elseif tabs[ i ] == Group:Local( "Effects_Tab_3" ) then --Alert
                        ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Effect List
                        ui:AddItem(
                            UI:Create( "EFFECTLIST", ui, function( ui )
                                ui:SetSize( w / 2, h )

                                main.alert_list = ui

                                function ui:GetSelected()
                                    return self.scroll.list.selected
                                end

                                function ui:HasItems()
                                    main.alert:SetVisible( true )
                                end

                                function ui:NoItems()
                                    main.alert:SetVisible( false )
                                end

                                function ui:CreateItem( name, icon, effect )
                                    local item = false

                                    self:AddItem(
                                        UI:Create( "EFFECTITEM", self, function( ui, parent )
                                            ui:SetWidth( w / 2 )
                                            
                                            ui:SetName( name )
                                            ui:SetIcon( icon )

                                            ui.effect = effect

                                            item = ui

                                            function ui:Selected()
                                                main.alert_info:SetName( name )
                                                main.alert_info:SetIcon( icon )
                                                main.alert_info:SetDebuff( self.effect.debuff )
                                                main.alert_info:SetCurable( self.effect.curable )
                                                main.alert_info:SetCategory( self.effect.category )
                                                main.alert_info:SetDuration( self.effect.duration )
                                                main.alert_info:SetDescription( self.effect.description )

                                                main.alert:SetVisible( true )

                                                main.alert_config.enable_self.check:SetChecked( Group.Effect.Alert[ name ][ icon ].alert_self, true )

                                                local text_self = Group.Effect.Alert[ name ][ icon ].message_self

                                                main.alert_config.placeholder_self:SetText( text_self )

                                                if text_self == "" then
                                                    main.alert_config.placeholder_self.placeholder:SetVisible( true )
                                                else
                                                    main.alert_config.placeholder_self.placeholder:SetVisible( false )
                                                end

                                                local text_others = Group.Effect.Alert[ name ][ icon ].message_others

                                                main.alert_config.enable_others.check:SetChecked( Group.Effect.Alert[ name ][ icon ].alert_others, true )

                                                main.alert_config.placeholder_others:SetText( text_others )

                                                if text_others == "" then
                                                    main.alert_config.placeholder_others.placeholder:SetVisible( true )
                                                else
                                                    main.alert_config.placeholder_others.placeholder:SetVisible( false )
                                                end
                                                
                                            end
                                        end )
                                    )

                                    item:SetSelected( function()
                                        main.alert:SetVisible( true )
                                    end )
                                end
                            end )
                        )

                        ui:AddFiller( 14, 0 )

                        --Info + Config + Remove
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w / 2 - 14, h )
                                ui:SetVisible( false )

                                main.alert = ui

                                --Info
                                ui:AddItem(
                                    UI:Create( "EFFECTINFO", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.alert_info = ui
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Config
                                ui:AddItem(
                                    UI:Create( "EFFECTALERTCONFIG", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.alert_config = ui

                                        function ui:CheckChangedSelf( checked )
                                            local selected = main.alert_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Alert[ name ][ icon ].alert_self = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
                                        end
                                
                                        function ui:CheckChangedOthers( checked )
                                            local selected = main.alert_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Alert[ name ][ icon ].alert_others = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
                                        end

                                        function ui:PlaceholderChangedSelf( str )
                                            local selected = main.alert_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Alert[ name ][ icon ].message_self = str

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
                                        end

                                        function ui:PlaceholderChangedOthers( str )
                                            local selected = main.alert_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Alert[ name ][ icon ].message_others = str

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Remove
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Remove" ) )

                                        function ui:Pressed()
                                            local selected = main.alert_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Alert[ name ] then
                                                if Group.Effect.Alert[ name ][ icon ] then
                                                    Group.Effect.Alert[ name ][ icon ] = nil

                                                    local clear = true

                                                    for k, v in pairs( Group.Effect.Alert[ name ] ) do
                                                        clear = false
                                                    end

                                                    if clear then
                                                        Group.Effect.Alert[ name ] = nil
                                                    end

                                                    selected:ClearSelected( function()
                                                        main.alert:SetVisible( false )
                                                    end )

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.alert_list:RemoveItem( selected )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Alert_Removed" ), "<effect>", name ), 3 )
                                                end
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        for name, effect in pairs( Group.Effect.Alert ) do
                            for icon, item in pairs( effect ) do
                                main.alert_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )
                            end
                        end
                    elseif tabs[ i ] == Group:Local( "Effects_Tab_4" ) then --Tracking
                        ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Effect List
                        ui:AddItem(
                            UI:Create( "EFFECTLIST", ui, function( ui )
                                ui:SetSize( w / 2, h )

                                main.tracking_list = ui

                                function ui:GetSelected()
                                    return self.scroll.list.selected
                                end

                                function ui:HasItems()
                                    main.tracking:SetVisible( true )
                                end

                                function ui:NoItems()
                                    main.tracking:SetVisible( false )
                                end

                                function ui:CreateItem( name, icon, effect )
                                    local item = false

                                    self:AddItem(
                                        UI:Create( "EFFECTITEM", self, function( ui, parent )
                                            ui:SetWidth( w / 2 )
                                            
                                            ui:SetName( name )
                                            ui:SetIcon( icon )

                                            ui.effect = effect

                                            item = ui

                                            function ui:Selected()
                                                main.tracking_info:SetName( name )
                                                main.tracking_info:SetIcon( icon )
                                                main.tracking_info:SetDebuff( self.effect.debuff )
                                                main.tracking_info:SetCurable( self.effect.curable )
                                                main.tracking_info:SetCategory( self.effect.category )
                                                main.tracking_info:SetDuration( self.effect.duration )
                                                main.tracking_info:SetDescription( self.effect.description )

                                                main.tracking_config:SetConfig( Group.Effect.Track[ name ][ icon ] )

                                                main.tracking:SetVisible( true )
                                            end
                                        end )
                                    )

                                    item:SetSelected( function()
                                        main.tracking:SetVisible( true )
                                    end )
                                end
                            end )
                        )

                        ui:AddFiller( 14, 0 )

                        --Info + Config + Remove
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w / 2 - 14, h )
                                ui:SetVisible( false )

                                main.tracking = ui

                                --Info
                                ui:AddItem(
                                    UI:Create( "EFFECTINFO", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.tracking_info = ui
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Config
                                ui:AddItem(
                                    UI:Create( "EFFECTCONFIG", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.tracking_config = ui

                                        function ui:TrackSelfChanged( checked )
                                            local selected = main.tracking_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Track[ name ][ icon ].track_self = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].localplayer then
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )

                                                    break
                                                end
                                            end
                                        end
                                
                                        function ui:TrackOthersChanged( checked )
                                            local selected = main.tracking_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Track[ name ][ icon ].track_others = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                            for i = 1, #Group.Party do
                                                if not Group.Party[ i ].localplayer then
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    
                                                    break
                                                end
                                            end
                                        end
                                
                                        function ui:StackAmountChanged( amount )
                                            local selected = main.tracking_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Track[ name ][ icon ].stack_amount = amount

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                            for i = 1, #Group.Party do
                                                if not Group.Party[ i ].localplayer then
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    
                                                    break
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Remove
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Remove" ) )

                                        function ui:Pressed()
                                            local selected = main.tracking_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Track[ name ] then
                                                if Group.Effect.Track[ name ][ icon ] then
                                                    Group.Effect.Track[ name ][ icon ] = nil

                                                    local clear = true

                                                    for k, v in pairs( Group.Effect.Track[ name ] ) do
                                                        clear = false
                                                    end

                                                    if clear then
                                                        Group.Effect.Track[ name ] = nil
                                                    end

                                                    selected:ClearSelected( function()
                                                        main.tracking:SetVisible( false )
                                                    end )

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.tracking_list:RemoveItem( selected )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Track_Removed" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        for name, effect in pairs( Group.Effect.Track ) do
                            for icon, item in pairs( effect ) do
                                main.tracking_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )
                            end
                        end
                    elseif tabs[ i ] == Group:Local( "Effects_Tab_5" ) then --Library
                        ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Effect List
                        ui:AddItem(
                            UI:Create( "EFFECTLIST", ui, function( ui )
                                ui:SetSize( w / 2, h )

                                main.library_list = ui

                                function ui:GetSelected()
                                    return self.scroll.list.selected
                                end

                                function ui:HasItems()
                                    main.library:SetVisible( true )
                                end

                                function ui:NoItems()
                                    main.library:SetVisible( false )
                                end

                                function ui:CreateItem( name, icon, effect )
                                    local item = false

                                    self:AddItem(
                                        UI:Create( "EFFECTITEM", self, function( ui, parent )
                                            ui:SetWidth( w / 2 )
                                            
                                            ui:SetName( name )
                                            ui:SetIcon( icon )

                                            ui.effect = effect

                                            item = ui

                                            function ui:Selected()
                                                main.library_info:SetName( name )
                                                main.library_info:SetIcon( icon )
                                                main.library_info:SetDebuff( self.effect.debuff )
                                                main.library_info:SetCurable( self.effect.curable )
                                                main.library_info:SetCategory( self.effect.category )
                                                main.library_info:SetDuration( self.effect.duration )
                                                main.library_info:SetDescription( self.effect.description )
                                            end
                                        end )
                                    )

                                    item:SetSelected( function()
                                        main.library:SetVisible( true )
                                    end )
                                end
                            end )
                        )

                        ui:AddFiller( 14, 0 )

                        --Info + Track + Alert + Blacklist + Special
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w / 2 - 14, h )
                                ui:SetVisible( false )

                                main.library = ui 

                                --Info
                                ui:AddItem(
                                    UI:Create( "EFFECTINFO", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.library_info = ui
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Track
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Track" ) )

                                        function ui:Pressed()
                                            local selected = main.library_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Track[ name ] then
                                                if Group.Effect.Track[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Track[ name ][ icon ] = {
                                                        track_self   = true,
                                                        track_others = true,
                                                        stack_amount = 1
                                                    }

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.tracking_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Track" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            else
                                                Group.Effect.Track[ name ] = {
                                                    [ icon ] = {
                                                        track_self   = true,
                                                        track_others = true,
                                                        stack_amount = 1
                                                    }
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.tracking_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Track" ), "<effect>", name ), 3 )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Alert
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Alert" ) )

                                        function ui:Pressed()
                                            local selected = main.library_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Alert[ name ] then
                                                if Group.Effect.Alert[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Alert[ name ][ icon ] = {
                                                        alert_self     = true,
                                                        message_self   = "",
                                                        alert_others   = true,
                                                        message_others = ""
                                                    }

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.alert_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Alert" ), "<effect>", name ), 3 )
                                                end
                                            else
                                                Group.Effect.Alert[ name ] = {
                                                    [ icon ] = {
                                                        alert_self     = true,
                                                        message_self   = "",
                                                        alert_others   = true,
                                                        message_others = ""
                                                    }
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.alert_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Alert" ), "<effect>", name ), 3 )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Blacklist
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Blacklist" ) )

                                        function ui:Pressed()
                                            local selected = main.library_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Blacklist[ name ] then
                                                if Group.Effect.Blacklist[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Blacklist[ name ][ icon ] = true

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.blacklist_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Blacklist" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            else
                                                Group.Effect.Blacklist[ name ] = {
                                                    [ icon ] = true
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.blacklist_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Blacklist" ), "<effect>", name ), 3 )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Special
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Special" ) )

                                        function ui:Pressed()
                                            local selected = main.library_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Special[ 1 ] = {
                                                name = name,
                                                icon = icon
                                            }

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                            local effect = Group.Library[ name ][ icon ]

                                            main.special_info:SetVisible( true )

                                            main.special_info:SetName( name )
                                            main.special_info:SetIcon( icon )
                                            main.special_info:SetDebuff( effect.debuff )
                                            main.special_info:SetCurable( effect.curable )
                                            main.special_info:SetCategory( effect.category )
                                            main.special_info:SetDuration( effect.duration )
                                            main.special_info:SetDescription( effect.description )

                                            Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Special" ), "<effect>", name ), 3 )
                                            
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        for name, effect in pairs( Group.Library ) do
                            for icon, item in pairs( effect ) do
                                main.library_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )
                            end
                        end
                    elseif tabs[ i ] == Group:Local( "Effects_Tab_6" ) then --Session
                        ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                        --Effect List
                        ui:AddItem(
                            UI:Create( "EFFECTLIST", ui, function( ui )
                                ui:SetSize( w / 2, h )

                                main.session_list = ui

                                function ui:GetSelected()
                                    return self.scroll.list.selected
                                end

                                function ui:HasItems()
                                    main.session:SetVisible( true )
                                end

                                function ui:NoItems()
                                    main.session:SetVisible( false )
                                end

                                function ui:CreateItem( name, icon, effect )
                                    local item = false

                                    self:AddItem(
                                        UI:Create( "EFFECTITEM", self, function( ui, parent )
                                            ui:SetWidth( w / 2 )
                                            
                                            ui:SetName( name )
                                            ui:SetIcon( icon )

                                            ui.effect = effect

                                            item = ui

                                            function ui:Selected()
                                                main.session_info:SetName( name )
                                                main.session_info:SetIcon( icon )
                                                main.session_info:SetDebuff( self.effect.debuff )
                                                main.session_info:SetCurable( self.effect.curable )
                                                main.session_info:SetCategory( self.effect.category )
                                                main.session_info:SetDuration( self.effect.duration )
                                                main.session_info:SetDescription( self.effect.description )
                                            end
                                        end )
                                    )

                                    item:SetSelected( function()
                                        main.session:SetVisible( true )
                                    end )
                                end
                            end )
                        )

                        ui:AddFiller( 14, 0 )

                        --Info + Track + Alert + Blacklist + Special
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w / 2 - 14, h )
                                ui:SetVisible( false )

                                main.session = ui

                                --Info
                                ui:AddItem(
                                    UI:Create( "EFFECTINFO", ui, function( ui )
                                        ui:SetWidth( w / 2 - 14 )

                                        main.session_info = ui
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Track
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Track" ) )

                                        function ui:Pressed()
                                            local selected = main.session_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Track[ name ] then
                                                if Group.Effect.Track[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Track[ name ][ icon ] = {
                                                        track_self   = true,
                                                        track_others = true,
                                                        stack_amount = 1
                                                    }

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.tracking_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Track" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            else
                                                Group.Effect.Track[ name ] = {
                                                    [ icon ] = {
                                                        track_self   = true,
                                                        track_others = true,
                                                        stack_amount = 1
                                                    }
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.tracking_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Track" ), "<effect>", name ), 3 )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Alert
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Alert" ) )

                                        function ui:Pressed()
                                            local selected = main.session_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Alert[ name ] then
                                                if Group.Effect.Alert[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Alert[ name ][ icon ] = {
                                                        alert_self     = true,
                                                        message_self   = "",
                                                        alert_others   = true,
                                                        message_others = ""
                                                    }

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.alert_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Alert" ), "<effect>", name ), 3 )
                                                end
                                            else
                                                Group.Effect.Alert[ name ] = {
                                                    [ icon ] = {
                                                        alert_self     = true,
                                                        message_self   = "",
                                                        alert_others   = true,
                                                        message_others = ""
                                                    }
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.alert_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Alert" ), "<effect>", name ), 3 )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Blacklist
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Blacklist" ) )

                                        function ui:Pressed()
                                            local selected = main.session_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()
                                            
                                            if Group.Effect.Blacklist[ name ] then
                                                if Group.Effect.Blacklist[ name ][ icon ] then
                                                    return
                                                else
                                                    Group.Effect.Blacklist[ name ][ icon ] = true

                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                    main.blacklist_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                    Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Blacklist" ), "<effect>", name ), 3 )

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                    end
                                                end
                                            else
                                                Group.Effect.Blacklist[ name ] = {
                                                    [ icon ] = true
                                                }

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                                main.blacklist_list:CreateItem( name, icon, Group.Library[ name ][ icon ] )

                                                Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Blacklist" ), "<effect>", name ), 3 )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, 16 )

                                --Special
                                ui:AddItem(
                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( w / 2 - 14, 32 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                                        if language == Turbine.Language.German then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Effects_Button_Special" ) )

                                        function ui:Pressed()
                                            local selected = main.library_list:GetSelected()

                                            local name = selected:GetName()
                                            local icon = selected:GetIcon()

                                            Group.Effect.Special[ 1 ] = {
                                                name = name,
                                                icon = icon
                                            }

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Effects", table.save( Group.Effect ) )

                                            local effect = Group.Library[ name ][ icon ]

                                            main.special_info:SetVisible( true )

                                            main.special_info:SetName( name )
                                            main.special_info:SetIcon( icon )
                                            main.special_info:SetDebuff( effect.debuff )
                                            main.special_info:SetCurable( effect.curable )
                                            main.special_info:SetCategory( effect.category )
                                            main.special_info:SetDuration( effect.duration )
                                            main.special_info:SetDescription( effect.description )

                                            Group.Notification:SetNotification( string.replace( Group:Local( "Notification_Effect_Special" ), "<effect>", name ), 3 )
                                            
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ]:RefreshEffects( Group.Party[ i ].player_effects, true )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )
                    end
                end )
            end
        end )

        main.header = UI:Create( "BOX", main, function( ui )
            ui:SetSize( 256, 32 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:SetSize( 256, 34 )
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
                ui:SetMouseVisible( false )
            end )

            UI:Create( "TEXT", ui, function( ui )
                ui:SetSize( 256, 32 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                if language == Turbine.Language.German then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                end
                
                ui:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
                ui:SetText( Group:Local( "Effects" ) )
                ui:SetMouseVisible( false )
            end )
        end )

        UI:Create( "BOX", main, function( ui )
            ui:SetPos( 1200 - 32 - 2, 0 )
            ui:SetSize( 32, 32 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:SetSize( 32, 34 )
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
                ui:SetMouseVisible( false )
            end )

            UI:Create( "IBUTTON", ui, function( ui )
                ui:SetSize( 32, 32 )

                ui.icon = UI:Create( "BOX", ui, function( ui )
                    ui:SetPos( 10, 10 )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetMouseVisible( false )
                    ui:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )
                    ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
                    ui:SetSize( 32, 32 )
                    ui:SetStretchMode( 1 )
                    ui:SetSize( 12, 12 )
                end )

                function ui:MouseEnter()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit_hover.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end

                function ui:MouseLeave()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end

                function ui:MouseDown()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end
                
                function ui:MouseUp() 
                    if not self:IsHovering() then return end

                    main:SetVisible( false )

                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )

                    Group.Settings.force_effect_menu = nil

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                end
            end )
        end )

        local tabs = main.tabs

        for i = 1, #tabs do
            UI:Create( "BOX", main, function( ui )
                ui:SetPos( 1200 - 100 * i - 16 * i - 32 - 16 - 2, 0 )
                ui:SetSize( 100, 32 )
                ui:SetColor( 25, 25, 25, 225 )
                
                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetSize( 100, 32 )
                    ui:SetBorderSize( 2 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                    ui:SetMouseVisible( false )
                end )
    
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( 100, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
                    ui:SetText( Group:Local( "Effects_Tab_" .. i ) )
                    ui:SetMouseVisible( false )
                end )

                if Group.Settings.force_effect_menu == i then
                    ui.selected = true

                    ui.border:SetHeight( 34 )

                    main.tab      = ui
                    main.selected = tabs[ i ]

                    main.window[ tabs[ i ] ]:SetVisible( true )

                    main.selected_tab = i
                elseif not Group.Settings.force_effect_menu and i == #tabs then
                    ui.selected = true

                    ui.border:SetHeight( 34 )

                    main.tab      = ui
                    main.selected = tabs[ i ]

                    main.window[ tabs[ i ] ]:SetVisible( true )

                    Group.Settings.force_effect_menu = i

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                    main.selected_tab = i
                end

                function ui:MouseEnter()
                    if self.selected then return end

                    self:SetColor( 75, 75, 75, 200 )
                end

                function ui:MouseLeave()
                    self:SetColor( 25, 25, 25, 225 )
                end

                function ui:MouseDown()
                    if self.selected then return end

                    self.selected = true

                    self:SetColor( 25, 25, 25, 225 )
                    self.border:SetHeight( 34 )

                    main.tab.border:SetHeight( 32 )
                    main.tab.selected = false

                    main.window[ main.selected ]:SetVisible( false )
                    
                    main.tab      = self
                    main.selected = tabs[ i ]

                    main.window[ main.selected ]:SetVisible( true )

                    Group.Settings.force_effect_menu = i

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                    main.selected_tab = i
                end
            end )
        end
    end )
end
