import "Turbine"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "Turbine.Gameplay"

UI = {}

import "Darf.Util"
import "Darf.UI"

Player = Turbine.Gameplay.LocalPlayer.GetInstance()

Group = {}

Group.enable_layout_preview = false 

import "Darf.MiniRaid.localization"

local version  = plugin:GetVersion()
local language = Group:GetLanguage()

Group.version = Group:Local( "Internal_Version" ) .. version

Group.Default_Settings = {
    Pos = {
        x = UI.SCREEN_W / 2,
        y = UI.SCREEN_H / 2
    },

    QuickMenu = {
        x      = UI.SCREEN_W / 2,
        y      = UI.SCREEN_H / 2,
        enable = false
    },

    Color = {
        Name                = Color( 255, 255, 255, 150 ),
        Name_Outline        = Color( 0, 0, 0, 255 ),
        Morale              = Color( 65, 255, 65, 65 ),
        Power               = Color( 0, 100, 200, 191 ),
        Wrath               = Color( 191, 145, 25, 191 ),
        Bubble              = Color( 255, 65, 255, 65 ),
        Dead                = Color( 100, 25, 25, 191 ),
        Background          = Color( 25, 25, 25, 191 ),
        Select              = Color( 255, 255, 255, 191 ),
        Hover               = Color( 165, 165, 165, 191 ),
        Player              = Color( 145, 145, 65, 191 ),
        Unique_Morale       = Color( 65, 255, 255, 65 ),
        Unique_Name         = Color( 65, 145, 191, 150 ),
        Unique_Name_Outline = Color( 0, 0, 0, 255 ),
        Threshold           = Color( 255, 65, 65, 65 ),
        Power_Text          = Color( 255, 255, 255, 150 ),
        Morale_Text         = Color( 255, 255, 255, 150 ),
        Special_Effect      = Color( 255, 255, 255, 150 ),

        Class = {
            [ Turbine.Gameplay.Class.Hunter ]     = Color( 145, 191, 65, 191 ),
            [ Turbine.Gameplay.Class.Warden ]     = Color( 191, 65, 145, 191 ),
            [ Turbine.Gameplay.Class.Burglar ]    = Color( 191, 145, 65, 191 ),
            [ Turbine.Gameplay.Class.Captain ]    = Color( 145, 65, 191, 191 ),
            [ Turbine.Gameplay.Class.Champion ]   = Color( 65, 65, 145, 191 ),
            [ Turbine.Gameplay.Class.Guardian ]   = Color( 191, 65, 65, 191 ),
            [ Turbine.Gameplay.Class.Minstrel ]   = Color( 191, 191, 65, 191 ),
            [ Turbine.Gameplay.Class.Beorning ]   = Color( 145, 145, 145, 191 ),
            [ Turbine.Gameplay.Class.LoreMaster ] = Color( 65, 145, 191, 191 ),
            [ Turbine.Gameplay.Class.RuneKeeper ] = Color( 65, 191, 65, 191 ),
            [ Turbine.Gameplay.Class.Brawler ]    = Color( 65, 65, 65, 191 ),

            [ Turbine.Gameplay.Class.Weaver ]     = Color( 65, 145, 191, 191 ),
            [ Turbine.Gameplay.Class.Reaver ]     = Color( 65, 65, 145, 191 ),
            [ Turbine.Gameplay.Class.Defiler ]    = Color( 191, 191, 65, 191 ),
            [ Turbine.Gameplay.Class.Stalker ]    = Color( 191, 145, 65, 191 ),
            [ Turbine.Gameplay.Class.WarLeader ]  = Color( 145, 65, 191, 191 ),
            [ Turbine.Gameplay.Class.BlackArrow ] = Color( 145, 191, 65, 191 )
        }
    },

    morale_threshold                  = 0.85,

    enable_classic_power_bar          = true,

    enable_class_icon                 = true,
    enable_leader_icon                = true,
    enable_player_name                = true,
    enable_power_text                 = false,
    enable_morale_text                = true,

    enable_alphabetical_sorting       = false,
    
    enable_unique_player_name_color   = false,
    
    enable_unique_class_morale_color  = false,
    enable_unique_player_morale_color = false,

    enable_effect_duration            = true,
    enable_special_effect_duration    = true,

    disable_vanilla_party_ui          = false,
    disable_vanilla_player_ui         = false,
    disable_vanilla_target_ui         = false,

    enable_unique_player_frame_pos    = false,

    Player_Frame_Pos = {
        x = UI.SCREEN_W / 2,
        y = UI.SCREEN_H / 2
    },

    hide_out_of_combat                = false,
    show_vitals_out_of_group          = false,

    track_all                         = false,
    track_curable                     = true,
    track_debuff                      = false,

    Frame = {
        Name = {
            x      = 24,
            y      = 4,
            anchor = 1  --Top Left
        },
        Class = {
            x      = 4,
            y      = 4,
            anchor = 1  --Top Left
        },
        Leader = {
            x      = 4,
            y      = 4,
            anchor = 2  --Top Right
        },
        Power_Text = {
            x      = 4,
            y      = 4,
            anchor = 4  --Bottom Left
        },
        Morale_Text = {
            x      = 24,
            y      = 22,
            anchor = 4  --Bottom Left
        },
        Effect = {
            x      = 4,
            y      = 4,
            anchor = 4  --Bottom Left
        },
        Special_Effect = {
            x      = 4,
            y      = 4,
            anchor = 3  --Bottom Right
        }
    },

    Layout = {
        { -- 1 - 3 Players
            x   = 3,
            y   = 1,
            min = 1,
            max = 3
        },
        { -- 4 - 6 Players
            x   = 2,
            y   = 3,
            min = 4,
            max = 6
        },
        { -- 7 - 12 Players
            x   = 3,
            y   = 4,
            min = 7,
            max = 12
        },
        { -- 13 - 18 Players
            x   = 4,
            y   = 5,
            min = 13,
            max = 18
        },
        { -- 19 - 24 Players
            x   = 4,
            y   = 6,
            min = 19,
            max = 24
        }
    },

    vitals_x                    = 155,
    vitals_y                    = 66,
    power_size                  = 6,

    name_width                  = 124,

    class_icon_size             = 16,
    leader_icon_size            = 16, 

    power_text_format           = 6,
    morale_text_format          = 6,

    selection_size              = 2,

    vitals_external_spacing     = 4,

    vertical_layout_direction   = 2,
    horizontal_layout_direction = 2,

    effect_size                 = 18,
    effect_spacing              = 2,
    effect_row_count            = 9,
    effect_flash_threshold      = 5,
    effect_duration_threshold   = 60,

    special_effect_size         = 18,

    player_font_type            = 7,
    player_font_size            = 1,

    power_font_type             = 7,
    power_font_size             = 1,

    morale_font_type            = 7,
    morale_font_size            = 1,
    
    effect_font_type            = 7,
    effect_font_size            = 5,

    special_effect_font_type    = 7,
    special_effect_font_size    = 1,

    player_font_outline         = false,
    power_font_outline          = false,
    morale_font_outline         = false,
    effect_font_outline         = true,
    special_effect_font_outline = false,

    force_option_menu           = false,
    force_effect_menu           = false
}

Group.Font = {
    [ 1 ] = { 
        name = "Arial",
        size = {
            12
        }
    },
    [ 2 ] = {
        name = "BookAntiqua",
        size = {
            12,
            14,
            16,
            18,
            20,
            22,
            24,
            26,
            28,
            32,
            36
        }
    },
    [ 3 ] = {
        name = "BookAntiquaBold",
        size = {
            12,
            14,
            18,
            19,
            22,
            24
        }
    },
    [ 4 ] = {
        name = "FixedSys",
        size = {
            15
        }
    },
    [ 5 ] = {
        name = "LucidaConsole",
        size = {
            12
        }
    },
    [ 6 ] = {
        name = "TrajanPro",
        size = {
            13,
            14,
            15,
            16,
            18,
            19,
            20,
            21,
            23,
            24,
            25,
            26,
            28
        }
    },
    [ 7 ] = {
        name = "TrajanProBold",
        size = {
            16,
            22,
            24,
            25,
            30,
            36
        }
    },
    [ 8 ] = {
        name = "Verdana",
        size = {
            10,
            12,
            14,
            16,
            18,
            20,
            22,
            23
        }
    },
    [ 9 ] = {
        name = "VerdanaBold",
        size = {
            16
        }
    },
    [ 10 ] = {
        name = "TimesRoman",
        size = {
            18,
            20,
            22,
            26,
            28,
            30,
            36
        }
    },
    [ 11 ] = {
        name = "TimesRomanBold",
        size = {
            20,
            22
        }
    },
    [ 12 ] = {
        name = "Courier",
        size = {
            12
        }
    }
}

Group.FontSpecial = {
    [ 10 ] = { --TimesRoman
        0x4200000e,
        0x420000f1,
        0x420000f4,
        0x42000169,
        0x42000168,
        0x4200016a,
        0x4200016b
    },
    [ 11 ] = { --TimesRomanBold
        0x4200000f,
        0x42000010
    },
    [ 12 ] = { --Courier12
        0x42000028
    }
}

function Group:GetFont( type, size )
    if Group.FontSpecial[ type ] then
        return Group.FontSpecial[ type ][ size ]
    else
        return Turbine.UI.Lotro.Font[ Group.Font[ type ].name .. Group.Font[ type ].size[ size ] ]
    end
end

Group.Settings = table.load( Turbine.PluginData.Load( Turbine.DataScope.Character, "MiniRaid_Settings" ) )

function table.add( target, source )
    local tbl = source --Wtf is this SSG :P

    for k, v in pairs( tbl ) do
        if type( v ) == "table" then
            if target[ k ] == nil then
                target[ k ] = {}

                table.add( target[ k ], v )

                newSettings = true
            else
                if target[ k ].A and target[ k ].R and target[ k ].G and target[ k ].B then
                    target[ k ] = Turbine.UI.Color( target[ k ].A, target[ k ].R, target[ k ].G, target[ k ].B )
                else
                    table.add( target[ k ], v )
                end
            end
        else
            if target[ k ] == nil then
                target[ k ] = v

                newSettings = true
            end
        end
    end
end

if not Group.Settings then
    Group.Settings = {}

    table.add( Group.Settings, Group.Default_Settings )

    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
else
    local newSettings = false

    table.add( Group.Settings, Group.Default_Settings )

    if newSettings then
        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
    end
end

Group.Global = table.load( Turbine.PluginData.Load( Turbine.DataScope.Account, "MiniRaid_Settings_Global" ) )

if not Group.Global then
    Group.Global = {}

    Turbine.PluginData.Save( Turbine.DataScope.Account, "MiniRaid_Settings_Global", table.save( Group.Default_Settings ) )
end

Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Party, not Group.Settings.disable_vanilla_party_ui )
Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, not Group.Settings.disable_vanilla_player_ui )

--Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Target, not Group.Settings.disable_vanilla_target_ui )

Group.record      = false
Group.reload      = true

Group.Party       = {}
Group.PartyName   = {}
Group.PartyCount  = 0
Group.Grid_Size_X = 0
Group.Grid_Size_Y = 0

Group.Layout = table.load( Turbine.PluginData.Load( Turbine.DataScope.Character, "MiniRaid_Layout" ) )

if not Group.Layout then
    Print( "MiniRaid v" .. version .. " by Darf" )
end

--0x41007DD0 --Voice Active
--0x41007DD1 --Voice Inactive

--0x41007DD1 Color Gradient

--0x410081FC --0x410081C3  --Leader

--0x410E6BF4 Frame

--Size = 36 x 36
--0x410F25C3 Disease
--0x410F25C4 Wound
--0x410F25C5 Fear
--0x410F25C6 Poison
--0x410F25C2 Corruption

Group.ClassIcon = {
    [ Turbine.Gameplay.Class.Hunter ]     = 0x41007dea, --0x410000E8 --0x41007DDA --0x41008201 --0x410095C2                 --0x41108678
    [ Turbine.Gameplay.Class.Warden ]     = 0x410ea239, --0x410095C5 --0x410EA237                                           --0x410F25C2
    [ Turbine.Gameplay.Class.Burglar ]    = 0x41007deb, --0x410000E4 --0x41007DDB --0x41007E87 --0x410095BB                 --0x41108674
    [ Turbine.Gameplay.Class.Captain ]    = 0x41007de7, --0x410000E5 --0x41007DD7 --0x410081F3 --0x410095C5                 --0x41108675
    [ Turbine.Gameplay.Class.Champion ]   = 0x41007dec, --0x410000E6 --0x41007DDC --0x410081CD --0x410095B5                 --0x41108676
    [ Turbine.Gameplay.Class.Guardian ]   = 0x41007de8, --0x410000E7 --0x41007DD8 --0x410081FF --0x410095B8                 --0x41108677
    [ Turbine.Gameplay.Class.Minstrel ]   = 0x41007de6, --0x410000E9 --0x41007DD6 --0x41004C49 --0x410081F7                 --0x4110867A
    [ Turbine.Gameplay.Class.Beorning ]   = 0x4115370d,
    [ Turbine.Gameplay.Class.LoreMaster ] = 0x41007de9, --0x410000E9 --0x41007DD9 --0x410081FC --0x410095BF                 --0x41108679
    [ Turbine.Gameplay.Class.RuneKeeper ] = 0x410ea238, --0x410E6BF4 --0x410EA236                                           --0x4110867B
    [ Turbine.Gameplay.Class.Brawler ]    = 0x4115370d, --Temp Beorning icon

    [ Turbine.Gameplay.Class.Weaver ]     = 0x41007DEE, --0x41007DE0 --0x41007DCA
    [ Turbine.Gameplay.Class.Reaver ]     = 0x41007DED, --0x41007DDD --0x41007DC9
    [ Turbine.Gameplay.Class.Defiler ]    = 0x410E6BF4, --0x410E6BF5 --0x410E6BF4
    [ Turbine.Gameplay.Class.Stalker ]    = 0x41007DF1, --0x41007DE1 --0x41007DCD
    [ Turbine.Gameplay.Class.WarLeader ]  = 0x41007DF0, --0x41007DDD --0x41007DCC
    [ Turbine.Gameplay.Class.BlackArrow ] = 0x41007DEF  --0x41007DDF --0x41007DCB
}

Group.hidden = false

Group.Notification = UI:Create( "BASE", nil, function( main )
    main:SetSize( UI.SCREEN_W, UI.SCREEN_H )
    main:SetMouseVisible( false )
    main:SetZOrder( 300 )
    main:SetColor( 0, 0, 0, 0 )

    main.background = UI:Create( "BOX", main, function( ui )
        ui:SetMouseVisible( false )
        ui:SetVisible( false )
        ui:SetColor( 150, 150, 150, 100 )
        ui:SetSize( UI.SCREEN_W, 32 )
        ui:SetPos( 0, UI.SCREEN_H / 2 - 16 )
    end )

    main.text = UI:Create( "TEXT", main, function( ui )
        ui:SetSize( UI.SCREEN_W, UI.SCREEN_H )
        ui:SetTextAlignment( 5 )
        ui:SetMouseVisible( false )
        ui:SetVisible( false )
        ui:SetFont( Turbine.UI.Lotro.Font.TrajanProBold24 )

        if language == Turbine.Language.German then
            ui:SetFont( Turbine.UI.Lotro.Font.Verdana23 )
        end

        ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
        ui:SetText( "MiniRaid Notification Message" )
    end )

    function main:SetNotification( str, duration )
        self.text:SetText( str )
        self.text:SetVisible( true )
        self.background:SetVisible( true )

        self.duration = Turbine.Engine.GetGameTime() + duration

        self.active = true
    end

    main:SetWantsUpdates( true )

    function main:Update()
        if self.active then
            local time = Turbine.Engine.GetGameTime()

            if self.duration < time then
                self.active = false

                self.text:SetVisible( false )
                self.background:SetVisible( false )
            end
        end
    end
end )

Group.QuickMenu = UI:Create( "BASE", nil, function( main )
    main:SetSize( 32 + UI.Spacing * 2, 32 + UI.Spacing * 2 )
    main:SetPos( Group.Settings.QuickMenu.x, Group.Settings.QuickMenu.y )
    main:SetColor( 0, 0, 0, 0 )
    main:SetVisible( Group.Settings.QuickMenu.enable )
    main:SetWantsKeyEvents( true )
    main:SetZOrder( 200 )

    main.ui_toggle    = false
    main.save_visible = main:IsVisible()

    function main:KeyDown( key )
        if key.Action == Turbine.UI.Lotro.Action.Escape then
            if self.ui_toggle then
                self.ui_toggle = false

                self:SetVisible( self.save_visible )
            end
        elseif key.Action == Turbine.UI.Lotro.Action.UI_Toggle then
            self.disable_changed = true

            if self.ui_toggle then
                self:SetVisible( self.save_visible )
            else
                self:SetVisible( false )
            end

            self.disable_changed = false

            self.ui_toggle = not self.ui_toggle
        end
    end

    function main:VisibleChanged()
        if not self.disable_changed then
            self.save_visible = self:IsVisible()
        end
    end

    local w, h = 192, 32 + UI.Spacing * 2 + 32 * 4 + 1 + UI.Spacing * 2

    main:SetSize( w, h )

    main.list = UI:Create( "LIST", main, function( ui )
        ui:SetSize( w, h )
        ui:SetColor( 25, 25, 25, 225 )
        ui:SetVisible( false )

        UI:Create( "BORDER", ui, function( ui )
            ui:SetSize( w, h )
            ui:SetBorderSize( 1 )
            ui:SetBorderColor( Color( 100, 100, 100 ) )
            ui:SetMouseVisible( false )
        end )

        UI:Create( "TEXT", ui, function( ui )
            ui:SetPos( 32 + UI.Spacing * 2, UI.Spacing )
            ui:SetSize( w - 32 + UI.Spacing * 2, 32 )
            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

            if language == Turbine.Language.German then
                ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
            end
            
            ui:SetText( Group:Local( "QuickMenu_Record" ) )
            ui:SetTextAlignment( 4 )
        end )

        main.status = UI:Create( "TEXT", ui, function( ui )
            ui:SetPos( 32 + UI.Spacing * 2 + 110, UI.Spacing )
            ui:SetSize( 64, 32 )
            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

            if language == Turbine.Language.German then
                ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )

                ui:SetPos( 32 + UI.Spacing * 2 + 116, UI.Spacing )
            end
            
            ui:SetText( Group:Local( "QuickMenu_Status_Off" ) )
            ui:SetTextAlignment( 4 )
            ui:SetForeColor( Color( 255, 0, 0 ) )
        end )

        UI:Create( "LIST", ui, function( ui )
            ui:SetPos( UI.Spacing + 1, 0 )
            ui:SetSize( w - UI.Spacing * 2 - 2, h )

            ui:AddFiller( 0, 32 + UI.Spacing * 2 - 1 )

            local w = ui:GetWidth()

            ui:AddItem( --Divider
                UI:Create( "DIVIDER", ui, function( ui )
                    ui:SetWidth( w )
                end )
            )

            ui:AddFiller( 0, UI.Spacing )

            ui:AddItem(
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( w, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetText( Group:Local( "QuickMenu_Options" ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                    function ui:MouseEnter()
                        self:SetColor( 150, 150, 150, 50 )
                    end

                    function ui:MouseLeave()
                        self:SetColor( 0, 0, 0, 0 )
                    end

                    function ui:MouseDown()
                        self:SetForeColor( Color( 200, 0, 0 ) )
                    end

                    function ui:MouseUp()
                        self:SetForeColor( Color( 255, 255, 255 ) )

                        if Group.Options then
                            local visible = Group.Options:IsVisible()
            
                            Group.Options:SetVisible( not visible )
            
                            if visible then
                                Group.Settings.force_option_menu = nil
            
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            else
                                Group.Settings.force_option_menu = Group.Options.selected_tab
            
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            end
                        else
                            Group:CreateOptionMenu()
                        end

                        main:MouseLeave()
                    end
                end )
            )

            ui:AddItem(
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( w, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetText( Group:Local( "QuickMenu_Effects" ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                    function ui:MouseEnter()
                        self:SetColor( 150, 150, 150, 50 )
                    end

                    function ui:MouseLeave()
                        self:SetColor( 0, 0, 0, 0 )
                    end

                    function ui:MouseDown()
                        self:SetForeColor( Color( 200, 0, 0 ) )
                    end

                    function ui:MouseUp()
                        self:SetForeColor( Color( 255, 255, 255 ) )

                        if Group.Effects then
                            local visible = Group.Effects:IsVisible()
            
                            Group.Effects:SetVisible( not visible )
            
                            if visible then
                                Group.Settings.force_effect_menu = nil
            
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            else
                                Group.Settings.force_effect_menu = Group.Effects.selected_tab
            
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            end
                        else
                            Group:CreateEffectMenu()
                        end

                        main:MouseLeave()
                    end
                end )
            )

            ui:AddItem(
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( w, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetText( Group:Local( "QuickMenu_Toggle" ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                    function ui:MouseEnter()
                        self:SetColor( 150, 150, 150, 50 )
                    end

                    function ui:MouseLeave()
                        self:SetColor( 0, 0, 0, 0 )
                    end

                    function ui:MouseDown()
                        self:SetForeColor( Color( 200, 0, 0 ) )
                    end

                    function ui:MouseUp()
                        self:SetForeColor( Color( 255, 255, 255 ) )

                        Group.hidden = not Group.hidden

                        if Group.hidden then
                            PartyVitals:SetVisible( false )

                            Group.Notification:SetNotification( Group:Local( "Notification_Toggle_Off" ), 3 )
                        else
                            if Group.Settings.hide_out_of_combat then
                                PartyVitals:SetVisible( Player:IsInCombat() )
                            else
                                PartyVitals:SetVisible( true )
                            end

                            Group.Notification:SetNotification( Group:Local( "Notification_Toggle_On" ), 3 )
                        end

                        main:MouseLeave()
                    end
                end )
            )

            ui:AddItem(
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( w, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                    if language == Turbine.Language.German then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                    end
                    
                    ui:SetText( Group:Local( "QuickMenu_Reload" ) )
                    ui:SetTextAlignment( 5 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                    function ui:MouseEnter()
                        self:SetColor( 150, 150, 150, 50 )
                    end

                    function ui:MouseLeave()
                        self:SetColor( 0, 0, 0, 0 )
                    end

                    function ui:MouseDown()
                        self:SetForeColor( Color( 200, 0, 0 ) )
                    end

                    function ui:MouseUp()
                        self:SetForeColor( Color( 255, 255, 255 ) )

                        main:MouseLeave()

                        Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                    end
                end )
            )
        end )
    end )

    main.button = UI:Create( "IBUTTON", main, function( ui )
        ui:SetBackground( "Darf/MiniRaid/Resources/toggle.tga" )
        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
        ui:SetSize( 63, 63 )
        ui:SetStretchMode( 1 )
        ui:SetSize( 32, 32 )
        ui:SetPos( UI.Spacing, UI.Spacing )
        ui:SetOpacity( 0.5 )
         
        function ui:MouseDown()
            self:SetBackground( "Darf/MiniRaid/Resources/toggle.tga" )

            if self:IsShiftKeyDown() then
                self.dragging = true

                local x, y   = main:GetPos()
                local mx, my = Turbine.UI.Display.GetMousePosition()

                self.drag_offset_x = mx - x
                self.drag_offset_y = my - y
            end
        end

        function ui:MouseMove()
            if self.dragging then
                local mx, my = Turbine.UI.Display.GetMousePosition()

                self.mx = mx - self.drag_offset_x
                self.my = my - self.drag_offset_y

                main:SetPos( self.mx, self.my )
            end
        end

        function ui:MouseUp()
            if self.dragging then
                Group.Settings.QuickMenu.x = self.mx
                Group.Settings.QuickMenu.y = self.my

                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
            end

            self.dragging = false

            if self:IsShiftKeyDown() then return end

            self:SetBackground( "Darf/MiniRaid/Resources/toggle_disabled.tga" )

            Group.record = not Group.record

            if Group.record then
                main.status:SetText( Group:Local( "QuickMenu_Status_On" ) )
                main.status:SetForeColor( Color( 0, 255, 0 ) )
            else
                main.status:SetText( Group:Local( "QuickMenu_Status_Off" ) )
                main.status:SetForeColor( Color( 255, 0, 0 ) )
            end

            if Group.Effects then
                Group.Effects.record_button.recording = Group.record
                Group.Effects.record_button:MouseLeave()
            end
        end

        function ui:MouseEnter()
            self:SetOpacity( 1 )

            if self:IsShiftKeyDown() then return end

            main:SetSize( w, h )
            main.list:SetVisible( true )

            self:SetBackground( "Darf/MiniRaid/Resources/toggle_disabled.tga" )
        end

        function ui:MouseLeave()
            self:SetBackground( "Darf/MiniRaid/Resources/toggle.tga" )
        end
    end )

    function main:MouseLeave()
        self.button:SetOpacity( 0.5 )

        self.list:SetVisible( false )

        self:SetSize( 32 + UI.Spacing * 2, 32 + UI.Spacing * 2 )
    end

    function main:SizeChanged()
        self.button:SetStretchMode( 1 )
        self.button:SetSize( 32, 32 )
    end
end )

import "Darf.MiniRaid.settings"
import "Darf.MiniRaid.effects"
import "Darf.MiniRaid.group"

if Group.Settings.force_option_menu then
    Group:CreateOptionMenu()
end

if Group.Settings.force_effect_menu then
    Group:CreateEffectMenu()
end

function Turbine.Plugin:Unload()
    if Group.PartyCount > 1 then
        Group.Layout = {}

        for i = 1, Group.PartyCount do
            Group.Layout[ i ] = Group.Party[ i ]:GetName()
        end

        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Layout", table.save( Group.Layout ) )
    else
        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Layout", false )
    end
end

local status  = 0
control_miniraid = Turbine.UI.Control()

function control_miniraid:Update()
	if status == 1 then
        Turbine.PluginManager.UnloadScriptState( "MiniRaidReload" )
    elseif status > 1 then
        self:SetWantsUpdates( false )
        
        control_miniraid = nil
	end

    status = status + 1
end

control_miniraid:SetWantsUpdates( true )

if Player:GetParty() then
    Group:Create()
else
    if Group.Settings.show_vitals_out_of_group then
        Group:Create( true )
    end
end

if Group.Settings.hide_out_of_combat and not Player:IsInCombat() then
    PartyVitals:SetVisible( false )
end

AddCallback( Player, "InCombatChanged", function() 
    if Group.Settings.hide_out_of_combat and not Player:IsInCombat() then
        PartyVitals:SetVisible( false )
    elseif not Group.hidden then
        PartyVitals:SetVisible( true )
    end
end )

AddCallback( Player, "TargetChanged", function( self )
    Target = self:GetTarget()

    if Target then
        local name = Target:GetName()

        if Group.Selected then
            Group.Selected:SetSelected( false )
        end

        if Group.PartyName and Group.PartyName[ name ] then
            Group.Selected = Group.Party[ Group.PartyName[ name ] ]:SetSelected( true )
        end
    else
        if Group.Selected then
            Group.Selected:SetSelected( false )
        end
    end
end )

local string = string

AddCallback( Turbine.Chat, "Received", function( self, chat )
    if chat.ChatType == Turbine.ChatType.Fellowship then 
        local message = string.lower( chat.Message )

        --"You have joined the chat room for your raid, raid chat is now available."
        local raid = string.find( message, Group:Local( "Chat_Raid_Convert") )

        --Convert to raid
        if raid then
            Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
        end
    end

    if chat.ChatType == Turbine.ChatType.Standard then
        local message = string.lower( chat.Message )

        local group = Group

        --" has joined your Fellowship."
        local fellowship_join, _ = string.find( message, group:Local( "Chat_Fellowship_Join" ) )
        --" has joined your Raid."
        local raid_join, _ = string.find( message, group:Local( "Chat_Raid_Join" ) )

        --------------------------------------------------------------------------------------------------------

        --"You have joined a Fellowship."
        local fellowship_join_self, _ = string.find( message, group:Local( "Chat_Fellowship_Join_Self" ) )
        --"You have joined a Raid."
        local raid_join_self, _ = string.find( message, group:Local( "Chat_Raid_Join_Self" ) )

        --------------------------------------------------------------------------------------------------------

        --" has left your Fellowship."
        local fellowship_leave, _ = string.find( message, group:Local( "Chat_Fellowship_Leave" ) )
        --" has left your Raid."
        local raid_leave, _ = string.find( message, group:Local( "Chat_Raid_Leave" ) )

        --------------------------------------------------------------------------------------------------------

        --"You leave your Fellowship."
        local fellowship_leave_self, _ = string.find( message, group:Local( "Chat_Fellowship_Leave_Self" ) )
        --"You leave your Raid."
        local raid_leave_self, _ = string.find( message, group:Local( "Chat_Raid_Leave_Self" ) )

        --------------------------------------------------------------------------------------------------------

        --"You dismiss "
        local _, self_dismiss_start = string.find( message, group:Local( "Chat_Self_Dismiss_Start" ) )
        --" from the fellowship."
        local fellowship_dismiss_end, _ = string.find( message, group:Local( "Chat_Fellowship_Dismiss_End" ) )
        --" from the Raid."
        local raid_dismiss_end, _ = string.find( message, group:Local( "Chat_Raid_Dismiss_End" ) )

        --------------------------------------------------------------------------------------------------------

        --" has been dismissed from your Fellowship."
        local fellowship_dismiss, _ = string.find( message, group:Local( "Chat_Fellowship_Dismiss" ) )
        --" has been dismissed from your Raid."
        local raid_dismiss, _ = string.find( message, group:Local( "Chat_Raid_Dismiss" ) )

        --------------------------------------------------------------------------------------------------------

        --You have been dismissed from your Fellowship."
        local fellowship_dismiss_self, _ = string.find( message, group:Local( "Chat_Fellowship_Dismiss_Self" ) )
        --"You have been dismissed from your Raid."
        local raid_dismiss_self, _ = string.find( message, group:Local( "Chat_Raid_Dismiss_Self" ) )

        --------------------------------------------------------------------------------------------------------

        --"Your Fellowship has been disbanded."
        local fellowship_disband, _ = string.find( message, group:Local( "Chat_Fellowship_Disband" ) )
        --"Your Raid has been disbanded."
        local raid_disband, _ = string.find( message, group:Local( "Chat_Raid_Disband" ) )

        --------------------------------------------------------------------------------------------------------

        if fellowship_join or raid_join then
            local name = string.sub( message, 1, ( fellowship_join or raid_join ) - 1 )

            if group.Settings.show_vitals_out_of_group then
                if group.PartyCount >= 1 and group.reload then
                    group.reload = false

                    Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                end
            else
                if group.PartyCount >= 2 and group.reload then
                    group.reload = false

                    Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                end
            end
        end

        if fellowship_join_self or raid_join_self then
            if group.PartyCount < 2 then

                if group.Settings.show_vitals_out_of_group then
                    group:Remove()
                end

                group:Create()
            end
        end

        if fellowship_leave or raid_leave then
            local name = ( string.sub( message, 1, ( fellowship_leave or raid_leave ) - 1 ):gsub( "^%l", string.upper ) )

            if group.PartyCount > 2 then
                group:RemoveMember( name )
            elseif group.PartyCount <= 2 then
                group:Remove()

                if group.Settings.show_vitals_out_of_group then
                    group:Create( true )
                end
            end
        end

        if fellowship_leave_self or raid_leave_self then
            group:Remove()

            if group.Settings.show_vitals_out_of_group then
                group:Create( true )
            end
        end

        if self_dismiss_start and ( fellowship_dismiss_end or raid_dismiss_end ) then
            local name = ( string.sub( message, self_dismiss_start + 1, ( fellowship_dismiss_end or raid_dismiss_end ) - 1 ):gsub( "^%l", string.upper ) )

            if group.PartyCount > 2 then
                group:RemoveMember( name )
            elseif group.PartyCount <= 2 then
                group:Remove()

                if group.Settings.show_vitals_out_of_group then
                    group:Create( true )
                end
            end
        end

        if fellowship_dismiss or raid_dismiss then
            local name = ( string.sub( message, 1, ( fellowship_dismiss or raid_dismiss ) - 1 ):gsub( "^%l", string.upper ) )

            if group.PartyCount > 2 then
                group:RemoveMember( name )
            elseif group.PartyCount <= 2 then
                group:Remove()

                if group.Settings.show_vitals_out_of_group then
                    group:Create( true )
                end
            end
        end

        if fellowship_dismiss_self or raid_dismiss_self then
            group:Remove()

            if group.Settings.show_vitals_out_of_group then
                group:Create( true )
            end
        end

        if fellowship_disband or raid_disband then
            group:Remove()

            if group.Settings.show_vitals_out_of_group then
                group:Create( true )
            end
        end
    end
end )

command = Turbine.ShellCommand()

local language_reload = Group:Local( "Chat_Command_Reload" )

if Group:GetLanguage() == Turbine.Language.German then
    language_reload = "neu"
end

function command:Execute( _, str )
    if str == nil or string.len( str ) == 0 then
        return
    end

    local list  = { }
    local index = 1

    for word in str:gmatch( "%w+" ) do
        list[ index ] = word
        index = index + 1
    end

    local cmd = string.lower( list[ 1 ] )

    local group = Group

    if cmd == group:Local( "Chat_Command_Options" ) then
        if group.Options then
            group.Options:SetVisible( true )

            group.Settings.force_option_menu = group.Options.selected_tab

            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( group.Settings ) )
        else
            group:CreateOptionMenu()
        end
    elseif cmd == group:Local( "Chat_Command_Effects" ) then
        if group.Effects then
            group.Effects:SetVisible( true )

            group.Settings.force_effect_menu = group.Effects.selected_tab

            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( group.Settings ) )
        else
            group:CreateEffectMenu()
        end
    elseif cmd == language_reload then
        if group:GetLanguage() == Turbine.Language.German then
            local cmd = string.lower( list[ 2 ] or "" )

            if cmd == "laden" then
                Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
            else
                if cmd == "" then
                    Print( group:Local( "Chat_Message_Invalid" ) .. "neu" )
                    Print( group:Local( "Chat_Message_Help" ) )
                else
                    Print( group:Local( "Chat_Message_Invalid" ) .. cmd )
                    Print( group:Local( "Chat_Message_Help" ) )
                end
            end
        else
            Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
        end
    elseif cmd == group:Local( "Chat_Command_Record" ) then
        
        group.record = not group.record

        if group.Effects then
            group.Effects.record_button.recording = group.record
            group.Effects.record_button:MouseLeave()
        end

        if group.record then
            group.Notification:SetNotification( group:Local( "Notification_Record_Start" ), 3 )

            group.QuickMenu.status:SetText( group:Local( "QuickMenu_Status_On" ) )
            group.QuickMenu.status:SetForeColor( Color( 0, 255, 0 ) )
        else
            group.Notification:SetNotification( group:Local( "Notification_Record_Stop" ), 3 )

            group.QuickMenu.status:SetText( group:Local( "QuickMenu_Status_Off" ) )
            group.QuickMenu.status:SetForeColor( Color( 255, 0, 0 ) )
        end
    elseif cmd == group:Local( "Chat_Command_Help" ) then
        Print( group:Local( "Chat_Message_Help" ) )
    elseif cmd == group:Local( "Chat_Command_Toggle_Party_Vitals" ) then
        group.Settings.disable_vanilla_party_ui = not group.Settings.disable_vanilla_party_ui

        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Party, not group.Settings.disable_vanilla_party_ui )

        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( group.Settings ) )

        if group.Options then
            group.Options.checkbox_toggle_party_vitals:SetChecked( group.Settings.disable_vanilla_party_ui, true )
        end
    elseif cmd == group:Local( "Chat_Command_Toggle_Player_Vitals" ) then
        group.Settings.disable_vanilla_player_ui = not group.Settings.disable_vanilla_player_ui

        Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, not group.Settings.disable_vanilla_player_ui )

        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( group.Settings ) )

        if group.Options then
            group.Options.checkbox_toggle_player_vitals:SetChecked( group.Settings.disable_vanilla_player_ui, true )
        end

    else
        Print( group:Local( "Chat_Message_Invalid" ) .. cmd )
        Print( group:Local( "Chat_Message_Help" ) )
    end
end

Turbine.Shell.AddCommand( Group:Local( "Chat_Command_Self" ), command )

--Debug
--[[ UI:Create( "BASE", nil, function( main )
    main:SetColor( 200, 200, 200, 0 )
    main:SetSize( 56, 16 )
    main:SetPos( 1800, 1416 )

    UI:Create( "BUTTON", main, function( ui )
        ui:SetSize( 56, 16 )
        function ui:Pressed()
            Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
        end
    end )
end )

Group:CreateOptionMenu() ]]