function Group:CreateOptionMenu()
    self.Options = UI:Create( "BASE", nil, function( main )
        main:SetSize( 1200 - 2, 800 - 44 )
        main:SetPos( UI.SCREEN_W / 2 - 600, UI.SCREEN_H / 2 - 400 )
        main:SetWantsKeyEvents( true )
        main:SetColor( 0, 0, 0, 0 )

        local language = self:GetLanguage()

        function main:MouseDown()
            if self.header:IsHovering() then
                self.canDrag = true

                local x, y   = self:GetPos()
                local mx, my = Turbine.UI.Display.GetMousePosition()

                self.start_x = mx - x
                self.start_y = my - y
            end
        end

        function main:MouseUp()
            self.canDrag = false
        end

        function main:MouseMove()
            if self.canDrag then
                local mx, my = Turbine.UI.Display.GetMousePosition()

                local x = mx - self.start_x
                local y = my - self.start_y

                local w, h = self:GetSize()

                if x < 0 then x = 0 end
                if x > ( UI.SCREEN_W - w ) then x = UI.SCREEN_W - w end

                if y < 0 then y = 0 end
                if y > ( UI.SCREEN_H - h ) then y = UI.SCREEN_H - h end

                self:SetPos( x, y )
            end
        end

        main.ui_toggle    = false
        main.save_visible = main:IsVisible()

        function main:KeyDown( key )
            if key.Action == Turbine.UI.Lotro.Action.Escape then
                if self.ui_toggle then
                    self.ui_toggle = false

                    self:SetVisible( self.save_visible )
                else
                    self:SetVisible( false )

                    Group.Settings.force_option_menu = nil

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                end
            elseif key.Action == Turbine.UI.Lotro.Action.UI_Toggle then
                self.disable_changed = true
    
                if self.ui_toggle then
                    self:SetVisible( self.save_visible )
                else
                    self:SetVisible( false )
                end
    
                self.disable_changed = false
    
                self.ui_toggle = not self.ui_toggle
            end
        end
    
        function main:VisibleChanged()
            if not self.disable_changed then
                self.save_visible = self:IsVisible()
            end
        end

        main.tabs = {
            Group:Local( "Options_Tab_1" ),
            Group:Local( "Options_Tab_2" ),
            Group:Local( "Options_Tab_3" ),
            Group:Local( "Options_Tab_4" ),
            Group:Local( "Options_Tab_5" ),
            Group:Local( "Options_Tab_6" ),
            Group:Local( "Options_Tab_7" )
        }

        main.window = {}

        UI:Create( "BOX", main, function( ui )
            ui:SetPos( 0, 30 )
            ui:SetSize( 1200 - 2, 800 - 30 - 44 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:Fill()
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
            end )

            UI:Create( "BOX", ui, function( ui )
                ui:SetPos( 256 + 32 + 32 + 2, 16 + 2 )
                ui:SetSize( 1, 800 - 30 - 32 - 4 - 44 )
                ui:SetColor( 100, 100, 100 )
            end )

            --Main
            UI:Create( "LIST", ui, function( ui )
                ui:SetPos( 16 + 2, 16 + 2 )
                ui:SetSize( 256 + 32, 394 - 44 )

                --Disable Vanilla Party Vitals
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        if language == Turbine.Language.German or language == Turbine.Language.French then
                            ui:SetHeight( 32 )
                        end

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end

                            ui:SetText( Group:Local( "Options_Vanilla_Toggle" ) )

                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                ui:SetHeight( 32 )
                                ui:SetMultiline( true )
                            end
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Settings.disable_vanilla_party_ui )

                            main.checkbox_toggle_party_vitals = ui

                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                ui:SetPos( 256 + 32 - 16, 8 )
                            end

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()
                                
                                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Party, not checked )
        
                                Group.Settings.disable_vanilla_party_ui = checked
        
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Disable Vanilla Player Vitals
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end

                            ui:SetText( Group:Local( "Options_Vanilla_Player_Toggle" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Settings.disable_vanilla_player_ui )

                            main.checkbox_toggle_player_vitals = ui

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()
                                
                                Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Vitals, not checked )
        
                                Group.Settings.disable_vanilla_player_ui = checked
        
                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            end
                        end )
                    end )
                )

                ui:AddFiller( 0, 16 )

                ui:AddItem( --Divider
                    UI:Create( "DIVIDER", ui, function( ui )
                        ui:SetWidth( 256 + 32 )
                    end )
                )

                ui:AddFiller( 0, 16 )

                --Enable Toggle Button
                ui:AddItem(
                    UI:Create( "BOX", ui, function( ui )
                        ui:SetSize( 256 + 32, 16 )
                        ui:SetColor( 0, 100, 0, 0 )
                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        UI:Create( "TEXT", ui, function( ui )
                            ui:SetSize( 256 + 32, 16 )
                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                            end

                            ui:SetText( Group:Local( "Options_QuickMenu" ) )
                        end )

                        UI:Create( "CHECKBOX", ui, function( ui )
                            ui:SetSize( 16, 16 )
                            ui:SetPos( 256 + 32 - 16, 0 )
                            ui:SetChecked( Group.Settings.QuickMenu.enable )

                            function ui:CheckedChanged()
                                local checked = self:IsChecked()

                                Group.Settings.QuickMenu.enable = checked

                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                Group.QuickMenu:SetVisible( checked )
                            end
                        end )
                    end )
                )
            end )

            UI:Create( "HEADER", ui, function( ui )
                ui:SetPos( 16 + 2, 800 - 256 - 32 - 2 - 16 - 30 - 16 - 20 - 44 )
                ui:SetSize( 256 + 32, 20 )
                ui:SetText( Group:Local( "Options_Preview" ) )
            end )
            
            UI:Create( "BOX", ui, function( ui )
                ui:SetPos( 16 + 2, 800 - 256 - 32 - 2 - 16 - 30 - 44 )
                ui:SetSize( 256 + 32, 256 + 32 )
                ui:SetColor( 0, 0, 0, 0 )

                UI:Create( "BORDER", ui, function( ui )
                    ui:SetSize( 256 + 32, 256 + 32 )
                    ui:SetBorderSize( 1 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                end )

                local settings = Group.Settings

                main.Frame = UI:Create( "BOX", ui, function( ui )
                    ui:SetSize( settings.vitals_x, settings.vitals_y )
                    ui:SetPos( ( 256 + 32 ) / 2 - settings.vitals_x / 2, ( 256 + 32 ) / 2 - settings.vitals_y / 2 )
                    ui:SetColor( settings.Color.Background )

                    main.Pos = {
                        [ 1 ] = function( ui, x, y, w, h ) ui:SetPos( x, y ) end, --Top left

                        [ 2 ] = function( ui, x, y, w, h )
                            local settings = Group.Settings

                            if settings.enable_classic_power_bar then
                                ui:SetPos( settings.vitals_x - x - w, y )
                            else
                                ui:SetPos( settings.vitals_x - settings.power_size - x - w, y )
                            end
                        end, --Top Right

                        [ 3 ] = function( ui, x, y, w, h ) 
                            local settings = Group.Settings

                            if settings.enable_classic_power_bar then
                                ui:SetPos( settings.vitals_x - x - w, settings.vitals_y - y - h - settings.power_size )
                            else
                                ui:SetPos( settings.vitals_x - settings.power_size - x - w, settings.vitals_y - y - h ) 
                            end
                        end, --Bottom Right

                        [ 4 ] = function( ui, x, y, w, h ) 
                            local settings = Group.Settings

                            if settings.enable_classic_power_bar then
                                ui:SetPos( x, settings.vitals_y - y - h - settings.power_size ) 
                            else
                                ui:SetPos( x, settings.vitals_y - y - h ) 
                            end
                        end, --Bottom Left

                        [ 5 ] = function( ui, x, y, w, h )
                            local settings = Group.Settings

                            if settings.enable_classic_power_bar then
                                ui:SetPos( settings.vitals_x / 2 - w / 2 + x, ( settings.vitals_y - settings.power_size ) / 2 - h / 2 + y )
                            else
                                ui:SetPos( ( settings.vitals_x - settings.power_size ) / 2 - w / 2 + x, settings.vitals_y / 2 - h / 2 + y ) 
                            end
                        end --Center
                    }

                    ui.power = UI:Create( "BOX", ui, function( ui, parent )
                        if settings.enable_classic_power_bar then
                            ui:SetPos( 0, settings.vitals_y - settings.power_size )
                            ui:SetSize( settings.vitals_x, settings.power_size )
                        else
                            ui:SetSize( settings.power_size, settings.vitals_y )
                        end

                        if Player:GetClass() == Turbine.Gameplay.Class.Beorning or Player:GetClass() == Turbine.Gameplay.Class.Brawler then
                            ui:SetColor( settings.Color.Wrath )
                        else
                            ui:SetColor( settings.Color.Power )
                        end

                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        ui.text = UI:Create( "TEXT", ui, function( ui, parent )
                            ui:SetMultiline( false )
                            ui:SetSize( 256, Group.Font[ settings.power_font_type ].size[ settings.power_font_size ] )
                            ui:SetFont( Group:GetFont( settings.power_font_type, settings.power_font_size ) )
                            ui:SetVisible( settings.enable_power_text )

                            if settings.Frame.Power_Text.anchor == 1 or settings.Frame.Power_Text.anchor == 4 then
                                ui:SetTextAlignment( 4 )
                            elseif settings.Frame.Power_Text.anchor == 5 then
                                ui:SetTextAlignment( 5 )
                            else
                                ui:SetTextAlignment( 6 )
                            end
                
                            if settings.power_font_outline then
                                ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
                            else
                                ui:SetFontStyle( Turbine.UI.FontStyle.None )
                            end
                
                            ui:SetForeColor( settings.Color.Power_Text )

                            ui.format = {
                                [ 1 ] = function() --%
                                    local percent = ( Player:GetPower() / Player:GetMaxPower() ) * 100
                    
                                    ui:SetText( math.round( percent ) .. "%" ) 
                                end,
                                [ 2 ] = function() --Current
                                    ui:SetText( math.format( math.round( Player:GetPower() ) ) ) 
                                end,
                                [ 3 ] = function() --Current + %
                                    local percent = ( Player:GetPower() / Player:GetMaxPower() ) * 100
                    
                                    ui:SetText( math.format( math.round( Player:GetPower() ) ) .. " " .. math.round( percent ) .. "%" ) 
                                end,
                                [ 4 ] = function() --Current / Max
                                    ui:SetText( math.format( math.round( Player:GetPower() ) ) .. "/" .. math.format( math.round( Player:GetMaxPower() ) ) ) 
                                end,
                                [ 5 ] = function() --Current / Max + %
                                    local percent = ( Player:GetPower() / Player:GetMaxPower() ) * 100
                    
                                    ui:SetText( math.format( math.round( Player:GetPower() ) ) .. "/" .. math.format( math.round( Player:GetMaxPower() ) ) .. " " .. math.round( percent ) .. "%" )
                                end,
                                [ 6 ] = function() --Current ( K )
                                    ui:SetText( math.formatK( Player:GetPower() ) )
                                end,
                                [ 7 ] = function() --Current + % ( K )
                                    local percent = ( Player:GetPower() / Player:GetMaxPower() ) * 100

                                    ui:SetText( math.formatK( Player:GetPower() ) .. " " .. math.round( percent ) .. "%" )
                                end,
                                [ 8 ] = function() --Current / Max ( K )
                                    ui:SetText( math.formatK( Player:GetPower() ) .. "/" .. math.formatK( Player:GetMaxPower() ) )
                                end,
                                [ 9 ] = function() --Current / Max + % ( K )
                                    local percent = ( Player:GetPower() / Player:GetMaxPower() ) * 100

                                    ui:SetText( math.formatK( Player:GetPower() ) .. "/" .. math.formatK( Player:GetMaxPower() ) .. " " .. math.round( percent ) .. "%" )
                                end
                            }

                            ui.Pos = {
                                [ 1 ] = function( ui, x, y, w, h ) ui:SetPos( x, y ) end, --Top left
        
                                [ 2 ] = function( ui, x, y, w, h )
                                    local settings = Group.Settings
        
                                    if settings.enable_classic_power_bar then
                                        ui:SetPos( settings.vitals_x - x - w, y )
                                    else
                                        ui:SetPos( settings.power_size - x - w, y )
                                    end
                                end, --Top Right
        
                                [ 3 ] = function( ui, x, y, w, h ) 
                                    local settings = Group.Settings
        
                                    if settings.enable_classic_power_bar then
                                        ui:SetPos( settings.vitals_x - x - w, settings.power_size - y - h )
                                    else
                                        ui:SetPos( settings.power_size - x - w, settings.vitals_y - y - h ) 
                                    end
                                end, --Bottom Right
        
                                [ 4 ] = function( ui, x, y, w, h ) 
                                    local settings = Group.Settings
        
                                    if settings.enable_classic_power_bar then
                                        ui:SetPos( x, settings.power_size - y - h )
                                    else
                                        ui:SetPos( x, settings.vitals_y - y - h ) 
                                    end
                                end, --Bottom Left
        
                                [ 5 ] = function( ui, x, y, w, h )
                                    local settings = Group.Settings
        
                                    if settings.enable_classic_power_bar then
                                        ui:SetPos( settings.vitals_x / 2 - w / 2 + x, settings.power_size / 2 - h / 2 + y )
                                    else
                                        ui:SetPos( settings.power_size / 2 - w / 2 + x, settings.vitals_y / 2 - h / 2 + y ) 
                                    end
                                end --Center
                            }

                            ui.format[ settings.power_text_format ]()

                            ui.Pos[ settings.Frame.Power_Text.anchor ]( ui, settings.Frame.Power_Text.x, settings.Frame.Power_Text.y, ui:GetSize() )

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                self.Pos[ settings.Frame.Power_Text.anchor ]( self, settings.Frame.Power_Text.x, settings.Frame.Power_Text.y, self:GetSize() )
                            end
                        end )

                        function ui:SizeChanged()
                            local w, h = self:GetSize()

                            local settings = Group.Settings

                            if settings.enable_classic_power_bar then
                                parent.morale:SetHeight( settings.vitals_y - h )
                            else
                                parent.morale:SetPos( w, 0 )
                            end
                        end
                    end )

                    ui.morale = UI:Create( "BOX", ui, function( ui, parent )
                        if settings.enable_classic_power_bar then
                            ui:SetSize( settings.vitals_x, settings.vitals_y - settings.power_size )
                        else
                            ui:SetPos( settings.power_size, 0 )
                            ui:SetSize( settings.vitals_x - settings.power_size, settings.vitals_y )
                        end

                        ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                        if settings.enable_unique_player_morale_color then
                            ui:SetColor( settings.Color.Unique_Morale )
                        else
                            ui:SetColor( settings.Color.Morale )
                        end

                        ui.text = UI:Create( "TEXT", ui, function( ui, parent )
                            ui:SetMultiline( false )
                            ui:SetSize( 256, Group.Font[ settings.morale_font_type ].size[ settings.morale_font_size ] )
                            ui:SetFont( Group:GetFont( settings.morale_font_type, settings.morale_font_size ) )
                            ui:SetVisible( settings.enable_morale_text )

                            if settings.Frame.Morale_Text.anchor == 1 or settings.Frame.Morale_Text.anchor == 4 then
                                ui:SetTextAlignment( 4 )
                            elseif settings.Frame.Morale_Text.anchor == 5 then
                                ui:SetTextAlignment( 5 )
                            else
                                ui:SetTextAlignment( 6 )
                            end
                
                            if settings.morale_font_outline then
                                ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
                            else
                                ui:SetFontStyle( Turbine.UI.FontStyle.None )
                            end
                
                            ui:SetForeColor( settings.Color.Morale_Text )

                            ui.format = {
                                [ 1 ] = function() --%
                                    local percent = ( Player:GetMorale() / Player:GetMaxMorale() ) * 100
                    
                                    ui:SetText( math.round( percent ) .. "%" ) 
                                end,
                                [ 2 ] = function() --Current
                                    ui:SetText( math.format( math.round( Player:GetMorale() ) ) ) 
                                end,
                                [ 3 ] = function() --Current + %
                                    local percent = ( Player:GetMorale() / Player:GetMaxMorale() ) * 100
                    
                                    ui:SetText( math.format( math.round( Player:GetMorale() ) ) .. " " .. math.round( percent ) .. "%" ) 
                                end,
                                [ 4 ] = function() --Current / Max
                                    ui:SetText( math.format( math.round( Player:GetMorale() ) ) .. "/" .. math.format( math.round( Player:GetMaxMorale() ) ) ) 
                                end,
                                [ 5 ] = function() --Current / Max + %
                                    local percent = ( Player:GetMorale() / Player:GetMaxMorale() ) * 100
                    
                                    ui:SetText( math.format( math.round( Player:GetMorale() ) ) .. "/" .. math.format( math.round( Player:GetMaxMorale() ) ) .. " " .. math.round( percent ) .. "%" )
                                end,
                                [ 6 ] = function() --Current ( K )
                                    ui:SetText( math.formatK( Player:GetMorale() ) )
                                end,
                                [ 7 ] = function() --Current + % ( K )
                                    local percent = ( Player:GetMorale() / Player:GetMaxMorale() ) * 100

                                    ui:SetText( math.formatK( Player:GetMorale() ) .. " " .. math.round( percent ) .. "%" )
                                end,
                                [ 8 ] = function() --Current / Max ( K )
                                    ui:SetText( math.formatK( Player:GetMorale() ) .. "/" .. math.formatK( Player:GetMaxMorale() ) )
                                end,
                                [ 9 ] = function() --Current / Max + % ( K )
                                    local percent = ( Player:GetMorale() / Player:GetMaxMorale() ) * 100

                                    ui:SetText( math.formatK( Player:GetMorale() ) .. "/" .. math.formatK( Player:GetMaxMorale() ) .. " " .. math.round( percent ) .. "%" )
                                end
                            }

                            ui.format[ settings.morale_text_format ]()

                            main.Pos[ settings.Frame.Morale_Text.anchor ]( ui, settings.Frame.Morale_Text.x, settings.Frame.Morale_Text.y, ui:GetSize() )

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                main.Pos[ settings.Frame.Morale_Text.anchor ]( self, settings.Frame.Morale_Text.x, settings.Frame.Morale_Text.y, self:GetSize() )
                            end
                        end )

                        parent.name = UI:Create( "TEXT", ui, function( ui, parent )
                            ui:SetMultiline( false )
                            ui:SetSize( settings.name_width, Group.Font[ settings.player_font_type ].size[ settings.player_font_size ] )
                            ui:SetVisible( settings.enable_player_name )
                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                            if settings.Frame.Name.anchor == 1 or settings.Frame.Name.anchor == 4 then
                                ui:SetTextAlignment( 4 )
                            elseif settings.Frame.Name.anchor == 5 then
                                ui:SetTextAlignment( 5 )
                            else
                                ui:SetTextAlignment( 6 )
                            end
                            
                            ui:SetFont( Group:GetFont( settings.player_font_type, settings.player_font_size ) )

                            if settings.player_font_outline then
                                ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
                            else
                                ui:SetFontStyle( Turbine.UI.FontStyle.None )
                            end
                            
                            if settings.enable_unique_player_name_color then
                                ui:SetForeColor( settings.Color.Unique_Name )
                                ui:SetOutlineColor( settings.Color.Unique_Name_Outline )
                            else
                                ui:SetForeColor( settings.Color.Name )
                                ui:SetOutlineColor( settings.Color.Name_Outline )
                            end

                            ui:SetText( Player:GetName() )

                            main.Pos[ settings.Frame.Name.anchor ]( ui, settings.Frame.Name.x, settings.Frame.Name.y, ui:GetSize() )

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                main.Pos[ settings.Frame.Name.anchor ]( self, settings.Frame.Name.x, settings.Frame.Name.y, self:GetSize() )
                            end
                        end )
    
                        parent.class = UI:Create( "BOX", ui, function( ui, parent )
                            ui:SetSize( 20, 20 )
                            ui:SetStretchMode( 1 )
                            ui:SetSize( settings.class_icon_size, settings.class_icon_size )
                            ui:SetColor( 0, 0, 0, 0 )
                            ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
                            ui:SetBackground( Group.ClassIcon[ Player:GetClass() ] )
                            ui:SetVisible( settings.enable_class_icon )

                            main.Pos[ settings.Frame.Class.anchor ]( ui, settings.Frame.Class.x, settings.Frame.Class.y, settings.class_icon_size, settings.class_icon_size )

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                main.Pos[ settings.Frame.Class.anchor ]( self, settings.Frame.Class.x, settings.Frame.Class.y, settings.class_icon_size, settings.class_icon_size )
                            end
                        end )
    
                        parent.leader = UI:Create( "BOX", ui, function( ui, parent )
                            ui:SetSize( 16, 16 )
                            ui:SetStretchMode( 1 )
                            ui:SetSize( settings.leader_icon_size, settings.leader_icon_size )
                            ui:SetColor( 0, 0, 0, 0 )
                            ui:SetBackground( 0x410081C3 )
                            ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
                            ui:SetVisible( settings.enable_leader_icon )
                            
                            main.Pos[ settings.Frame.Leader.anchor ]( ui, settings.Frame.Leader.x, settings.Frame.Leader.y, settings.leader_icon_size, settings.leader_icon_size )

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                main.Pos[ settings.Frame.Leader.anchor ]( self, settings.Frame.Leader.x, settings.Frame.Leader.y, settings.leader_icon_size, settings.leader_icon_size )
                            end
                        end )

                        parent.special_text = UI:Create( "TEXT", ui, function( ui, parent )
                            ui:SetSize( 256, Group.Font[ settings.special_effect_font_type ].size[ settings.special_effect_font_size ] )
                            ui:SetTextAlignment( 6 )
                            ui:SetMultiline( false )
                            ui:SetVisible( settings.enable_special_effect_duration )

                            ui:SetFont( Group:GetFont( settings.special_effect_font_type, settings.special_effect_font_size ) )

                            if settings.special_effect_font_outline then
                                ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
                            else
                                ui:SetFontStyle( Turbine.UI.FontStyle.None )
                            end

                            if Group.Effect.enable_special then
                                ui:SetVisible( settings.enable_special_effect_duration )
                            else
                                ui:SetVisible( false )
                            end

                            ui:SetForeColor( settings.Color.Special_Effect )
                            ui:SetText( "00:00" )
                        end )
    
                        parent.special = UI:Create( "BOX", ui, function( ui, parent )
                            ui:SetSize( settings.special_effect_size, settings.special_effect_size )
                            ui:SetColor( 50, 50, 50, 255 )
                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                            ui:SetVisible( Group.Effect.enable_special )

                            function ui:PositionChanged()
                                local x, y = self:GetPos()

                                local settings = Group.Settings

                                if settings.Frame.Special_Effect.anchor == 1 or settings.Frame.Special_Effect.anchor == 4 then
                                    parent:GetParent().special_text:SetTextAlignment( 4 )

                                    parent:GetParent().special_text:SetPos( x + settings.special_effect_size + UI.Spacing, 
                                    y + settings.special_effect_size / 2 - Group.Font[ settings.special_effect_font_type ].size[ settings.special_effect_font_size ] / 2 )
                                else
                                    parent:GetParent().special_text:SetTextAlignment( 6 )

                                    parent:GetParent().special_text:SetPos( x - 256 - UI.Spacing, y + settings.special_effect_size / 2 - Group.Font[ settings.special_effect_font_type ].size[ settings.special_effect_font_size ] / 2 )
                                end
                            end

                            function ui:SizeChanged()
                                local settings = Group.Settings

                                main.Pos[ settings.Frame.Special_Effect.anchor ]( self, settings.Frame.Special_Effect.x, settings.Frame.Special_Effect.y, settings.special_effect_size, settings.special_effect_size )
                            end

                            main.Pos[ settings.Frame.Special_Effect.anchor ]( ui, settings.Frame.Special_Effect.x, settings.Frame.Special_Effect.y, settings.special_effect_size, settings.special_effect_size )
                        end )

                        parent.effect = {}

                        for i = 1, 20 do
                            parent.effect[ i ] = UI:Create( "BOX", ui, function( ui )
                                ui:SetColor( 50, 50, 50, 255 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui.offset = {
                                    [ 1 ] = function( x, y, w, h, ex, ey ) 
                                        local settings = Group.Settings

                                        local offset_x = settings.effect_size * ( x - 1 ) + settings.effect_spacing * ( x - 1 ) + ex
                                        local offset_y = settings.effect_size * ( y - 1 ) + settings.effect_spacing * ( y - 1 ) + ey

                                        return offset_x, offset_y
                                    end,
                                    [ 2 ] = function( x, y, w, h, ex, ey ) 
                                        local settings = Group.Settings

                                        local offset_x = w - settings.effect_size * x - settings.effect_spacing * ( x - 1 ) - ex
                                        local offset_y = settings.effect_size * ( y - 1 ) + settings.effect_spacing * ( y - 1 ) + ey

                                        return offset_x, offset_y
                                    end,
                                    [ 3 ] = function( x, y, w, h, ex, ey )
                                        local settings = Group.Settings

                                        local offset_x = w - settings.effect_size * x - settings.effect_spacing * ( x - 1 ) - ex
                                        local offset_y = h - settings.effect_size * y - settings.effect_spacing * ( y - 1 ) - ey

                                        return offset_x, offset_y
                                    end,
                                    [ 4 ] = function( x, y, w, h, ex, ey ) 
                                        local settings = Group.Settings

                                        local offset_x = settings.effect_size * ( x - 1 ) + settings.effect_spacing * ( x - 1 ) + ex
                                        local offset_y = h - settings.effect_size * y - settings.effect_spacing * ( y - 1 ) - ey

                                        return offset_x, offset_y
                                    end
                                }

                                main.effect_count = 4

                                if i > main.effect_count then
                                    ui:SetVisible( false )
                                else
                                    ui:SetVisible( Group.Effect.enable )
                                end

                                ui.text = UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( 32, 32 )
                                    ui:SetTextAlignment( 5 )
                                    ui:SetMultiline( false )
                                    ui:SetVisible( settings.enable_effect_duration )
                                    ui:SetFont( Group:GetFont( settings.effect_font_type, settings.effect_font_size ) )

                                    ui:SetText( i )

                                    if settings.effect_font_outline then
                                        ui:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                    else
                                        ui:SetFontStyle( Turbine.UI.FontStyle.None )
                                    end
                                end )

                                ui:SetSize( 32, 32 )
                                ui:SetStretchMode( 1 )
                                ui:SetSize( settings.effect_size, settings.effect_size )
                            end )
                        end

                        function ui:PositionChanged()
                            local settings = Group.Settings

                            parent.class:SetStretchMode( 1 )
                            parent.class:SetSize( settings.class_icon_size, settings.class_icon_size )

                            parent.leader:SetStretchMode( 1 )
                            parent.leader:SetSize( settings.leader_icon_size, settings.leader_icon_size )
                        end
                    end )

                    function ui:SizeChanged()
                        local w, h = self:GetSize()

                        local settings = Group.Settings

                        if settings.enable_classic_power_bar then
                            self.power:SetPos( 0, h - settings.power_size )
                            self.power:SetSize( w, settings.power_size )

                            self.morale:SetPos( 0, 0 )
                            self.morale:SetSize( w, h - settings.power_size )
                            self.morale:PositionChanged()
                        else
                            self.power:SetPos( 0, 0 )
                            self.power:SetSize( settings.power_size, h )

                            self.morale:SetPos( settings.power_size, 0 )
                            self.morale:SetSize( w - settings.power_size, h )
                            self.morale:PositionChanged()
                        end

                        main.Pos[ settings.Frame.Name.anchor ]( self.name, settings.Frame.Name.x, settings.Frame.Name.y, self.name:GetSize() )
                        main.Pos[ settings.Frame.Class.anchor ]( self.class, settings.Frame.Class.x, settings.Frame.Class.y, settings.class_icon_size, settings.class_icon_size )
                        main.Pos[ settings.Frame.Leader.anchor ]( self.leader, settings.Frame.Leader.x, settings.Frame.Leader.y, settings.leader_icon_size, settings.leader_icon_size )
                        main.Pos[ settings.Frame.Morale_Text.anchor ]( self.morale.text, settings.Frame.Morale_Text.x, settings.Frame.Morale_Text.y, self.morale.text:GetSize() )
                        main.Pos[ settings.Frame.Special_Effect.anchor ]( self.special, settings.Frame.Special_Effect.x, settings.Frame.Special_Effect.y, settings.special_effect_size, settings.special_effect_size )

                        self.power.text.Pos[ Group.Settings.Frame.Power_Text.anchor ]( self.power.text, Group.Settings.Frame.Power_Text.x, Group.Settings.Frame.Power_Text.y, self.power.text:GetSize() )

                        self:EffectLayout()
                    end

                    function ui:EffectLayout()
                        local count = 1

                        local settings = Group.Settings

                        local row_x = settings.effect_row_count
                        local row_y = math.ceil( main.effect_count / settings.effect_row_count )

                        local w, h = self.morale:GetSize()

                        for y = 1, row_y do
                            for x = 1, row_x do
                                local offset_x, offset_y = self.effect[ count ].offset[ settings.Frame.Effect.anchor ]( x, y, w, h, settings.Frame.Effect.x, settings.Frame.Effect.y )

                                self.effect[ count ]:SetStretchMode( 1 )
                                self.effect[ count ]:SetSize( settings.effect_size, settings.effect_size )

                                self.effect[ count ]:SetPos( offset_x, offset_y )
                                
                                count = count + 1
                
                                if count > main.effect_count then return end
                            end
                        end
                    end

                    ui:EffectLayout()
                end )
            end )

            local tabs = main.tabs

            for i = 1, #tabs do
                main.window[ tabs[ i ] ] = UI:Create( "LIST", ui, function( ui )
                    ui:SetPos( 256 + 32 + 32 + 2 + 16, 16 + 2 )
                    ui:SetSize( 1200 - 4 - 256 - 32 - 32 - 32 - 2, 800 - 30 - 32 - 4 - 44 )
                    ui:SetColor( 0, 0, 0, 0 )
                    --ui:SetColor( ( 255 / #tabs ) * i, 100, 100 )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                    ui:SetVisible( false )
                    --ui:SetMouseVisible( false )
                    

                    local w, h = ui:GetSize()

                    --Group:Local( "TEST" )

                    --[[
                        Help    1
                        Misc    2
                        Font    3
                        Class   4
                        Color   5
                        Frame   6
                        Layout  7
                    ]]

                    if tabs[ i ] == Group:Local( "Options_Tab_1" ) then
                        UI:Create( "SCROLLBOX", ui, function( ui )
                            ui:SetSize( w, h )
                            ui:SizeChanged()

                            w, h = ui.list:GetSize()

                            ui:AddItem(
                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end

                                    local text = Group:Local( "Options_Help" )

                                    ui:SetText( text )

                                    ui:SetForeColor( Color( 200, 200, 200 ) )

                                    --ui:SetColor( 100, 0, 0, 100 )

                                    
                                    local size = 16 * 81

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        size = 16 * 83
                                    end

                                    ui:SetSize( w, size )
                                end )
                            )

                            for i = 1, 50 do
                                ui:AddFiller( 0, 0 )
                            end
                        end )
                    elseif tabs[ i ] == Group:Local( "Options_Tab_2" ) then
                        UI:Create( "SCROLLBOX", ui, function( ui )
                            ui:SetSize( w, h )
                            ui:SizeChanged()

                            w, h = ui.list:GetSize()

                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Classic_Power" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_classic_power_bar )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_classic_power_bar = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame:SizeChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ]:SizeChanged()
                                            end
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Class Icon
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Class_Icon" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_class_icon )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_class_icon = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].class:SetVisible( checked )
                                            end

                                            main.Frame.class:SetVisible( checked )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Leader Icon
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Leader_Icon" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_leader_icon )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_leader_icon = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            if Group.Leader then
                                                Group.Party[ Group.PartyName[ Group.Leader ] ]:SetLeader( checked )
                                            end

                                            main.Frame.leader:SetVisible( checked )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Player Name
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Name_Text" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_player_name )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_player_name = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetVisible( checked )
                                            end

                                            main.Frame.name:SetVisible( checked )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Power Text
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Power_Text" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_power_text )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_power_text = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetVisible( checked )
                                                Group.Party[ i ]:Init()
                                            end

                                            main.Frame.power.text:SetVisible( checked )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Morale Text
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Morale_Text" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_morale_text )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_morale_text = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetVisible( checked )
                                                Group.Party[ i ]:Init()
                                            end

                                            main.Frame.morale.text:SetVisible( checked )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Effect Duration Text
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Effect_Duration" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_effect_duration )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_effect_duration = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetVisible( checked )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    if not Group.Party[ i ].effect[ k ].static then
                                                        Group.Party[ i ].effect[ k ].text:SetVisible( checked )
                                                    end
                                                end
                                            end
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Special Effect Duration Text
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Special_Duration" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_special_effect_duration )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()

                                            Group.Settings.enable_special_effect_duration = checked

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.special_text:SetVisible( checked )

                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].special.active then
                                                    Group.Party[ i ].special_text:SetVisible( checked )
                                                end
                                            end
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Unique Class Morale Color
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Class_Morale_Color" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_unique_class_morale_color )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()
                    
                                            Group.Settings.enable_unique_class_morale_color = checked

                                            if checked and not Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( Group.Settings.Color.Class[ Player:GetClass() ] )
                                            else
                                                if Group.Settings.enable_unique_player_morale_color then
                                                    main.Frame.morale:SetColor( Group.Settings.Color.Unique_Morale )
                                                else
                                                    main.Frame.morale:SetColor( Group.Settings.Color.Morale )
                                                end
                                            end
                    
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Unique Player Name Color
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Unique_Name_Color" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_unique_player_name_color )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()
                    
                                            Group.Settings.enable_unique_player_name_color = checked

                                            if checked then
                                                main.Frame.name:SetForeColor( Group.Settings.Color.Unique_Name )
                                                main.Frame.name:SetOutlineColor( Group.Settings.Color.Unique_Name_Outline )
                                            else
                                                main.Frame.name:SetForeColor( Group.Settings.Color.Name )
                                                main.Frame.name:SetOutlineColor( Group.Settings.Color.Name_Outline )
                                            end

                                            if checked then
                                                for i = 1, #Group.Party do
                                                    if Group.Party[ i ].localplayer then
                                                        Group.Party[ i ].name:SetForeColor( Group.Settings.Color.Unique_Name )
                                                        Group.Party[ i ].name:SetOutlineColor( Group.Settings.Color.Unique_Name_Outline )

                                                        break
                                                    end
                                                end
                                            else
                                                for i = 1, #Group.Party do
                                                    if Group.Party[ i ].localplayer then
                                                        Group.Party[ i ].name:SetForeColor( Group.Settings.Color.Name )
                                                        Group.Party[ i ].name:SetOutlineColor( Group.Settings.Color.Name_Outline )

                                                        break
                                                    end
                                                end
                                            end
                    
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Enable Unique Player Morale Color
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( w / 2, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Unique_Morale_Color" ) )
                                    end )

                                    UI:Create( "CHECKBOX", ui, function( ui )
                                        ui:SetSize( 16, 16 )
                                        ui:SetPos( w / 2 - 16, 0 )
                                        ui:SetChecked( Group.Settings.enable_unique_player_morale_color )

                                        function ui:CheckedChanged()
                                            local checked = self:IsChecked()
                    
                                            Group.Settings.enable_unique_player_morale_color = checked

                                            if checked then
                                                main.Frame.morale:SetColor( Group.Settings.Color.Unique_Morale )
                                            else
                                                if Group.Settings.enable_unique_class_morale_color then
                                                    main.Frame.morale:SetColor( Group.Settings.Color.Class[ Player:GetClass() ] )
                                                else
                                                    main.Frame.morale:SetColor( Group.Settings.Color.Morale )
                                                end
                                            end

                                            if checked then
                                                main.select_unique_name:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                                main.select_name:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                                main.select_special_effect:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                                main.select_unique_morale_text:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                            else
                                                main.select_unique_name:SetLayerColor( Group.Default_Settings.Color.Morale )
                                                main.select_name:SetLayerColor( Group.Default_Settings.Color.Morale )
                                                main.select_special_effect:SetLayerColor( Group.Default_Settings.Color.Morale )
                                                main.select_unique_morale_text:SetLayerColor( Group.Default_Settings.Color.Morale )
                                            end
                    
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Morale Threshold
                            ui:AddItem(
                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end

                                    ui:SetText( Group:Local( "Options_Misc_Morale_Threshold" ) )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            --Morale Threshold
                            ui:AddItem(
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( w / 2 )
                                        ui:SetKey( "" )
                                        ui:SetRange( 0, 100 )
                                        ui:SetValue( 100 * Group.Settings.morale_threshold )
                                        ui:SetKeySize( 0 )

                                        function ui:ValueChanged( value )
                                            Group.Settings.morale_threshold = value / 100

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end
                        
                                        function ui:SliderChanged( value )
                                            Group.Settings.morale_threshold = value / 100
                                        end

                                        function ui:Reset()
                                            self:SetValue( 100 * Group.Default_Settings.morale_threshold )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Load Plugin Preset
                            ui:AddItem( 
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 24 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( 256 - 64, 24 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetSize( 256, 24 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Load" ) )

                                        main.load = UI:Create( "BOX", main, function( ui )
                                            ui:SetPos( 1200 / 2 - 600 / 2, 800 / 2 - ( 16 * 4 + 14 ) / 2 )
                                            ui:SetSize( 600, 16 * 4 + 14 )
                                            ui:SetColor( 25, 25, 25, 225 )
                                            ui:SetMouseVisible( false )
                                            ui:SetVisible( false )
                                
                                            UI:Create( "BORDER", ui, function( ui )
                                                ui:Fill()
                                                ui:SetBorderSize( 2 )
                                                ui:SetBorderColor( Color( 100, 100, 100 ) )
                                            end )

                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetPos( 0, 16 )
                                                ui:SetSize( 600, 14 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )

                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetTextAlignment( 5 )
                                                ui:SetForeColor( Color( 200, 100, 100 ) )

                                                ui:SetText( Group:Local( "Options_Misc_Load_Confirmation" ) )
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui )
                                                ui:SetPos( 128 - 32 + 100, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )

                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_Yes" ) )
                                    
                                                function ui:Pressed()
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Global ) )
                                                    Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                                                end
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui, parent )
                                                ui:SetPos( 400 - 128 - 32 + 100, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_No" ) )
                                    
                                                function ui:Pressed()
                                                    parent:SetVisible( false )
                                                end
                                            end )

                                        end )
                            
                                        function ui:Pressed()
                                            main.load:SetVisible( true )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Save Plugin Preset
                            ui:AddItem( 
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 24 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( 256 - 64, 24 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetSize( 256, 24 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Save" ) )

                                        main.save = UI:Create( "BOX", main, function( ui )
                                            ui:SetPos( 1200 / 2 - 600 / 2, 800 / 2 - ( 16 * 4 + 14 ) / 2 )
                                            ui:SetSize( 600, 16 * 4 + 14 )
                                            ui:SetColor( 25, 25, 25, 225 )
                                            ui:SetMouseVisible( false )
                                            ui:SetVisible( false )
                                
                                            UI:Create( "BORDER", ui, function( ui )
                                                ui:Fill()
                                                ui:SetBorderSize( 2 )
                                                ui:SetBorderColor( Color( 100, 100, 100 ) )
                                            end )

                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetPos( 0, 16 )
                                                ui:SetSize( 600, 14 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetTextAlignment( 5 )
                                                ui:SetForeColor( Color( 200, 100, 100 ) )
                                                ui:SetText( Group:Local( "Options_Misc_Save_Confirmation" ) )
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui )
                                                ui:SetPos( 128 - 32 + 100, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_Yes" ) )
                                    
                                                function ui:Pressed()
                                                    Turbine.PluginData.Save( Turbine.DataScope.Account, "MiniRaid_Settings_Global", table.save( Group.Settings ) )

                                                    main.save:SetVisible( false )
                                                end
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui, parent )
                                                ui:SetPos( 400 - 128 - 32 + 100, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_No" ) )
                                    
                                                function ui:Pressed()
                                                    parent:SetVisible( false )
                                                end
                                            end )

                                        end )
                            
                                        function ui:Pressed()
                                            main.save:SetVisible( true )
                                        end
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Divider
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            --Plugin Reset
                            ui:AddItem( 
                                UI:Create( "BOX", ui, function( ui )
                                    ui:SetSize( w, 24 )
                                    ui:SetColor( 0, 100, 0, 0 )
                                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                    UI:Create( "BUTTON", ui, function( ui )
                                        ui:SetSize( 256 - 64, 24 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end

                                        ui:SetText( Group:Local( "Options_Misc_Reset" ) )

                                        main.reset = UI:Create( "BOX", main, function( ui )
                                            ui:SetPos( 1200 / 2 - 400 / 2, 800 / 2 - ( 16 * 4 + 14 ) / 2 )
                                            ui:SetSize( 400, 16 * 4 + 14 )
                                            ui:SetColor( 25, 25, 25, 225 )
                                            ui:SetMouseVisible( false )
                                            ui:SetVisible( false )
                                
                                            UI:Create( "BORDER", ui, function( ui )
                                                ui:Fill()
                                                ui:SetBorderSize( 2 )
                                                ui:SetBorderColor( Color( 100, 100, 100 ) )
                                            end )

                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetPos( 0, 16 )
                                                ui:SetSize( 400, 14 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetTextAlignment( 5 )
                                                ui:SetForeColor( Color( 200, 100, 100 ) )
                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation" ) )
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui )
                                                ui:SetPos( 128 - 32, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_Yes" ) )
                                    
                                                function ui:Pressed()
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Default_Settings ) )
                                                    Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                                                end
                                            end )

                                            UI:Create( "BUTTON", ui, function( ui, parent )
                                                ui:SetPos( 400 - 128 - 32, 16 + 14 + 16 )
                                                ui:SetSize( 64, 16 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                                end

                                                ui:SetText( Group:Local( "Options_Misc_Reset_Confirmation_No" ) )
                                    
                                                function ui:Pressed()
                                                    parent:SetVisible( false )
                                                end
                                            end )

                                        end )
                            
                                        function ui:Pressed()
                                            main.reset:SetVisible( true )
                                        end
                                    end )

                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetPos( 256 + 16 - 64 )
                                        ui:SetSize( w, 24 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                        end

                                        ui:SetTextAlignment( 4 )
                                        ui:SetForeColor( Color( 200, 100, 100 ) )
                                        ui:SetText( Group:Local( "Options_Misc_Reset_Description" ) )
                                    end )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                -- Version
                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end

                                    ui:SetTextAlignment( 9 )
                                    ui:SetText( Group.version )
                                end )
                            )

                            ui.thumb:SetVisible( false )
                            ui.scroll.background:SetVisible( false )
                        end )
                    elseif tabs[ i ] == Group:Local( "Options_Tab_3" ) then
                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( 400, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end
                                    
                                ui:SetText( Group:Local( "Options_Font_Name" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        --Player Name
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetWidth( 400 )
                                ui:SetColor( 0, 0, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui:AddItem(
                                    UI:Create( "SLIDER", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font )
                                        ui:SetValueAlignment( 6 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetValue( Group.Settings.player_font_type, true )
                                        ui:SetKey( Group:Local( "Options_Font_Type" ) )
                                        ui:SetKeySize( 32 )
                                        ui:SetValueSize( 144 )
                                        ui:SetText( Group.Font[ Group.Settings.player_font_type ].name )
                                        ui:DisableValueEdit( true )

                                        main.slider_player_name_type = ui
                        
                                        function ui:ValueChanged( value )
                                            Group.Settings.player_font_type = value
                                            Group.Settings.player_font_size = 1
                        
                                            self:SetText( Group.Font[ value ].name )
                        
                                            main.slider_player_name_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_player_name_size:SetValue( 1, true )
                                            main.slider_player_name_size:SetText( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )

                                            main.Frame.name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                            main.Frame.name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                            main.Frame.name:SetText( main.Frame.name:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                                Group.Party[ i ].name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                                Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                            end
                        
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end
                        
                                        function ui:SliderChanged( value )
                                            Group.Settings.player_font_type = value
                                            Group.Settings.player_font_size = 1
                        
                                            self:SetText( Group.Font[ value ].name )
                        
                                            main.slider_player_name_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_player_name_size:SetValue( 1, true )
                                            main.slider_player_name_size:SetText( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )

                                            main.Frame.name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                            main.Frame.name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                            main.Frame.name:SetText( main.Frame.name:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                                Group.Party[ i ].name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                                Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font[ Group.Settings.player_font_type ].size )
                                        ui:SetValue( Group.Settings.player_font_size, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.slider.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetKeySize( 48 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Size" ) )
                                        ui:SetText( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                        ui:DisableValueEdit( true )

                                        main.slider_player_name_size = ui
                        
                                        function ui:ValueChanged( value )
                                            Group.Settings.player_font_size = value
                        
                                            self:SetText( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                        
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                            main.Frame.name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                            main.Frame.name:SetText( main.Frame.name:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                                Group.Party[ i ].name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                                Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                            end
                                        end
                        
                                        function ui:SliderChanged( value )
                                            Group.Settings.player_font_size = value
                        
                                            self:SetText( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )

                                            main.Frame.name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                            main.Frame.name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                            main.Frame.name:SetText( main.Frame.name:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                                Group.Party[ i ].name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                                Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                            end
                                        end

                                        function ui:Reset()
                                            Group.Settings.player_font_type = Group.Default_Settings.player_font_type
                                            Group.Settings.player_font_size = Group.Default_Settings.player_font_size

                                            main.slider_player_name_type:SetValue( Group.Default_Settings.player_font_type, true )
                                            main.slider_player_name_type:SetText( Group.Font[ Group.Default_Settings.player_font_type ].name )

                                            self:SetRange( 1, #Group.Font[ Group.Default_Settings.player_font_type ].size )
                                            self:SetValue( Group.Default_Settings.player_font_size, true )
                                            self:SetText( Group.Font[ Group.Default_Settings.player_font_type ].size[ Group.Default_Settings.player_font_size ] )

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                            main.Frame.name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                            main.Frame.name:SetText( main.Frame.name:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].name:SetHeight( Group.Font[ Group.Settings.player_font_type ].size[ Group.Settings.player_font_size ] )
                                                Group.Party[ i ].name:SetFont( Group:GetFont( Group.Settings.player_font_type, Group.Settings.player_font_size ) )
                                                Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                            end

                                            main.player_name_outline:SetChecked( Group.Default_Settings.player_font_outline )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( 400, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Font_Outline" ) )

                                        main.player_name_outline = UI:Create( "CHECKBOX", ui, function( ui )
                                            ui:SetSize( 16, 16 )
                                            ui:SetPos( 75, 0 )
                                            ui:SetChecked( Group.Settings.player_font_outline )
                            
                                            function ui:CheckedChanged()
                                                local checked = self:IsChecked()
                            
                                                Group.Settings.player_font_outline = checked
                            
                                                if checked then
                                                    main.Frame.name:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                else
                                                    main.Frame.name:SetFontStyle( Turbine.UI.FontStyle.None )
                                                end

                                                main.Frame.name:SetText( main.Frame.name:GetText() )
            
                                                for i = 1, #Group.Party do
                                                    if checked then
                                                        Group.Party[ i ].name:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                        Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                                    else
                                                        Group.Party[ i ].name:SetFontStyle( Turbine.UI.FontStyle.None )
                                                        Group.Party[ i ].name:SetText( Group.Party[ i ].player_name )
                                                    end
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end
                                        end )
                                    end )
                                )

                                ui:SetHeightToChildren()
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( 400, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Font_Morale" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        --Morale Text
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetWidth( 400 )
                                ui:SetColor( 0, 0, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui:AddItem(
                                    UI:Create( "SLIDER", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font )
                                        ui:SetValueAlignment( 6 )
                                        ui:SetValue( Group.Settings.morale_font_type, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Type" ) )
                                        ui:SetValueSize( 144 )
                                        ui:SetText( Group.Font[ Group.Settings.morale_font_type ].name )
                                        ui:DisableValueEdit( true )

                                        main.slider_morale_text_type = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.morale_font_type = value
                                            Group.Settings.morale_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_morale_text_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_morale_text_font_size:SetValue( 1, true )
                                            main.slider_morale_text_font_size:SetText( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )

                                            main.Frame.morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                            main.Frame.morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                            main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                                Group.Party[ i ].morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                                Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.morale_font_type = value
                                            Group.Settings.morale_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_morale_text_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_morale_text_font_size:SetValue( 1, true )
                                            main.slider_morale_text_font_size:SetText( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )

                                            main.Frame.morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                            main.Frame.morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                            main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                                Group.Party[ i ].morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                                Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font[ Group.Settings.morale_font_type ].size )
                                        ui:SetValue( Group.Settings.morale_font_size, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.slider.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetKeySize( 48 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Size" ) )
                                        ui:SetText( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                        ui:DisableValueEdit( true )

                                        main.slider_morale_text_font_size = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.morale_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )

                                            main.Frame.morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                            main.Frame.morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                            main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                                Group.Party[ i ].morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                                Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.morale_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )

                                            main.Frame.morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                            main.Frame.morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                            main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                                Group.Party[ i ].morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                                Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                            end
                                        end

                                        function ui:Reset()
                                            Group.Settings.morale_font_type = Group.Default_Settings.morale_font_type
                                            Group.Settings.morale_font_size = Group.Default_Settings.morale_font_size

                                            main.slider_morale_text_type:SetValue( Group.Default_Settings.morale_font_type, true )
                                            main.slider_morale_text_type:SetText( Group.Font[ Group.Default_Settings.morale_font_type ].name )

                                            self:SetRange( 1, #Group.Font[ Group.Default_Settings.morale_font_type ].size )
                                            self:SetValue( Group.Default_Settings.morale_font_size, true )
                                            self:SetText( Group.Font[ Group.Default_Settings.morale_font_type ].size[ Group.Default_Settings.morale_font_size ] )

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                            main.Frame.morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                            main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetHeight( Group.Font[ Group.Settings.morale_font_type ].size[ Group.Settings.morale_font_size ] )
                                                Group.Party[ i ].morale.text:SetFont( Group:GetFont( Group.Settings.morale_font_type, Group.Settings.morale_font_size ) )
                                                Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                            end

                                            main.morale_text_outline:SetChecked( Group.Default_Settings.morale_font_outline )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( 400, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Font_Outline" ) )

                                        main.morale_text_outline = UI:Create( "CHECKBOX", ui, function( ui )
                                            ui:SetSize( 16, 16 )
                                            ui:SetPos( 75 ,0 )
                                            ui:SetChecked( Group.Settings.morale_font_outline )
                            
                                            function ui:CheckedChanged()
                                                local checked = self:IsChecked()
                            
                                                Group.Settings.morale_font_outline = checked
                            
                                                if checked then
                                                    main.Frame.morale.text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                else
                                                    main.Frame.morale.text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                end

                                                main.Frame.morale.text:SetText( main.Frame.morale.text:GetText() )
                
                                                for i = 1, #Group.Party do
                                                    if checked then
                                                        Group.Party[ i ].morale.text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                    else
                                                        Group.Party[ i ].morale.text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                    end
                
                                                    Group.Party[ i ].morale.text:SetText( Group.Party[ i ].morale.text:GetText() )
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end
                                        end )
                                    end )
                                )

                                ui:SetHeightToChildren()
                            end )
                        )


                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( 400, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Font_Power" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        --Power Text
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetWidth( 400 )
                                ui:SetColor( 0, 0, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui:AddItem(
                                    UI:Create( "SLIDER", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font )
                                        ui:SetValueAlignment( 6 )
                                        ui:SetValue( Group.Settings.power_font_type, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Type" ) )
                                        ui:SetValueSize( 144 )
                                        ui:SetText( Group.Font[ Group.Settings.power_font_type ].name )
                                        ui:DisableValueEdit( true )

                                        main.slider_power_text_type = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.power_font_type = value
                                            Group.Settings.power_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_power_text_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_power_text_font_size:SetValue( 1, true )
                                            main.slider_power_text_font_size:SetText( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )

                                            main.Frame.power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                            main.Frame.power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                            main.Frame.power.text:SetText( main.Frame.power.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                                Group.Party[ i ].power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                                Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.power_font_type = value
                                            Group.Settings.power_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_power_text_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_power_text_font_size:SetValue( 1, true )
                                            main.slider_power_text_font_size:SetText( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )

                                            main.Frame.power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                            main.Frame.power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                            main.Frame.power.text:SetText( main.Frame.power.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                                Group.Party[ i ].power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                                Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font[ Group.Settings.power_font_type ].size )
                                        ui:SetValue( Group.Settings.power_font_size, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.slider.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetKeySize( 48 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Size" ) )
                                        ui:SetText( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                        ui:DisableValueEdit( true )

                                        main.slider_power_text_font_size = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.power_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )

                                            main.Frame.power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                            main.Frame.power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                            main.Frame.power.text:SetText( main.Frame.power.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                                Group.Party[ i ].power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                                Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.power_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )

                                            main.Frame.power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                            main.Frame.power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                            main.Frame.power.text:SetText( main.Frame.power.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                                Group.Party[ i ].power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                                Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                            end
                                        end

                                        function ui:Reset()
                                            Group.Settings.power_font_type = Group.Default_Settings.power_font_type
                                            Group.Settings.power_font_size = Group.Default_Settings.power_font_size

                                            main.slider_power_text_type:SetValue( Group.Default_Settings.power_font_type, true )
                                            main.slider_power_text_type:SetText( Group.Font[ Group.Default_Settings.power_font_type ].name )

                                            self:SetRange( 1, #Group.Font[ Group.Default_Settings.power_font_type ].size )
                                            self:SetValue( Group.Default_Settings.power_font_size, true )
                                            self:SetText( Group.Font[ Group.Default_Settings.power_font_type ].size[ Group.Default_Settings.power_font_size ] )

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                            main.Frame.power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                            main.Frame.power.text:SetText( main.Frame.power.text:GetText() )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].power.text:SetHeight( Group.Font[ Group.Settings.power_font_type ].size[ Group.Settings.power_font_size ] )
                                                Group.Party[ i ].power.text:SetFont( Group:GetFont( Group.Settings.power_font_type, Group.Settings.power_font_size ) )
                                                Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                            end

                                            main.power_text_outline:SetChecked( Group.Default_Settings.power_font_outline )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( 400, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Font_Outline" ) )

                                        main.power_text_outline = UI:Create( "CHECKBOX", ui, function( ui )
                                            ui:SetSize( 16, 16 )
                                            ui:SetPos( 75 ,0 )
                                            ui:SetChecked( Group.Settings.power_font_outline )
                            
                                            function ui:CheckedChanged()
                                                local checked = self:IsChecked()
                            
                                                Group.Settings.power_font_outline = checked
                            
                                                if checked then
                                                    main.Frame.power.text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                else
                                                    main.Frame.power.text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                end

                                                main.Frame.power.text:SetText( main.Frame.power.text:GetText() )
                
                                                for i = 1, #Group.Party do
                                                    if checked then
                                                        Group.Party[ i ].power.text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                    else
                                                        Group.Party[ i ].power.text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                    end
                
                                                    Group.Party[ i ].power.text:SetText( Group.Party[ i ].power.text:GetText() )
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end
                                        end )
                                    end )
                                )

                                ui:SetHeightToChildren()
                            end )
                        )


                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( 400, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Font_Effect" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        --Effect Duration
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( 400, 64 + 16 + UI.Spacing )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui:AddItem(
                                    UI:Create( "SLIDER", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font )
                                        ui:SetValueAlignment( 6 )
                                        ui:SetValue( Group.Settings.effect_font_type, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Type" ) )
                                        ui:SetValueSize( 144 )
                                        ui:SetText( Group.Font[ Group.Settings.effect_font_type ].name )
                                        ui:DisableValueEdit( true )

                                        main.slider_effect_font_type = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.effect_font_type = value
                                            Group.Settings.effect_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_effect_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_effect_font_size:SetValue( 1, true )
                                            main.slider_effect_font_size:SetText( Group.Font[ Group.Settings.effect_font_type ].size[ Group.Settings.effect_font_size ] )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    Group.Party[ i ].effect[ k ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                end
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.effect_font_type = value
                                            Group.Settings.effect_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_effect_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_effect_font_size:SetValue( 1, true )
                                            main.slider_effect_font_size:SetText( Group.Font[ Group.Settings.effect_font_type ].size[ Group.Settings.effect_font_size ] )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    Group.Party[ i ].effect[ k ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font[ Group.Settings.effect_font_type ].size )
                                        ui:SetValue( Group.Settings.effect_font_size, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.slider.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetKeySize( 48 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Size" ) )
                                        ui:SetText( Group.Font[ Group.Settings.effect_font_type ].size[ Group.Settings.effect_font_size ] )
                                        ui:DisableValueEdit( true )

                                        main.slider_effect_font_size = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.effect_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.effect_font_type ].size[ Group.Settings.effect_font_size ] )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    Group.Party[ i ].effect[ k ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                end
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.effect_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.effect_font_type ].size[ Group.Settings.effect_font_size ] )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    Group.Party[ i ].effect[ k ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                end
                                            end
                                        end

                                        function ui:Reset()
                                            Group.Settings.effect_font_type = Group.Default_Settings.effect_font_type
                                            Group.Settings.effect_font_size = Group.Default_Settings.effect_font_size

                                            main.slider_effect_font_type:SetValue( Group.Default_Settings.effect_font_type, true )
                                            main.slider_effect_font_type:SetText( Group.Font[ Group.Default_Settings.effect_font_type ].name )

                                            self:SetRange( 1, #Group.Font[ Group.Default_Settings.effect_font_type ].size )
                                            self:SetValue( Group.Default_Settings.effect_font_size, true )
                                            self:SetText( Group.Font[ Group.Default_Settings.effect_font_type ].size[ Group.Default_Settings.effect_font_size ] )

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #main.Frame.effect do
                                                main.Frame.effect[ i ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                            end

                                            for i = 1, #Group.Party do
                                                for k = 1, #Group.Party[ i ].effect do
                                                    Group.Party[ i ].effect[ k ].text:SetFont( Group:GetFont( Group.Settings.effect_font_type, Group.Settings.effect_font_size ) )
                                                end
                                            end

                                            main.effect_outline:SetChecked( Group.Default_Settings.effect_font_outline )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( 400, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Font_Outline" ) )

                                        main.effect_outline = UI:Create( "CHECKBOX", ui, function( ui )
                                            ui:SetSize( 16, 16 )
                                            ui:SetPos( 75, 0 )
                                            ui:SetChecked( Group.Settings.effect_font_outline )
                            
                                            function ui:CheckedChanged()
                                                local checked = self:IsChecked()
                            
                                                Group.Settings.effect_font_outline = checked
                                                
                                                for i = 1, #main.Frame.effect do
                                                    if checked then
                                                        main.Frame.effect[ i ].text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                    else
                                                        main.Frame.effect[ i ].text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                    end

                                                    main.Frame.effect[ i ].text:SetText( main.Frame.effect[ i ].text:GetText() )
                                                end
            
                                                for i = 1, #Group.Party do
                                                    for k = 1, #Group.Party[ i ].effect do
                                                        if checked then
                                                            Group.Party[ i ].effect[ k ].text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                        else
                                                            Group.Party[ i ].effect[ k ].text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                        end
                                                    end
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end
                                        end )
                                    end )
                                )

                                ui:SetHeightToChildren()
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( 400, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Font_Special" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        --Special Effect Duration
                        ui:AddItem(
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetWidth( 400 )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                ui:AddItem(
                                    UI:Create( "SLIDER", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font )
                                        ui:SetValueAlignment( 6 )
                                        ui:SetValue( Group.Settings.special_effect_font_type, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Type" ) )
                                        ui:SetValueSize( 144 )
                                        ui:SetText( Group.Font[ Group.Settings.special_effect_font_type ].name )
                                        ui:DisableValueEdit( true )

                                        main.slider_special_effect_font_type = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.special_effect_font_type = value
                                            Group.Settings.special_effect_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_special_effect_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_special_effect_font_size:SetValue( 1, true )
                                            main.slider_special_effect_font_size:SetText( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )

                                            main.Frame.special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                            main.Frame.special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )
                                            main.Frame.special_text:SetText( main.Frame.special_text:GetText() )

                                            main.Frame.special:PositionChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                                Group.Party[ i ].special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )

                                                Group.Party[ i ].special:PositionChanged()
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.special_effect_font_type = value
                                            Group.Settings.special_effect_font_size = 1

                                            self:SetText( Group.Font[ value ].name )

                                            main.slider_special_effect_font_size:SetRange( 1, #Group.Font[ value ].size )
                                            main.slider_special_effect_font_size:SetValue( 1, true )
                                            main.slider_special_effect_font_size:SetText( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )

                                            main.Frame.special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                            main.Frame.special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )
                                            main.Frame.special_text:SetText( main.Frame.special_text:GetText() )

                                            main.Frame.special:PositionChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                                Group.Party[ i ].special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )

                                                Group.Party[ i ].special:PositionChanged()
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( 400 )
                                        ui:SetRange( 1, #Group.Font[ Group.Settings.special_effect_font_type ].size )
                                        ui:SetValue( Group.Settings.special_effect_font_size, true )
                                        ui:SetKeySize( 32 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui.slider.Key:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            ui:SetKeySize( 48 )
                                        end
                                        
                                        ui:SetKey( Group:Local( "Options_Font_Size" ) )
                                        ui:SetText( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                        ui:DisableValueEdit( true )

                                        main.slider_special_effect_font_size = ui

                                        function ui:ValueChanged( value )
                                            Group.Settings.special_effect_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )

                                            main.Frame.special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                            main.Frame.special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )
                                            main.Frame.special_text:SetText( main.Frame.special_text:GetText() )

                                            main.Frame.special:PositionChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                                Group.Party[ i ].special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )

                                                Group.Party[ i ].special:PositionChanged()
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.special_effect_font_size = value

                                            self:SetText( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )

                                            main.Frame.special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                            main.Frame.special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )
                                            main.Frame.special_text:SetText( main.Frame.special_text:GetText() )

                                            main.Frame.special:PositionChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                                Group.Party[ i ].special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )

                                                Group.Party[ i ].special:PositionChanged()
                                            end
                                        end

                                        function ui:Reset()
                                            Group.Settings.special_effect_font_type = Group.Default_Settings.special_effect_font_type
                                            Group.Settings.special_effect_font_size = Group.Default_Settings.special_effect_font_size

                                            main.slider_special_effect_font_type:SetValue( Group.Default_Settings.special_effect_font_type, true )
                                            main.slider_special_effect_font_type:SetText( Group.Font[ Group.Default_Settings.special_effect_font_type ].name )

                                            self:SetRange( 1, #Group.Font[ Group.Default_Settings.special_effect_font_type ].size )
                                            self:SetValue( Group.Default_Settings.special_effect_font_size, true )
                                            self:SetText( Group.Font[ Group.Default_Settings.special_effect_font_type ].size[ Group.Default_Settings.special_effect_font_size ] )

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            main.Frame.special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                            main.Frame.special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )
                                            main.Frame.special_text:SetText( main.Frame.special_text:GetText() )

                                            main.Frame.special:PositionChanged()

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetHeight( Group.Font[ Group.Settings.special_effect_font_type ].size[ Group.Settings.special_effect_font_size ] )
                                                Group.Party[ i ].special_text:SetFont( Group:GetFont( Group.Settings.special_effect_font_type, Group.Settings.special_effect_font_size ) )

                                                Group.Party[ i ].special:PositionChanged()
                                            end

                                            main.special_effect_outline:SetChecked( Group.Default_Settings.special_effect_font_outline )
                                        end
                                    end )
                                )

                                ui:AddFiller( 0, UI.Spacing )

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( 400, 16 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Font_Outline" ) )

                                        main.special_effect_outline = UI:Create( "CHECKBOX", ui, function( ui )
                                            ui:SetSize( 16, 16 )
                                            ui:SetPos( 75, 0 )
                                            ui:SetChecked( Group.Settings.special_effect_font_outline )
                            
                                            function ui:CheckedChanged()
                                                local checked = self:IsChecked()
                            
                                                Group.Settings.special_effect_font_outline = checked
                            
                                                if checked then
                                                    main.Frame.special_text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                else
                                                    main.Frame.special_text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                end

                                                main.Frame.special_text:SetText( main.Frame.special_text:GetText() )
            
                                                for i = 1, #Group.Party do
                                                    if checked then
                                                        Group.Party[ i ].special_text:SetFontStyle( Turbine.UI.FontStyle.Outline )
                                                    else
                                                        Group.Party[ i ].special_text:SetFontStyle( Turbine.UI.FontStyle.None )
                                                    end
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end
                                        end )
                                    end )
                                )

                                ui:SetHeightToChildren()
                            end )
                        )
                    elseif tabs[ i ] == Group:Local( "Options_Tab_4" ) then
                        local tbl = {
                            "Hunter",
                            "Warden",
                            "Burglar",
                            "Captain",
                            "Champion",
                            "Guardian",
                            "Minstrel",
                            "Beorning",
                            "LoreMaster",
                            "RuneKeeper",
                            "Brawler"
                        }

                        main.select_unique_class_morale = {}

                        for i = 1, 10, 3 do
                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Class_" .. tbl[ i ] ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 1 ] then
                                        ui:AddItem(
                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                end
                                                
                                                ui:SetText( Group:Local( "Options_Class_" .. tbl[ i + 1 ] ) )
                                            end )
                                        )
                                    end

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 2 ] then
                                        ui:AddItem(
                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                end
                                                
                                                ui:SetText( Group:Local( "Options_Class_" .. tbl[ i + 2 ] ) )
                                            end )
                                        )
                                    end
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Color
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 76 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    if tbl[ i ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 1 ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i + 1 ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 1 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 1 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end 

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 2 ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i + 2 ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 2 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 2 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end
                                end )
                            )

                            ui:AddFiller( 0, 16 )
                        end

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        local tbl = {
                            "Weaver",
                            "Reaver",
                            "Defiler",
                            "Stalker",
                            "WarLeader",
                            "BlackArrow"
                        }
                        
                        for i = 1, 6, 3 do
                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Class_" .. tbl[ i ] ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 1 ] then
                                        ui:AddItem(
                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                end
                                                
                                                ui:SetText( Group:Local( "Options_Class_" .. tbl[ i + 1 ] ) )
                                            end )
                                        )
                                    end

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 2 ] then
                                        ui:AddItem(
                                            UI:Create( "TEXT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
                                                
                                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                end
                                                
                                                ui:SetText( Group:Local( "Options_Class_" .. tbl[ i + 2 ] ) )
                                            end )
                                        )
                                    end
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Color
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 76 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    if tbl[ i ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 1 ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i + 1 ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 1 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 1 ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 1 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end 

                                    ui:AddFiller( 16, 0 )

                                    if tbl[ i + 2 ] then
                                        ui:AddItem(
                                            UI:Create( "COLOR_SELECT", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetInternalColor( Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] )
                                                ui:SetDefaultColor( Group.Default_Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] )
                                                ui:SetBackgroundColor( Group.Settings.Color.Background )
                        
                                                main.select_unique_class_morale[ tbl[ i + 1 ] ] = ui
                        
                                                function ui:ColorChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] = color
                        
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 2 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                        
                                                function ui:ColorSliderChanged( color )
                                                    Group.Settings.Color.Class[ Turbine.Gameplay.Class[ tbl[ i + 2 ] ] ] = color
                        
                                                    if not Group.Settings.enable_unique_player_morale_color and Group.Settings.enable_unique_class_morale_color then
                                                        if Turbine.Gameplay.Class[ tbl[ i + 2 ] ] == Player:GetClass() then
                                                            main.Frame.morale:SetColor( color )
                                                        end
                                                    end
                                                end
                                            end )
                                        )
                                    end
                                end )
                            )

                            ui:AddFiller( 0, 16 )
                        end
                    elseif tabs[ i ] == Group:Local( "Options_Tab_5" ) then
                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Morale
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Morale" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Power
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Power" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Bubble
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Bubble" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Morale
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Morale )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Morale )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        main.select_morale = ui
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Morale = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            if not Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end
                
                                            main.select_name:SetLayerColor( color )
                
                                            if not Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Morale = color
                
                                            if not Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end
                
                                            main.select_name:SetLayerColor( color )
                
                                            if not Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Power
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Power )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Power )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        main.select_power = ui
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Power = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            
                                            if Player:GetClass() ~= Turbine.Gameplay.Class.Beorning and Player:GetClass() ~= Turbine.Gameplay.Class.Brawler then
                                                main.Frame.power:SetColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].player_class ~= Turbine.Gameplay.Class.Beorning and Group.Party[ i ].player_class ~= Turbine.Gameplay.Class.Brawler then
                                                    Group.Party[ i ].power:SetColor( color )
                                                end
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Power = color
                
                                            if Player:GetClass() ~= Turbine.Gameplay.Class.Beorning and Player:GetClass() ~= Turbine.Gameplay.Class.Brawler then
                                                main.Frame.power:SetColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].player_class ~= Turbine.Gameplay.Class.Beorning and Group.Party[ i ].player_class ~= Turbine.Gameplay.Class.Brawler then
                                                    Group.Party[ i ].power:SetColor( color )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Bubble
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Bubble )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Bubble )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        main.select_bubble = ui
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Bubble = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].bubble:SetColor( color )
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Bubble = color
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].bubble:SetColor( color )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Defeated
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Dead" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Background
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Background" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Threshold
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Threshold" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Defeated
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Dead )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Dead )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_dead = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Dead = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].dead:SetColor( color )
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Dead = color

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].dead:SetColor( color )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Background
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Background )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Background )
                                        ui:SetBackgroundColor( 0, 0, 0, 0 )

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Background = color

                                            main.Frame:SetColor( color )

                                            main.select_morale:SetBackgroundColor( color )
                                            main.select_power:SetBackgroundColor( color )
                                            main.select_bubble:SetBackgroundColor( color )
                                            main.select_dead:SetBackgroundColor( color )
                                            main.select_threshold:SetBackgroundColor( color )
                                            main.select_select:SetBackgroundColor( color )
                                            main.select_hover:SetBackgroundColor( color )
                                            main.select_name:SetBackgroundColor( color )
                                            main.select_unique_name:SetBackgroundColor( color )
                                            main.select_unique_morale:SetBackgroundColor( color )
                                            main.select_unique_morale_text:SetBackgroundColor( color )
                                            main.select_special_effect:SetBackgroundColor( color )
                                            main.select_wrath:SetBackgroundColor( color )

                                            for k, v in pairs( main.select_unique_class_morale ) do
                                                v:SetBackgroundColor( color )
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ]:SetColor( color )
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Background = color

                                            main.Frame:SetColor( color )

                                            main.select_morale:SetBackgroundColor( color )
                                            main.select_power:SetBackgroundColor( color )
                                            main.select_bubble:SetBackgroundColor( color )
                                            main.select_dead:SetBackgroundColor( color )
                                            main.select_threshold:SetBackgroundColor( color )
                                            main.select_select:SetBackgroundColor( color )
                                            main.select_hover:SetBackgroundColor( color )
                                            main.select_name:SetBackgroundColor( color )
                                            main.select_unique_name:SetBackgroundColor( color )
                                            main.select_unique_morale:SetBackgroundColor( color )
                                            main.select_unique_morale_text:SetBackgroundColor( color )
                                            main.select_special_effect:SetBackgroundColor( color )
                                            main.select_wrath:SetBackgroundColor( color )

                                            for k, v in pairs( main.select_unique_class_morale ) do
                                                v:SetBackgroundColor( color )
                                            end

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ]:SetColor( color )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Threshold
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Threshold )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Threshold )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_threshold = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Threshold = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Threshold = color
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Selected
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Selected" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Hover
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Hover" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Morale
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Unique_Morale" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Select
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Select )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Select )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_select = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Select = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].selection.border:SetBorderColor( color )
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Select = color

                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].selection.border:SetBorderColor( color )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Hover
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Hover )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Hover )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_hover = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Hover = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Hover = color
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Morale
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Unique_Morale )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Unique_Morale )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_unique_morale = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Unique_Morale = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Unique_Morale = color

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Wrath
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Wrath" ) )
                                    end )
                                )

                                --[[ ui:AddFiller( 16, 0 )

                                --Hover
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( "Hover" )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Morale
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( "Unique Morale" )
                                    end )
                                ) ]]
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Wrath
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Wrath )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Wrath )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_wrath = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Wrath = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            if Player:GetClass() == Turbine.Gameplay.Class.Beorning or Player:GetClass() == Turbine.Gameplay.Class.Brawler then
                                                main.Frame.power:SetColor( color )
                                            end

                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].player_class == Turbine.Gameplay.Class.Beorning or Group.Party[ i ].player_class == Turbine.Gameplay.Class.Brawler then
                                                    Group.Party[ i ].power:SetColor( color )
                                                end
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Wrath = color

                                            if Player:GetClass() == Turbine.Gameplay.Class.Beorning or Player:GetClass() == Turbine.Gameplay.Class.Brawler then
                                                main.Frame.power:SetColor( color )
                                            end

                                            for i = 1, #Group.Party do
                                                if Group.Party[ i ].player_class == Turbine.Gameplay.Class.Beorning or Group.Party[ i ].player_class == Turbine.Gameplay.Class.Brawler then
                                                    Group.Party[ i ].power:SetColor( color )
                                                end
                                            end
                                        end
                                    end )
                                )

                                --[[ ui:AddFiller( 16, 0 )

                                --Hover
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Hover )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Hover )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_hover = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Hover = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Hover = color
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Morale
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Unique_Morale )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Unique_Morale )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        main.select_unique_morale = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Unique_Morale = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Unique_Morale = color

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.Frame.morale:SetColor( color )
                                            end

                                            if Group.Settings.enable_unique_player_morale_color then
                                                main.select_unique_name:SetLayerColor( color )
                                                main.select_name:SetLayerColor( color )
                                                main.select_special_effect:SetLayerColor( color )
                                                main.select_unique_morale_text:SetLayerColor( color )
                                            end
                                        end
                                    end )
                                ) ]]
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Name Font
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Name" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Name Font
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Unique_Name" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Morale Font
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Morale" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Name Font
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Name )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Name )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        if Group.Settings.enable_unique_player_morale_color then
                                            ui:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                        else
                                            ui:SetLayerColor( Group.Default_Settings.Color.Morale )
                                        end
                
                                        ui:EnableLayer( true )
                
                                        main.select_name = ui
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Name = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            if not Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetForeColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if not ( Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer ) then
                                                    Group.Party[ i ].name:SetForeColor( color )
                                                end
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Name = color
                
                                            if not Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetForeColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if not ( Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer ) then
                                                    Group.Party[ i ].name:SetForeColor( color )
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Name Font
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Unique_Name )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Unique_Name )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )

                                        if Group.Settings.enable_unique_player_morale_color then
                                            ui:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                        else
                                            ui:SetLayerColor( Group.Default_Settings.Color.Morale )
                                        end

                                        ui:EnableLayer( true )

                                        main.select_unique_name = ui

                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Unique_Name = color

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                            if Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetForeColor( color )
                                            end

                                            for i = 1, #Group.Party do
                                                if Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer then
                                                    Group.Party[ i ].name:SetForeColor( color )

                                                    break
                                                end
                                            end
                                        end

                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Unique_Name = color

                                            if Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetForeColor( color )
                                            end

                                            for i = 1, #Group.Party do
                                                if Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer then
                                                    Group.Party[ i ].name:SetForeColor( color )

                                                    break
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Morale Font
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Morale_Text )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Morale_Text )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        main.select_unique_morale_text = ui
                
                                        if Group.Settings.enable_unique_player_morale_color then
                                            ui:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                        else
                                            ui:SetLayerColor( Group.Default_Settings.Color.Morale )
                                        end
                
                                        ui:EnableLayer( true )
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Morale_Text = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            main.Frame.morale.text:SetForeColor( color )
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetForeColor( color )
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Morale_Text = color
                
                                            main.Frame.morale.text:SetForeColor( color )
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].morale.text:SetForeColor( color )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Name Font Outline
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Name_Outline" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Name Font Outline
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Unique_Name_Outline" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Special Effect Font
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Color_Font_Special" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Color
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 76 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Name Font Outline
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Name_Outline )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Name_Outline )
                                        ui:SetBackgroundColor( 0, 0, 0, 0 )
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Name_Outline = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            if not Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetOutlineColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if not ( Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer ) then
                                                    Group.Party[ i ].name:SetOutlineColor( color )
                
                                                    break
                                                end
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Name_Outline = color
                
                                            if not Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetOutlineColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if not ( Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer ) then
                                                    Group.Party[ i ].name:SetOutlineColor( color )
                
                                                    break
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Unique Name Font Outline
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Unique_Name_Outline )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Unique_Name_Outline )
                                        ui:SetBackgroundColor( 0, 0, 0, 0 )
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Unique_Name_Outline = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            if Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetOutlineColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer then
                                                    Group.Party[ i ].name:SetOutlineColor( color )
                
                                                    break
                                                end
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Unique_Name_Outline = color
                
                                            if Group.Settings.enable_unique_player_name_color then
                                                main.Frame.name:SetOutlineColor( color )
                                            end
                
                                            for i = 1, #Group.Party do
                                                if Group.Settings.enable_unique_player_name_color and Group.Party[ i ].localplayer then
                                                    Group.Party[ i ].name:SetOutlineColor( color )
                
                                                    break
                                                end
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Special Effect Font
                                ui:AddItem(
                                    UI:Create( "COLOR_SELECT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetInternalColor( Group.Settings.Color.Special_Effect )
                                        ui:SetDefaultColor( Group.Default_Settings.Color.Special_Effect )
                                        ui:SetBackgroundColor( Group.Settings.Color.Background )
                
                                        main.select_special_effect = ui
                
                                        if Group.Settings.enable_unique_player_morale_color then
                                            ui:SetLayerColor( Group.Default_Settings.Color.Unique_Morale )
                                        else
                                            ui:SetLayerColor( Group.Default_Settings.Color.Morale )
                                        end
                
                                        ui:EnableLayer( true )
                
                                        function ui:ColorChanged( color )
                                            Group.Settings.Color.Special_Effect = color
                
                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                
                                            main.Frame.special_text:SetForeColor( color )
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetForeColor( color )
                                            end
                                        end
                
                                        function ui:ColorSliderChanged( color )
                                            Group.Settings.Color.Special_Effect = color
                
                                            main.Frame.special_text:SetForeColor( color )
                
                                            for i = 1, #Group.Party do
                                                Group.Party[ i ].special_text:SetForeColor( color )
                                            end
                                        end
                                    end )
                                )
                            end )
                        )
                    elseif tabs[ i ] == Group:Local( "Options_Tab_6" ) then
                        UI:Create( "SCROLLBOX", ui, function( ui )
                            ui:SetSize( w, h )
                            ui:SizeChanged()

                            w, h = ui.list:GetSize()

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Frame Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Size" ) )
                                            ui:SetMouseVisible( false )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Power Bar Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Power_Size" ) )
                                            ui:SetMouseVisible( false )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Selection Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Selection_Size" ) )
                                            ui:SetMouseVisible( false )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    ui:AddItem( --Frame Size
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing )
                                            ui:SetMouseVisible( false )

                                            --Frame Width
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 16, 256 )
                                                    ui:SetValue( Group.Settings.vitals_x )
                                                    ui:SetKey( "X" )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.vitals_x = value

                                                        main.Frame:SetSize( Group.Settings.vitals_x, Group.Settings.vitals_y )
                                                        main.Frame:SetPos( ( 256 + 32 ) / 2 - Group.Settings.vitals_x / 2, ( 256 + 32 ) / 2 - Group.Settings.vitals_y / 2 )

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:SetWidth( value )
                                                            Group.Party[ i ]:SizeChanged()
                                                        end

                                                        Group:RefreshLayout()

                                                        if main.enable_layout_preview then
                                                            for i = 1, 24 do
                                                                PartyVitals.preview[ i ]:SetWidth( value )
                                                            end

                                                            PartyVitals:RefreshPreview()
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.vitals_x = value

                                                        main.Frame:SetSize( Group.Settings.vitals_x, Group.Settings.vitals_y )
                                                        main.Frame:SetPos( ( 256 + 32 ) / 2 - Group.Settings.vitals_x / 2, ( 256 + 32 ) / 2 - Group.Settings.vitals_y / 2 )

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:SetWidth( value )
                                                            Group.Party[ i ]:SizeChanged()
                                                        end

                                                        Group:RefreshLayout()

                                                        if main.enable_layout_preview then
                                                            for i = 1, 24 do
                                                                PartyVitals.preview[ i ]:SetWidth( value )
                                                            end
                                                            
                                                            PartyVitals:RefreshPreview()
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.vitals_x )
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Frame Height
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 16, 256 )
                                                    ui:SetValue( Group.Settings.vitals_y )
                                                    ui:SetKey( "Y" )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.vitals_y = value

                                                        main.Frame:SetSize( Group.Settings.vitals_x, Group.Settings.vitals_y )
                                                        main.Frame:SetPos( ( 256 + 32 ) / 2 - Group.Settings.vitals_x / 2, ( 256 + 32 ) / 2 - Group.Settings.vitals_y / 2 )

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:SetHeight( value )
                                                        end

                                                        Group:RefreshLayout()

                                                        if main.enable_layout_preview then
                                                            for i = 1, 24 do
                                                                PartyVitals.preview[ i ]:SetHeight( value )
                                                            end
                                                            
                                                            PartyVitals:RefreshPreview()
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.vitals_y = value

                                                        main.Frame:SetSize( Group.Settings.vitals_x, Group.Settings.vitals_y )
                                                        main.Frame:SetPos( ( 256 + 32 ) / 2 - Group.Settings.vitals_x / 2, ( 256 + 32 ) / 2 - Group.Settings.vitals_y / 2 )

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:SetHeight( value )
                                                        end

                                                        Group:RefreshLayout()

                                                        if main.enable_layout_preview then
                                                            for i = 1, 24 do
                                                                PartyVitals.preview[ i ]:SetHeight( value )
                                                            end
                                                            
                                                            PartyVitals:RefreshPreview()
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.vitals_y )
                                                    end
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 0, 128 )
                                                    ui:SetValue( Group.Settings.power_size )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.power_size = value

                                                        main.Frame:SizeChanged()

                                                        local settings = Group.Settings

                                                        for i = 1, #Group.Party do
                                                            if settings.enable_classic_power_bar then
                                                                Group.Party[ i ].morale:SetHeight( settings.vitals_y - value )

                                                                Group.Party[ i ].power:SetPos( settings.vitals_y - value )
                                                                Group.Party[ i ].power:SetHeight( value )
                                                            else
                                                                Group.Party[ i ].morale:SetPos( value, 0 )
                                                                Group.Party[ i ].power:SetWidth( value )
                                                            end

                                                            Group.Party[ i ]:SizeChanged()
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", settings )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.power_size = value

                                                        main.Frame:SizeChanged()

                                                        local settings = Group.Settings

                                                        for i = 1, #Group.Party do
                                                            if settings.enable_classic_power_bar then
                                                                Group.Party[ i ].morale:SetHeight( settings.vitals_y - value )

                                                                Group.Party[ i ].power:SetPos( settings.vitals_y - value )
                                                                Group.Party[ i ].power:SetHeight( value )
                                                            else
                                                                Group.Party[ i ].morale:SetPos( value, 0 )
                                                                Group.Party[ i ].power:SetWidth( value )
                                                            end

                                                            Group.Party[ i ]:SizeChanged()
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.power_size )
                                                    end
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "SLIDER_RESET", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetRange( 0, 16 )
                                            ui:SetValue( Group.Settings.selection_size )
                                            ui:SetKey( "" )
                                            ui:SetKeySize( 0 )

                                            function ui:ValueChanged( value )
                                                Group.Settings.selection_size = value

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].selection.border:SetBorderSize( value )
                                                end

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end

                                            function ui:SliderChanged( value )
                                                Group.Settings.selection_size = value

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].selection.border:SetBorderSize( value )
                                                end
                                            end

                                            function ui:Reset()
                                                self:SetValue( Group.Default_Settings.selection_size )
                                            end
                                        end )
                                    )

                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Name Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Name_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Name Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Name_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Name Width
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Name_Size" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            local object = {
                                Name           = function()
                                    main.Pos[ Group.Settings.Frame.Name.anchor ]( main.Frame.name, Group.Settings.Frame.Name.x, Group.Settings.Frame.Name.y, main.Frame.name:GetSize() ) 
                                    
                                    if Group.Settings.Frame.Name.anchor == 1 or Group.Settings.Frame.Name.anchor == 4 then
                                        main.Frame.name:SetTextAlignment( 4 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Name.anchor ]( Group.Party[ i ].name, Group.Settings.Frame.Name.x, Group.Settings.Frame.Name.y, Group.Party[ i ].name:GetSize() ) 
                                            Group.Party[ i ].name:SetTextAlignment( 4 )
                                        end
                                    elseif Group.Settings.Frame.Name.anchor == 5 then
                                        main.Frame.name:SetTextAlignment( 5 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Name.anchor ]( Group.Party[ i ].name, Group.Settings.Frame.Name.x, Group.Settings.Frame.Name.y, Group.Party[ i ].name:GetSize() ) 
                                            Group.Party[ i ].name:SetTextAlignment( 5 )
                                        end
                                    else
                                        main.Frame.name:SetTextAlignment( 6 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Name.anchor ]( Group.Party[ i ].name, Group.Settings.Frame.Name.x, Group.Settings.Frame.Name.y, Group.Party[ i ].name:GetSize() ) 
                                            Group.Party[ i ].name:SetTextAlignment( 6 )
                                        end
                                    end
                                end,
                                Class          = function() 
                                    main.Pos[ Group.Settings.Frame.Class.anchor ]( main.Frame.class, Group.Settings.Frame.Class.x, Group.Settings.Frame.Class.y, Group.Settings.class_icon_size, Group.Settings.class_icon_size ) 

                                    for i = 1, #Group.Party do
                                        Group.Party[ i ].Pos[ Group.Settings.Frame.Class.anchor ]( Group.Party[ i ].class, Group.Settings.Frame.Class.x, Group.Settings.Frame.Class.y, Group.Settings.class_icon_size, Group.Settings.class_icon_size )
                                    end
                                end,
                                Leader         = function() 
                                    main.Pos[ Group.Settings.Frame.Leader.anchor ]( main.Frame.leader, Group.Settings.Frame.Leader.x, Group.Settings.Frame.Leader.y, Group.Settings.leader_icon_size, Group.Settings.leader_icon_size ) 
                                
                                    for i = 1, #Group.Party do
                                        Group.Party[ i ].Pos[ Group.Settings.Frame.Leader.anchor ]( Group.Party[ i ].leader, Group.Settings.Frame.Leader.x, Group.Settings.Frame.Leader.y, Group.Settings.leader_icon_size, Group.Settings.leader_icon_size )
                                    end
                                end,
                                Power_Text    = function()
                                    main.Frame.power.text.Pos[ Group.Settings.Frame.Power_Text.anchor ]( main.Frame.power.text, Group.Settings.Frame.Power_Text.x, Group.Settings.Frame.Power_Text.y, main.Frame.power.text:GetSize() )

                                    if Group.Settings.Frame.Power_Text.anchor == 1 or Group.Settings.Frame.Power_Text.anchor == 4 then
                                        main.Frame.power.text:SetTextAlignment( 4 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].power.text.Pos[ Group.Settings.Frame.Power_Text.anchor ]( Group.Party[ i ].power.text, Group.Settings.Frame.Power_Text.x, Group.Settings.Frame.Power_Text.y, Group.Party[ i ].power.text:GetSize() ) 
                                            Group.Party[ i ].power.text:SetTextAlignment( 4 )
                                        end
                                    elseif Group.Settings.Frame.Power_Text.anchor == 5 then
                                        main.Frame.power.text:SetTextAlignment( 5 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].power.text.Pos[ Group.Settings.Frame.Power_Text.anchor ]( Group.Party[ i ].power.text, Group.Settings.Frame.Power_Text.x, Group.Settings.Frame.Power_Text.y, Group.Party[ i ].power.text:GetSize() ) 
                                            Group.Party[ i ].power.text:SetTextAlignment( 5 )
                                        end
                                    else
                                        main.Frame.power.text:SetTextAlignment( 6 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].power.text.Pos[ Group.Settings.Frame.Power_Text.anchor ]( Group.Party[ i ].power.text, Group.Settings.Frame.Power_Text.x, Group.Settings.Frame.Power_Text.y, Group.Party[ i ].power.text:GetSize() ) 
                                            Group.Party[ i ].power.text:SetTextAlignment( 6 )
                                        end
                                    end
                                end,
                                Morale_Text    = function() 
                                    main.Pos[ Group.Settings.Frame.Morale_Text.anchor ]( main.Frame.morale.text, Group.Settings.Frame.Morale_Text.x, Group.Settings.Frame.Morale_Text.y, main.Frame.morale.text:GetSize() )

                                    if Group.Settings.Frame.Morale_Text.anchor == 1 or Group.Settings.Frame.Morale_Text.anchor == 4 then
                                        main.Frame.morale.text:SetTextAlignment( 4 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Morale_Text.anchor ]( Group.Party[ i ].morale.text, Group.Settings.Frame.Morale_Text.x, Group.Settings.Frame.Morale_Text.y, Group.Party[ i ].morale.text:GetSize() ) 
                                            Group.Party[ i ].morale.text:SetTextAlignment( 4 )
                                        end
                                    elseif Group.Settings.Frame.Morale_Text.anchor == 5 then
                                        main.Frame.morale.text:SetTextAlignment( 5 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Morale_Text.anchor ]( Group.Party[ i ].morale.text, Group.Settings.Frame.Morale_Text.x, Group.Settings.Frame.Morale_Text.y, Group.Party[ i ].morale.text:GetSize() ) 
                                            Group.Party[ i ].morale.text:SetTextAlignment( 5 )
                                        end
                                    else
                                        main.Frame.morale.text:SetTextAlignment( 6 )

                                        for i = 1, #Group.Party do
                                            Group.Party[ i ].Pos[ Group.Settings.Frame.Morale_Text.anchor ]( Group.Party[ i ].morale.text, Group.Settings.Frame.Morale_Text.x, Group.Settings.Frame.Morale_Text.y, Group.Party[ i ].morale.text:GetSize() ) 
                                            Group.Party[ i ].morale.text:SetTextAlignment( 6 )
                                        end
                                    end
                                end,
                                Effect         = function() 
                                    main.Frame:EffectLayout() 

                                    for i = 1, #Group.Party do
                                        Group.Party[ i ]:EffectLayout()
                                    end
                                end,
                                Special_Effect = function()
                                    main.Pos[ Group.Settings.Frame.Special_Effect.anchor ]( main.Frame.special, Group.Settings.Frame.Special_Effect.x, Group.Settings.Frame.Special_Effect.y, Group.Settings.special_effect_size, Group.Settings.special_effect_size ) 

                                    for i = 1, #Group.Party do
                                        Group.Party[ i ].Pos[ Group.Settings.Frame.Special_Effect.anchor ]( Group.Party[ i ].special, Group.Settings.Frame.Special_Effect.x, Group.Settings.Frame.Special_Effect.y, Group.Settings.special_effect_size, Group.Settings.special_effect_size )
                                    end
                                end
                            }

                            local tbl = {
                                "Name",
                                "Class", 
                                "Leader",
                                "Morale_Text",
                                "Power_Text",
                                "Effect",
                                "Special_Effect"
                            }

                            main.FrameObject = {}

                            local i = 1

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )
                                            --Name Width
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 0, 256 )
                                                    ui:SetValue( Group.Settings.name_width )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )
            
                                                    function ui:ValueChanged( value )
                                                        Group.Settings.name_width = value
            
                                                        main.Frame.name:SetWidth( value )
            
                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ].name:SetWidth( value )
                                                        end
            
                                                        object[ k ]()
            
                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end
            
                                                    function ui:SliderChanged( value )
                                                        Group.Settings.name_width = value
            
                                                        main.Frame.name:SetWidth( value )
            
                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ].name:SetWidth( value )
                                                        end
            
                                                        object[ k ]()
                                                    end
            
                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.name_width )
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Bounds
                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Name_Preview" ) )

                                                    UI:Create( "CHECKBOX", ui, function( ui )
                                                        ui:SetSize( 16, 16 )
                                                        ui:SetPos( ( w - 32 ) / 3 - 16, 0 )
                                                        ui:SetChecked( false )
                            
                                                        function ui:CheckedChanged()
                                                            local checked = self:IsChecked()
            
                                                            if checked then
                                                                main.Frame.name:SetColor( 200, 0, 0, 100 )
                                                            else
                                                                main.Frame.name:SetColor( 0, 0, 0, 0 )
                                                            end
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Class Icon Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Class_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Class Icon Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Class_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Class Icon Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Class_Size" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Class Icon Size
                                    ui:AddItem(
                                        UI:Create( "SLIDER_RESET", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetRange( 4, 32 )
                                            ui:SetValue( Group.Settings.class_icon_size )
                                            ui:SetKey( "" )
                                            ui:SetKeySize( 0 )

                                            function ui:ValueChanged( value )
                                                Group.Settings.class_icon_size = value

                                                main.Frame.class:SetStretchMode( 1 )
                                                main.Frame.class:SetSize( value, value )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].class:SetStretchMode( 1 )
                                                    Group.Party[ i ].class:SetSize( value, value )
                                                end

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end

                                            function ui:SliderChanged( value )
                                                Group.Settings.class_icon_size = value

                                                main.Frame.class:SetStretchMode( 1 )
                                                main.Frame.class:SetSize( value, value )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].class:SetStretchMode( 1 )
                                                    Group.Party[ i ].class:SetSize( value, value )
                                                end
                                            end

                                            function ui:Reset()
                                                self:SetValue( Group.Default_Settings.class_icon_size )
                                            end
                                        end )
                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Leader Icon Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Leader_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Leader Icon Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Leader_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Leader Icon Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Leader_Size" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Leader Icon Size
                                    ui:AddItem(
                                        UI:Create( "SLIDER_RESET", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetRange( 4, 32 )
                                            ui:SetValue( Group.Settings.leader_icon_size )
                                            ui:SetKey( "" )
                                            ui:SetKeySize( 0 )

                                            function ui:ValueChanged( value )
                                                Group.Settings.leader_icon_size = value

                                                main.Frame.leader:SetStretchMode( 1 )
                                                main.Frame.leader:SetSize( value, value )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].leader:SetStretchMode( 1 )
                                                    Group.Party[ i ].leader:SetSize( value, value )
                                                end

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end

                                            function ui:SliderChanged( value )
                                                Group.Settings.leader_icon_size = value

                                                main.Frame.leader:SetStretchMode( 1 )
                                                main.Frame.leader:SetSize( value, value )

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].leader:SetStretchMode( 1 )
                                                    Group.Party[ i ].leader:SetSize( value, value )
                                                end
                                            end

                                            function ui:Reset()
                                                self:SetValue( Group.Default_Settings.leader_icon_size )
                                            end
                                        end )
                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Morale Text Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Morale_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Morale Text Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Morale_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Morale Text Format
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Morale_Format" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Morale
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )

                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Format_1" ),
                                                Group:Local( "UI_Dropdown_Format_2" ),
                                                Group:Local( "UI_Dropdown_Format_3" ),
                                                Group:Local( "UI_Dropdown_Format_4" ),
                                                Group:Local( "UI_Dropdown_Format_5" ),
                                                Group:Local( "UI_Dropdown_Format_6" ),
                                                Group:Local( "UI_Dropdown_Format_7" ),
                                                Group:Local( "UI_Dropdown_Format_8" ),
                                                Group:Local( "UI_Dropdown_Format_9" )
                                            }

                                            for i = 1, #tbl do
                                                local item = ui:Add( tbl[ i ], function()
                                                    Group.Settings.morale_text_format = i

                                                    main.Frame.morale.text.format[ i ]()

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:Init()
                                                    end
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                                end )

                                                if i == Group.Settings.morale_text_format then
                                                    ui:SetSelected( item )
                                                end
                                            end
                                        end )
                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Power Text Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Power_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Power Text Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Power_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Power Text Format
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Power_Position" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Power
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )

                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Format_1" ),
                                                Group:Local( "UI_Dropdown_Format_2" ),
                                                Group:Local( "UI_Dropdown_Format_3" ),
                                                Group:Local( "UI_Dropdown_Format_4" ),
                                                Group:Local( "UI_Dropdown_Format_5" ),
                                                Group:Local( "UI_Dropdown_Format_6" ),
                                                Group:Local( "UI_Dropdown_Format_7" ),
                                                Group:Local( "UI_Dropdown_Format_8" ),
                                                Group:Local( "UI_Dropdown_Format_9" )
                                            }

                                            for i = 1, #tbl do
                                                local item = ui:Add( tbl[ i ], function()
                                                    Group.Settings.power_text_format = i

                                                    main.Frame.power.text.format[ i ]()

                                                    for i = 1, #Group.Party do
                                                        Group.Party[ i ]:Init()
                                                    end
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                                end )

                                                if i == Group.Settings.power_text_format then
                                                    ui:SetSelected( item )
                                                end
                                            end
                                        end )
                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Effects Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Effect_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Effects Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Effect_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Effect Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Effect_Size" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 24 + UI.Spacing + ( 16 * 3 + UI.Spacing * 7 ) ) 
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 16 * 4 + UI.Spacing * 3 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 * 2 + UI.Spacing )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                            ui:AddFiller( 0, UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Effect_Spacing" ) )
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Effect Spacing
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 0, 16 )
                                                    ui:SetValue( Group.Settings.effect_spacing )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.effect_spacing = value

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:EffectLayout()
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.effect_spacing = value

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:EffectLayout()
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.effect_spacing )
                                                    end
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 + UI.Spacing + ( 16 * 3 + UI.Spacing * 7 ) )

                                            ui:AddItem(
                                                UI:Create( "DROPDOWN", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 24 )

                                                    main.FrameObject[ k ].anchor = ui
                            
                                                    local tbl = {
                                                        Group:Local( "UI_Dropdown_Top_Left" ),
                                                        Group:Local( "UI_Dropdown_Top_Right" ),
                                                        Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                        Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                        Group:Local( "UI_Dropdown_Center" )
                                                    }

                                                    ui.list = {}
                            
                                                    for i = 1, #tbl do
                                                        if i == 5 and k == "Effect" then break end

                                                        local item = ui:Add( tbl[ i ], function()
                                                            v.anchor = i

                                                            if tbl[ i ] == "Center" then 
                                                                main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                                main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                            else
                                                                main.FrameObject[ k ].x:SetValue( 0 )
                                                                main.FrameObject[ k ].y:SetValue( 0 )
                                                            end

                                                            object[ k ]()
                                        
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                            
                                                        end )
                            
                                                        if i == v.anchor then
                                                            ui:SetSelected( item )
                                                        end

                                                        ui.list[ i ] = item
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Effect_Flash_Threshold" ) )
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Effect Flash Threshold
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetKey( "" )
                                                    ui:SetRange( 0, 60 )
                                                    ui:SetValue( Group.Settings.effect_flash_threshold )
                                                    ui:SetKeySize( 0)

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.effect_flash_threshold = value

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end
                                    
                                                    function ui:SliderChanged( value )
                                                        Group.Settings.effect_flash_threshold = value
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.effect_flash_threshold )
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Effect_Duration_Threshold" ) )
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Effect Duration Threshold
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetKey( "" )
                                                    ui:SetRange( 0, 600 )
                                                    ui:SetValue( Group.Settings.effect_duration_threshold )
                                                    ui:SetKeySize( 0 )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.effect_duration_threshold = value

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end
                                    
                                                    function ui:SliderChanged( value )
                                                        Group.Settings.effect_duration_threshold = value
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.effect_duration_threshold )
                                                    end
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing + ( 16 * 2 + UI.Spacing ) * 2 )
                                            --Effect Size
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 8, 32 )
                                                    ui:SetValue( Group.Settings.effect_size )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.effect_size = value

                                                        for i = 1, #main.Frame.effect do
                                                            main.Frame.effect[ i ]:SetStretchMode( 1 )
                                                            main.Frame.effect[ i ]:SetSize( value, value )
                                                        end

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            for k = 1, Group.Party[ i ].effect_count do
                                                                Group.Party[ i ].effect[ k ]:SetStretchMode( 1 )
                                                                Group.Party[ i ].effect[ k ]:SetSize( value, value )
                                                                Group.Party[ i ]:EffectLayout()
                                                            end
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.effect_size = value

                                                        for i = 1, #main.Frame.effect do
                                                            main.Frame.effect[ i ]:SetStretchMode( 1 )
                                                            main.Frame.effect[ i ]:SetSize( value, value )
                                                        end

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            for k = 1, Group.Party[ i ].effect_count do
                                                                Group.Party[ i ].effect[ k ]:SetStretchMode( 1 )
                                                                Group.Party[ i ].effect[ k ]:SetSize( value, value )
                                                                Group.Party[ i ]:EffectLayout()
                                                            end
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.effect_size )
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Effect_Count_Preview" ) ) 
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Effect Count ( Preview )
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 0, 20 )
                                                    ui:SetValue( 4 )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )

                                                    main.effect_count = 4

                                                    function ui:ValueChanged( value )
                                                        main.effect_count = value

                                                        for i = 1, 20 do
                                                            if i > value then
                                                                main.Frame.effect[ i ]:SetVisible( false )
                                                            else
                                                                main.Frame.effect[ i ]:SetVisible( true )
                                                            end
                                                        end

                                                        main.Frame:EffectLayout()
                                                    end

                                                    function ui:SliderChanged( value )
                                                        main.effect_count = value

                                                        for i = 1, 20 do
                                                            if i > value then
                                                                main.Frame.effect[ i ]:SetVisible( false )
                                                            else
                                                                main.Frame.effect[ i ]:SetVisible( true )
                                                            end
                                                        end

                                                        main.Frame:EffectLayout()
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( 4 )
                                                    end
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            ui:AddItem(
                                                UI:Create( "TEXT", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 16 )
                                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                                    end
                                                    
                                                    ui:SetText( Group:Local( "Options_Frame_Effect_Max_Row" ) )
                                                end )
                                            )

                                            ui:AddFiller( 0, UI.Spacing )

                                            --Effects Per Row
                                            ui:AddItem(
                                                UI:Create( "SLIDER_RESET", ui, function( ui )
                                                    ui:SetWidth( ( w - 32 ) / 3 )
                                                    ui:SetRange( 1, 20 )
                                                    ui:SetValue( Group.Settings.effect_row_count )
                                                    ui:SetKey( "" )
                                                    ui:SetKeySize( 0 )

                                                    function ui:ValueChanged( value )
                                                        Group.Settings.effect_row_count = value

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:EffectLayout()
                                                        end

                                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                    end

                                                    function ui:SliderChanged( value )
                                                        Group.Settings.effect_row_count = value

                                                        main.Frame:EffectLayout()

                                                        for i = 1, #Group.Party do
                                                            Group.Party[ i ]:EffectLayout()
                                                        end
                                                    end

                                                    function ui:Reset()
                                                        self:SetValue( Group.Default_Settings.effect_row_count )
                                                    end
                                                end )
                                            )
                                        end )

                                    )
                                end )
                            )

                            i = i + 1

                            ui:AddFiller( 0, 16 )

                            ui:AddItem(
                                UI:Create( "DIVIDER", ui, function( ui )
                                    ui:SetWidth( w )
                                end )
                            )

                            ui:AddFiller( 0, 16 )

                            ui:AddItem( --Header
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 16 )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    --Special Effect Position
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Special_Position" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Special Effect Anchor Point
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Special_Anchor" ) )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    --Special Effect Size
                                    ui:AddItem(
                                        UI:Create( "TEXT", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                            if language == Turbine.Language.German or language == Turbine.Language.French then
                                                ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                            end
                                            
                                            ui:SetText( Group:Local( "Options_Frame_Special_Size" ) )
                                        end )
                                    )
                                end )
                            )

                            ui:AddFiller( 0, UI.Spacing )

                            ui:AddItem( --Frame Size
                                UI:Create( "LIST", ui, function( ui )
                                    ui:SetSize( w, 32 + UI.Spacing )
                                    --ui:SetColor( 100, 100, 100 )
                                    ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                    local k = tbl[ i ]
                                    local v = Group.Settings.Frame[ k ]
        
                                    main.FrameObject[ k ] = {}
        
                                    ui:AddItem(
                                        UI:Create( "LIST", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 116 )

                                            ui:AddItem(
                                                UI:Create( "LIST", ui, function( ui )
                                                    ui:SetSize( ( w - 32 ) / 3, 116 )

                                                    UI:Create( "BUTTON", ui, function( ui )
                                                        ui:SetPos( ( w - 32 ) / 3 - 56, 0 )
                                
                                                        function ui:Pressed()
                                                            local anchor = Group.Default_Settings.Frame[ k ].anchor
                
                                                            main.FrameObject[ k ].anchor:SetSelected( main.FrameObject[ k ].anchor.list[ anchor ] )
                
                                                            v.anchor = anchor
                
                                                            main.FrameObject[ k ].x:SetValue( Group.Default_Settings.Frame[ k ].x )
                                                            main.FrameObject[ k ].y:SetValue( Group.Default_Settings.Frame[ k ].y )
                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                                    end )
                                                    
                                                    main.FrameObject[ k ].x = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "X" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.x + 256 / 2 )
                                                            ui:SetText( v.x )
                                                        else
                                                            ui:SetValue( v.x )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                
                                                            v.x = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                
                                                    main.FrameObject[ k ].y = UI:Create( "SLIDER", ui, function( ui )
                                                        ui:SetPos( 0, 16 + UI.Spacing )
                                                        ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                        ui:SetKey( "Y" )
                                                        ui:SetRange( 0, 256 )
                                                        ui:SetValueSize( 32 )
                
                                                        if Group.Settings.Frame[ k ].anchor == 5 then
                                                            ui:SetValue( v.y + 256 / 2 )
                                                            ui:SetText( v.y )
                                                        else
                                                            ui:SetValue( v.y )
                                                        end
                                
                                                        function ui:ValueChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                
                                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                        end
                                
                                                        function ui:SliderChanged( value )
                                                            if Group.Settings.Frame[ k ].anchor == 5 then
                                                                value = value - 256 / 2
                
                                                                self:SetText( value )
                                                            end
                                                            
                                                            v.y = value
                
                                                            object[ k ]()
                                                        end
                                                    end )
                                                end )
                                            )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "DROPDOWN", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 24 )

                                            main.FrameObject[ k ].anchor = ui
                    
                                            local tbl = {
                                                Group:Local( "UI_Dropdown_Top_Left" ),
                                                Group:Local( "UI_Dropdown_Top_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Right" ),
                                                Group:Local( "UI_Dropdown_Bottom_Left" ),
                                                Group:Local( "UI_Dropdown_Center" )
                                            }

                                            ui.list = {}
                    
                                            for i = 1, #tbl do
                                                if i == 5 and k == "Effect" then break end

                                                local item = ui:Add( tbl[ i ], function()
                                                    v.anchor = i

                                                    if tbl[ i ] == "Center" then 
                                                        main.FrameObject[ k ].x:SetValue( 256 / 2 )
                                                        main.FrameObject[ k ].y:SetValue( 256 / 2 )
                                                    else
                                                        main.FrameObject[ k ].x:SetValue( 0 )
                                                        main.FrameObject[ k ].y:SetValue( 0 )
                                                    end

                                                    object[ k ]()
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                    
                                                end )
                    
                                                if i == v.anchor then
                                                    ui:SetSelected( item )
                                                end

                                                ui.list[ i ] = item
                                            end
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )

                                    ui:AddItem(
                                        UI:Create( "SLIDER_RESET", ui, function( ui )
                                            ui:SetWidth( ( w - 32 ) / 3 )
                                            ui:SetRange( 8, 32 )
                                            ui:SetValue( Group.Settings.special_effect_size )
                                            ui:SetKey( "" )
                                            ui:SetKeySize( 0 )

                                            function ui:ValueChanged( value )
                                                Group.Settings.special_effect_size = value

                                                main.Frame.special:SetSize( value, value )
                                                main.Frame.special:PositionChanged()

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].special:SetStretchMode( 1 )
                                                    Group.Party[ i ].special:SetSize( value, value )
                                                    Group.Party[ i ].special:PositionChanged()
                                                end

                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end

                                            function ui:SliderChanged( value )
                                                Group.Settings.special_effect_size = value

                                                main.Frame.special:SetSize( value, value )
                                                main.Frame.special:PositionChanged()

                                                for i = 1, #Group.Party do
                                                    Group.Party[ i ].special:SetStretchMode( 1 )
                                                    Group.Party[ i ].special:SetSize( value, value )
                                                    Group.Party[ i ].special:PositionChanged()
                                                end
                                            end

                                            function ui:Reset()
                                                self:SetValue( Group.Default_Settings.special_effect_size )
                                            end
                                        end )
                                    )
                                end )
                            )

                        end )
                    elseif tabs[ i ] == Group:Local( "Options_Tab_7" ) then

                        main.slider_x = {}
                        main.slider_y = {}

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --1 - 3 Players
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Party_3" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --4 - 6 Players
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Party_6" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --7 - 12 Players
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Party_12" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Layout
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 32 + UI.Spacing )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                for i = 1, 3 do
                                    ui:AddItem(
                                        UI:Create( "BOX", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing )
                                            ui:SetColor( 0, 0, 100, 0 )
                                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                                    
                                            main.slider_x[ i ] = UI:Create( "SLIDER", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                ui:SetRange( 1, Group.Settings.Layout[ i ].max )
                                                ui:SetValue( Group.Settings.Layout[ i ].x )
                                                ui:SetKey( "X" )
                                    
                                                function ui:ValueChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = value
                                                    Group.Settings.Layout[ i ].y = new
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_y[ i ]:SetValue( new, true )
                                                    main.slider_y[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                    
                                                function ui:SliderChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = value
                                                    Group.Settings.Layout[ i ].y = new
                                    
                                                    main.slider_y[ i ]:SetValue( new, true )
                                                    main.slider_y[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                            end )
        
                                            main.slider_y[ i ] = UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetPos( 0, 16 + UI.Spacing )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetRange( 1, Group.Settings.Layout[ i ].max )
                                                ui:SetValue( Group.Settings.Layout[ i ].y )
                                                ui:SetKey( "Y" )
                                
                                                function ui:ValueChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = new
                                                    Group.Settings.Layout[ i ].y = value
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_x[ i ]:SetValue( new, true )
                                                    main.slider_x[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                    
                                                function ui:SliderChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = new
                                                    Group.Settings.Layout[ i ].y = value
                                    
                                                    main.slider_x[ i ]:SetValue( new, true )
                                                    main.slider_x[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                            
                                                function ui:Reset()
                                                    Group.Settings.Layout[ i ].x = Group.Default_Settings.Layout[ i ].x
                                                    Group.Settings.Layout[ i ].y = Group.Default_Settings.Layout[ i ].y
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_x[ i ]:SetValue( Group.Default_Settings.Layout[ i ].x, true )
                                                    main.slider_x[ i ]:SetText( Group.Default_Settings.Layout[ i ].x )
                                                    self:SetValue( Group.Default_Settings.Layout[ i ].y, true )
                                                    self:SetText( Group.Default_Settings.Layout[ i ].y )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                            end )
                                        end )
                                    )
        
                                    ui:AddFiller( 16, 0 )
                                end
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --13 - 18 Players
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Party_18" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --19 - 24 Players
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Party_24" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem( --Layout
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 32 + UI.Spacing )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetSize( w, 32 + UI.Spacing * 3 )
                                end

                                for i = 4, 5 do
                                    ui:AddItem(
                                        UI:Create( "BOX", ui, function( ui )
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing )
                                            ui:SetColor( 0, 0, 100, 0 )
                                            ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                                    
                                            main.slider_x[ i ] = UI:Create( "SLIDER", ui, function( ui )
                                                ui:SetWidth( ( w - 32 ) / 3 - 56 - UI.Spacing )
                                                ui:SetRange( 1, Group.Settings.Layout[ i ].max )
                                                ui:SetValue( Group.Settings.Layout[ i ].x )
                                                ui:SetKey( "X" )
                                    
                                                function ui:ValueChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = value
                                                    Group.Settings.Layout[ i ].y = new
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_y[ i ]:SetValue( new, true )
                                                    main.slider_y[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                    
                                                function ui:SliderChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = value
                                                    Group.Settings.Layout[ i ].y = new
                                    
                                                    main.slider_y[ i ]:SetValue( new, true )
                                                    main.slider_y[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                            end )
        
                                            main.slider_y[ i ] = UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetPos( 0, 16 + UI.Spacing )
                                                ui:SetWidth( ( w - 32 ) / 3 )
                                                ui:SetRange( 1, Group.Settings.Layout[ i ].max )
                                                ui:SetValue( Group.Settings.Layout[ i ].y )
                                                ui:SetKey( "Y" )
                                
                                                function ui:ValueChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = new
                                                    Group.Settings.Layout[ i ].y = value
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_x[ i ]:SetValue( new, true )
                                                    main.slider_x[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                    
                                                function ui:SliderChanged( value )
                                                    local new = math.ceil( Group.Settings.Layout[ i ].max / value )
                                    
                                                    Group.Settings.Layout[ i ].x = new
                                                    Group.Settings.Layout[ i ].y = value
                                    
                                                    main.slider_x[ i ]:SetValue( new, true )
                                                    main.slider_x[ i ]:SetText( new )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                            
                                                function ui:Reset()
                                                    Group.Settings.Layout[ i ].x = Group.Default_Settings.Layout[ i ].x
                                                    Group.Settings.Layout[ i ].y = Group.Default_Settings.Layout[ i ].y
                                    
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    
                                                    main.slider_x[ i ]:SetValue( Group.Default_Settings.Layout[ i ].x, true )
                                                    main.slider_x[ i ]:SetText( Group.Default_Settings.Layout[ i ].x )
                                                    self:SetValue( Group.Default_Settings.Layout[ i ].y, true )
                                                    self:SetText( Group.Default_Settings.Layout[ i ].y )
                                    
                                                    if Group.PartyCount > 1 then
                                                        Group:DefineGridLayout( Group.PartyCount )
                                                        Group:RefreshLayout()
                                                    end
        
                                                    if main.enable_layout_preview then
                                                        PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                                        PartyVitals:RefreshPreview()
                                                    end
                                                end
                                            end )
                                        end )
                                    )

                                    ui:AddFiller( 16, 0 )
                                end

                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana14 )
                                        end
                                        
                                        ui:SetForeColor( Color( 125, 125, 125 ) )
                                        ui:SetText( Group:Local( "Options_Layout_Party_Description" ) )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetSize( ( w - 32 ) / 3, 32 + UI.Spacing * 3 )
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Horizontal Layout Direction
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Horizontal" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Vertical Layout Direction
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Vertical" ) )
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Spacing
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Spacing" ) )
                                    end )
                                )
                            end )
                        )
                        
                        ui:AddItem( --Layout
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 24 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Horizontal Layout Direction
                                ui:AddItem(
                                    UI:Create( "DROPDOWN", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )

                                        local tbl = {
                                            Group:Local( "UI_Dropdown_Left" ),
                                            Group:Local( "UI_Dropdown_Center" ),
                                            Group:Local( "UI_Dropdown_Right" )
                                        }

                                        for i = 1, #tbl do
                                            local item = ui:Add( tbl[ i ], function()
                                                Group.Settings.horizontal_layout_direction = i

                                                if main.enable_layout_preview then
                                                    PartyVitals:RefreshPreview()
                                                end
                            
                                                if Group.PartyCount > 0 then
                                                    Group:RefreshLayout()
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end )

                                            if i == Group.Settings.horizontal_layout_direction then
                                                ui:SetSelected( item )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Vertical Layout Direction
                                ui:AddItem(
                                    UI:Create( "DROPDOWN", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )

                                        local tbl = {
                                            Group:Local( "UI_Dropdown_Top" ),
                                            Group:Local( "UI_Dropdown_Bottom" )
                                        }

                                        for i = 1, #tbl do
                                            local item = ui:Add( tbl[ i ], function()
                                                Group.Settings.vertical_layout_direction = i

                                                if main.enable_layout_preview then
                                                    PartyVitals:RefreshPreview()
                                                end
                            
                                                if Group.PartyCount > 0 then
                                                    Group:RefreshLayout()
                                                end
                            
                                                Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                            end )

                                            if i == Group.Settings.vertical_layout_direction then
                                                ui:SetSelected( item )
                                            end
                                        end
                                    end )
                                )

                                ui:AddFiller( 16, 0 )

                                --Spacing
                                ui:AddItem(
                                    UI:Create( "SLIDER_RESET", ui, function( ui )
                                        ui:SetWidth( ( w - 32 ) / 3 )
                                        ui:SetKey( "" )
                                        ui:SetRange( 0, 128 )
                                        ui:SetValue( Group.Settings.vitals_external_spacing )
                                        ui:SetKeySize( 0 )

                                        function ui:ValueChanged( value )
                                            Group.Settings.vitals_external_spacing = value

                                            if Group.PartyCount > 1 then
                                                Group:RefreshLayout()
                                            end

                                            if main.enable_layout_preview then
                                                PartyVitals:RefreshPreview()
                                            end

                                            Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                        end

                                        function ui:SliderChanged( value )
                                            Group.Settings.vitals_external_spacing = value

                                            if Group.PartyCount > 1 then
                                                Group:RefreshLayout()
                                            end

                                            if main.enable_layout_preview then
                                                PartyVitals:RefreshPreview()
                                            end
                                        end

                                        function ui:Reset()
                                            self:SetValue( Group.Default_Settings.vitals_external_spacing )
                                        end
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Position
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( w / 1.5 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Position" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )
                        
                        ui:AddItem( --Layout
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 32 + UI.Spacing )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Position
                                ui:AddItem(
                                    UI:Create( "LIST", ui, function( ui )
                                        ui:SetSize( w / 1.5, 32 + UI.Spacing )

                                        ui:AddItem(
                                            UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetWidth( w / 1.5 )
                                                ui:SetRange( 0, UI.SCREEN_W )
                                                ui:SetKey( "X" )
                                                ui:SetResetText( Group:Local( "UI_Button_Center" ) )
                                                ui:SetValueSize( 48 )
                                                ui:SetButtonSize( 64 )
                                                ui:SetValue( Group.Settings.Pos.x )

                                                main.layout_x = ui
                                
                                                function ui:ValueChanged( value )
                                                    Group.Settings.Pos.x = value
                                
                                                    PartyVitals:SetPos( Group.Settings.Pos.x - UI.SCREEN_W, Group.Settings.Pos.y - UI.SCREEN_H )
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                end
                                
                                                function ui:SliderChanged( value )
                                                    Group.Settings.Pos.x = value
                                
                                                    PartyVitals:SetPos( Group.Settings.Pos.x - UI.SCREEN_W, Group.Settings.Pos.y - UI.SCREEN_H )
                                                end

                                                function ui:Reset()
                                                    self:SetValue( Group.Default_Settings.Pos.x )
                                                end
                                            end )
                                        )

                                        ui:AddFiller( 0, UI.Spacing )

                                        ui:AddItem(
                                            UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetWidth( w / 1.5 )
                                                ui:SetRange( 0, UI.SCREEN_H )
                                                ui:SetKey( "Y" )
                                                ui:SetResetText( Group:Local( "UI_Button_Center" ) )
                                                ui:SetValueSize( 48 )
                                                ui:SetButtonSize( 64 )
                                                ui:SetValue( Group.Settings.Pos.y )

                                                main.layout_y = ui
                                
                                                function ui:ValueChanged( value )
                                                    Group.Settings.Pos.y = value
                                
                                                    PartyVitals:SetPos( Group.Settings.Pos.x - UI.SCREEN_W, Group.Settings.Pos.y - UI.SCREEN_H )
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                                end
                                
                                                function ui:SliderChanged( value )
                                                    Group.Settings.Pos.y = value
                                
                                                    PartyVitals:SetPos( Group.Settings.Pos.x - UI.SCREEN_W, Group.Settings.Pos.y - UI.SCREEN_H )
                                                end

                                                function ui:Reset()
                                                    self:SetValue( Group.Default_Settings.Pos.y )
                                                end
                                            end )
                                        )
                                    end )
                                )
                            end )
                        )

                        function PartyVitals:LayoutPosChanged( x, y )
                            main.layout_x:SetValue( x, true )
                            main.layout_x:SetText( x )

                            main.layout_y:SetValue( y, true )
                            main.layout_y:SetText( y )
                        end

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Enable Layout Preview
                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Layout_Preview" ) )

                                UI:Create( "CHECKBOX", ui, function( ui )
                                    ui:SetSize( 16, 16 )
                                    ui:SetPos( w / 1.5 - 16, 0 )
                                    ui:SetChecked( Group.Settings.disable_vanilla_target_ui )

                                    function ui:CheckedChanged()
                                        local checked = self:IsChecked()

                                        main.enable_layout_preview  = checked
                                        Group.enable_layout_preview = checked

                                        if checked then
                                            PartyVitals:DefinePreviewLayout( PartyVitals.preview_count )
                                            PartyVitals:RefreshPreview()
                                        end

                                        local party = Group.Party

                                        for i = 1, #party do
                                            if checked then 
                                                party[ i ]:SetVisible( false )
                                            else
                                                if Group.Settings.hide_out_of_combat then
                                                    party[ i ]:SetVisible( Player:IsInCombat() )
                                                elseif not Group.hidden then
                                                    party[ i ]:SetVisible( true )
                                                end
                                            end
                                        end
                                        
                                        for i = 1, 24 do
                                            if checked then
                                                if i <= PartyVitals.preview_count then
                                                    PartyVitals.preview[ i ]:SetVisible( true )
                                                end
                                            else
                                                PartyVitals.preview[ i ]:SetVisible( false )
                                            end
                                        end
                                    end
                                end )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Player Count ( Preview )
                        ui:AddItem(
                            UI:Create( "TEXT", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                if language == Turbine.Language.German or language == Turbine.Language.French then
                                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                end
                                
                                ui:SetText( Group:Local( "Options_Layout_Preview_Count" ) )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )

                        ui:AddItem(
                            UI:Create( "SLIDER_RESET", ui, function( ui )
                                ui:SetWidth( w / 1.5 )
                                ui:SetRange( 1, 24 )
                                ui:SetValue( 12 )
                                ui:SetKey( "" )
                                ui:SetKeySize( 0 )

                                function ui:ValueChanged( value )
                                    PartyVitals.preview_count = value

                                    if main.enable_layout_preview then
                                        for i = 1, 24 do
                                            if i <= PartyVitals.preview_count then
                                                PartyVitals.preview[ i ]:SetVisible( true )
                                            else
                                                PartyVitals.preview[ i ]:SetVisible( false )
                                            end
                                        end
                                    end

                                    if main.enable_layout_preview then
                                        PartyVitals:DefinePreviewLayout( value )
                                        PartyVitals:RefreshPreview()
                                    end
                                end

                                function ui:SliderChanged( value )
                                    PartyVitals.preview_count = value

                                    if main.enable_layout_preview then
                                        for i = 1, 24 do
                                            if i <= PartyVitals.preview_count then
                                                PartyVitals.preview[ i ]:SetVisible( true )
                                            else
                                                PartyVitals.preview[ i ]:SetVisible( false )
                                            end
                                        end
                                    end

                                    if main.enable_layout_preview then
                                        PartyVitals:DefinePreviewLayout( value )
                                        PartyVitals:RefreshPreview()
                                    end
                                end

                                function ui:Reset()
                                    self:SetValue( 12 )
                                end
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Hide Group Frames When Not In Combat
                        ui:AddItem(
                            UI:Create( "BOX", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w / 1.5, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end
                                    
                                    ui:SetText( Group:Local( "Options_Layout_Combat_Hide" ) )
                                end )

                                UI:Create( "CHECKBOX", ui, function( ui )
                                    ui:SetSize( 16, 16 )
                                    ui:SetPos( w / 1.5 - 16, 0 )
                                    ui:SetChecked( Group.Settings.hide_out_of_combat )

                                    function ui:CheckedChanged()
                                        local checked = self:IsChecked()

                                        Group.Settings.hide_out_of_combat = checked
                                        
                                        if not Group.hidden then
                                            if checked then
                                                PartyVitals:SetVisible( Player:IsInCombat() )
                                            else
                                                PartyVitals:SetVisible( true )
                                            end
                                        end

                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    end
                                end )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Show Player Frame When Not In A Group
                        ui:AddItem(
                            UI:Create( "BOX", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w / 1.5, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end
                                    
                                    ui:SetText( Group:Local( "Options_Layout_Show_Solo" ) )
                                end )

                                UI:Create( "CHECKBOX", ui, function( ui )
                                    ui:SetSize( 16, 16 )
                                    ui:SetPos( w / 1.5 - 16, 0 )
                                    ui:SetChecked( Group.Settings.show_vitals_out_of_group )

                                    function ui:CheckedChanged()
                                        local checked = self:IsChecked()

                                        Group.Settings.show_vitals_out_of_group = checked

                                        if checked and Group.PartyCount == 0 then
                                            Group:Create( true )

                                            if Group.enable_layout_preview then
                                                Group.Party[ 1 ]:SetVisible( false )
                                            end
                                        elseif not checked and Group.PartyCount == 1 then
                                            Group:Remove()
                                        end

                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                                    end
                                end )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Enable unique player frame pos
                        ui:AddItem(
                            UI:Create( "BOX", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w / 1.5, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end
                                    
                                    ui:SetText( Group:Local( "Options_Layout_Unique_Player_Pos" ) )
                                end )

                                UI:Create( "CHECKBOX", ui, function( ui )
                                    ui:SetSize( 16, 16 )
                                    ui:SetPos( w / 1.5 - 16, 0 )
                                    ui:SetChecked( Group.Settings.enable_unique_player_frame_pos )

                                    function ui:CheckedChanged()
                                        local checked = self:IsChecked()

                                        Group.Settings.enable_unique_player_frame_pos = checked

                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                        Group:RefreshLayout()
                                    end
                                end )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Header
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 16 )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Position
                                ui:AddItem(
                                    UI:Create( "TEXT", ui, function( ui )
                                        ui:SetWidth( w / 1.5 )
                                        ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                        if language == Turbine.Language.German or language == Turbine.Language.French then
                                            ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                        end
                                        
                                        ui:SetText( Group:Local( "Options_Layout_Unique_Player_Position" ) )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, UI.Spacing )
                        
                        ui:AddItem( --Layout
                            UI:Create( "LIST", ui, function( ui )
                                ui:SetSize( w, 32 + UI.Spacing )
                                --ui:SetColor( 100, 100, 100 )
                                ui:SetOrientation( Turbine.UI.Orientation.Horizontal )

                                --Position
                                ui:AddItem(
                                    UI:Create( "LIST", ui, function( ui )
                                        ui:SetSize( w / 1.5, 32 + UI.Spacing )

                                        ui:AddItem(
                                            UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetWidth( w / 1.5 )
                                                ui:SetRange( 0, UI.SCREEN_W )
                                                ui:SetKey( "X" )
                                                ui:SetResetText( Group:Local( "UI_Button_Center" ) )
                                                ui:SetValueSize( 48 )
                                                ui:SetButtonSize( 64 )
                                                ui:SetValue( Group.Settings.Player_Frame_Pos.x )
                                
                                                function ui:ValueChanged( value )
                                                    Group.Settings.Player_Frame_Pos.x = value
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                                    if Group.Settings.enable_unique_player_frame_pos then
                                                        for i = 1, #Group.Party do
                                                            if Group.Party[ i ].localplayer then
                                                                Group.Party[ i ]:SetPos( UI.SCREEN_W - Group.Settings.Pos.x + Group.Settings.Player_Frame_Pos.x - Group.Settings.vitals_x / 2, UI.SCREEN_H - Group.Settings.Pos.y + Group.Settings.Player_Frame_Pos.y - Group.Settings.vitals_y / 2 )
                                            
                                                                break
                                                            end
                                                        end
                                                    end
                                                end
                                
                                                function ui:SliderChanged( value )
                                                    Group.Settings.Player_Frame_Pos.x = value
                                
                                                    if Group.Settings.enable_unique_player_frame_pos then
                                                        for i = 1, #Group.Party do
                                                            if Group.Party[ i ].localplayer then
                                                                Group.Party[ i ]:SetPos( UI.SCREEN_W - Group.Settings.Pos.x + Group.Settings.Player_Frame_Pos.x - Group.Settings.vitals_x / 2, UI.SCREEN_H - Group.Settings.Pos.y + Group.Settings.Player_Frame_Pos.y - Group.Settings.vitals_y / 2 )
                                            
                                                                break
                                                            end
                                                        end
                                                    end
                                                end

                                                function ui:Reset()
                                                    self:SetValue( Group.Default_Settings.Player_Frame_Pos.x )
                                                end
                                            end )
                                        )

                                        ui:AddFiller( 0, UI.Spacing )

                                        ui:AddItem(
                                            UI:Create( "SLIDER_RESET", ui, function( ui )
                                                ui:SetWidth( w / 1.5 )
                                                ui:SetRange( 0, UI.SCREEN_H )
                                                ui:SetKey( "Y" )
                                                ui:SetResetText( Group:Local( "UI_Button_Center" ) )
                                                ui:SetValueSize( 48 )
                                                ui:SetButtonSize( 64 )
                                                ui:SetValue( Group.Settings.Player_Frame_Pos.y )
                                
                                                function ui:ValueChanged( value )
                                                    Group.Settings.Player_Frame_Pos.y = value
                                
                                                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                                    if Group.Settings.enable_unique_player_frame_pos then
                                                        for i = 1, #Group.Party do
                                                            if Group.Party[ i ].localplayer then
                                                                Group.Party[ i ]:SetPos( UI.SCREEN_W - Group.Settings.Pos.x + Group.Settings.Player_Frame_Pos.x - Group.Settings.vitals_x / 2, UI.SCREEN_H - Group.Settings.Pos.y + Group.Settings.Player_Frame_Pos.y - Group.Settings.vitals_y / 2 )
                                            
                                                                break
                                                            end
                                                        end
                                                    end
                                                end
                                
                                                function ui:SliderChanged( value )
                                                    Group.Settings.Player_Frame_Pos.y = value
                                
                                                    if Group.Settings.enable_unique_player_frame_pos then
                                                        for i = 1, #Group.Party do
                                                            if Group.Party[ i ].localplayer then
                                                                Group.Party[ i ]:SetPos( UI.SCREEN_W - Group.Settings.Pos.x + Group.Settings.Player_Frame_Pos.x - Group.Settings.vitals_x / 2, UI.SCREEN_H - Group.Settings.Pos.y + Group.Settings.Player_Frame_Pos.y - Group.Settings.vitals_y / 2 )
                                            
                                                                break
                                                            end
                                                        end
                                                    end
                                                end

                                                function ui:Reset()
                                                    self:SetValue( Group.Default_Settings.Player_Frame_Pos.y )
                                                end
                                            end )
                                        )
                                    end )
                                )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        ui:AddItem( --Divider
                            UI:Create( "DIVIDER", ui, function( ui )
                                ui:SetWidth( w )
                            end )
                        )

                        ui:AddFiller( 0, 16 )

                        --Enable alphabetical sorting
                        ui:AddItem(
                            UI:Create( "BOX", ui, function( ui )
                                ui:SetSize( w / 1.5, 16 )
                                ui:SetColor( 0, 100, 0, 0 )
                                ui:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend )

                                UI:Create( "TEXT", ui, function( ui )
                                    ui:SetSize( w / 1.5, 16 )
                                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                                    if language == Turbine.Language.German or language == Turbine.Language.French then
                                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                                    end
                                    
                                    ui:SetText( Group:Local( "Options_Layout_Alphabetical_Sorting" ) )
                                end )

                                UI:Create( "CHECKBOX", ui, function( ui )
                                    ui:SetSize( 16, 16 )
                                    ui:SetPos( w / 1.5 - 16, 0 )
                                    ui:SetChecked( Group.Settings.enable_alphabetical_sorting )

                                    function ui:CheckedChanged()
                                        local checked = self:IsChecked()

                                        Group.Settings.enable_alphabetical_sorting = checked

                                        Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                                        if checked then --Don't reload?
                                            Turbine.PluginManager.LoadPlugin( "MiniRaidReload" )
                                        end
                                    end
                                end )
                            end )
                        )
                    end
                end )
            end
        end )

        main.header = UI:Create( "BOX", main, function( ui )
            ui:SetSize( 256, 32 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:SetSize( 256, 34 )
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
                ui:SetMouseVisible( false )
            end )

            UI:Create( "TEXT", ui, function( ui )
                ui:SetSize( 256, 32 )
                ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro20 )

                if language == Turbine.Language.German or language == Turbine.Language.French or language == Turbine.Language.French then
                    ui:SetFont( Turbine.UI.Lotro.Font.Verdana20 )
                end
                
                ui:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
                ui:SetText( Group:Local( "Options" ) )
                ui:SetMouseVisible( false )
            end )
        end )

        UI:Create( "BOX", main, function( ui )
            ui:SetPos( 1200 - 32 - 2, 0 )
            ui:SetSize( 32, 32 )
            ui:SetColor( 25, 25, 25, 225 )
            ui:SetMouseVisible( false )

            UI:Create( "BORDER", ui, function( ui )
                ui:SetSize( 32, 34 )
                ui:SetBorderSize( 2 )
                ui:SetBorderColor( Color( 100, 100, 100 ) )
                ui:SetMouseVisible( false )
            end )

            UI:Create( "IBUTTON", ui, function( ui )
                ui:SetSize( 32, 32 )

                ui.icon = UI:Create( "BOX", ui, function( ui )
                    ui:SetPos( 10, 10 )
                    ui:SetColor( 0, 0, 0, 0 )
                    ui:SetMouseVisible( false )
                    ui:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )
                    ui:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )
                    ui:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply )
                    ui:SetSize( 32, 32 )
                    ui:SetStretchMode( 1 )
                    ui:SetSize( 12, 12 )
                end )

                function ui:MouseEnter()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit_hover.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end

                function ui:MouseLeave()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end

                function ui:MouseDown()
                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )
                end
                
                function ui:MouseUp() 
                    if not self:IsHovering() then return end

                    main:SetVisible( false )

                    self.icon:SetBackground( "Darf/MiniRaid/Resources/exit.tga" )

                    self.icon:SetSize( 32, 32 )
                    self.icon:SetStretchMode( 1 )
                    self.icon:SetSize( 12, 12 )

                    Group.Settings.force_option_menu = nil

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )
                end
            end )
        end )

        local tabs = main.tabs

        for i = 1, #tabs do
            UI:Create( "BOX", main, function( ui )
                ui:SetPos( 1200 - 70 * i - 16 * i - 32 - 16 - 2, 0 )
                ui:SetSize( 70, 32 )
                ui:SetColor( 25, 25, 25, 225 )
                
                ui.border = UI:Create( "BORDER", ui, function( ui )
                    ui:SetSize( 70, 32 )
                    ui:SetBorderSize( 2 )
                    ui:SetBorderColor( Color( 100, 100, 100 ) )
                    ui:SetMouseVisible( false )
                end )
    
                UI:Create( "TEXT", ui, function( ui )
                    ui:SetSize( 70, 32 )
                    ui:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )

                    if language == Turbine.Language.German or language == Turbine.Language.French then
                        ui:SetFont( Turbine.UI.Lotro.Font.Verdana16 )
                    end
                    
                    ui:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter )
                    ui:SetText( Group:Local( "Options_Tab_" .. i ) )
                    ui:SetMouseVisible( false )
                end )

                if Group.Settings.force_option_menu == i then
                    ui.selected = true

                    ui.border:SetHeight( 34 )

                    main.tab      = ui
                    main.selected = tabs[ i ]

                    main.window[ tabs[ i ] ]:SetVisible( true )

                    main.selected_tab = i
                elseif not Group.Settings.force_option_menu and i == #tabs then
                    ui.selected = true

                    ui.border:SetHeight( 34 )

                    main.tab      = ui
                    main.selected = tabs[ i ]

                    main.window[ tabs[ i ] ]:SetVisible( true )

                    Group.Settings.force_option_menu = i

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                    main.selected_tab = i
                end

                function ui:MouseEnter()
                    if self.selected then return end

                    self:SetColor( 75, 75, 75, 200 )
                end

                function ui:MouseLeave()
                    self:SetColor( 25, 25, 25, 225 )
                end

                function ui:MouseDown()
                    if self.selected then return end

                    self.selected = true

                    self:SetColor( 25, 25, 25, 225 )
                    self.border:SetHeight( 34 )

                    main.tab.border:SetHeight( 32 )
                    main.tab.selected = false

                    main.window[ main.selected ]:SetVisible( false )
                    
                    main.tab      = self
                    main.selected = tabs[ i ]

                    main.window[ main.selected ]:SetVisible( true )

                    Group.Settings.force_option_menu = i

                    Turbine.PluginData.Save( Turbine.DataScope.Character, "MiniRaid_Settings", table.save( Group.Settings ) )

                    main.selected_tab = i
                end
            end )
        end
    end )
end